package com.chelvc.framework.aliyun;

import java.util.Objects;

import com.aliyun.captcha20230305.models.VerifyIntelligentCaptchaRequest;
import com.aliyun.captcha20230305.models.VerifyIntelligentCaptchaResponse;
import com.aliyun.dypnsapi20170525.models.GetMobileRequest;
import com.aliyun.dypnsapi20170525.models.GetMobileResponse;
import com.aliyun.teaopenapi.models.Config;
import com.aliyun.teautil.models.RuntimeOptions;
import com.chelvc.framework.aliyun.config.AliyunProperties;
import lombok.NonNull;

/**
 * 阿里云操作接口默认实现
 *
 * @author Woody
 * @date 2024/8/12
 */
public class DefaultAliyunHandler implements AliyunHandler {
    private final AliyunProperties properties;
    private final com.aliyun.captcha20230305.Client captcha;
    private final com.aliyun.dypnsapi20170525.Client dypnsapi;

    public DefaultAliyunHandler(@NonNull AliyunProperties properties) {
        this.properties = properties;

        try {
            // 初始化captcha客户端
            Config config = this.getConfig(properties);
            config.endpoint = "captcha.cn-shanghai.aliyuncs.com";
            this.captcha = new com.aliyun.captcha20230305.Client(config);

            // 初始化dypnsapi客户端
            config = this.getConfig(properties);
            config.endpoint = "dypnsapi.aliyuncs.com";
            this.dypnsapi = new com.aliyun.dypnsapi20170525.Client(config);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 获取配置对象
     *
     * @param properties 配置信息
     * @return 配置对象实例
     */
    private Config getConfig(AliyunProperties properties) {
        return new Config().setAccessKeyId(properties.getAppid()).setAccessKeySecret(properties.getSecret());
    }

    @Override
    public String code2mobile(@NonNull String token) {
        RuntimeOptions runtime = new RuntimeOptions();
        GetMobileRequest request = new GetMobileRequest().setAccessToken(token);
        GetMobileResponse response;
        try {
            response = this.dypnsapi.getMobileWithOptions(request, runtime);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
        if (response == null || response.body == null) {
            throw new RuntimeException("Invalid aliyun response");
        }
        if (!Objects.equals(response.body.code, "OK")) {
            throw new RuntimeException(response.body.code + ", " + response.body.message);
        }
        return response.body.getGetMobileResultDTO().getMobile();
    }

    @Override
    public boolean verifyCaptcha(@NonNull String verification) {
        VerifyIntelligentCaptchaRequest request = new VerifyIntelligentCaptchaRequest();
        request.captchaVerifyParam = verification;
        request.sceneId = this.properties.getCaptcha().getScene();
        VerifyIntelligentCaptchaResponse response;
        try {
            response = this.captcha.verifyIntelligentCaptcha(request);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
        if (response == null || response.body == null || response.body.result == null) {
            throw new RuntimeException("Invalid aliyun response");
        }
        if (!Boolean.TRUE.equals(response.body.success)) {
            throw new RuntimeException(response.body.code + ", " + response.body.message);
        }
        if (!Boolean.TRUE.equals(response.body.result.verifyResult)) {
            throw new RuntimeException(response.body.result.verifyCode);
        }
        return true;
    }
}
