package com.chelvc.framework.base.config;

import java.util.List;
import javax.servlet.Filter;

import com.chelvc.framework.base.context.DefaultSessionFactory;
import com.chelvc.framework.base.context.SessionFactory;
import com.chelvc.framework.base.interceptor.ResponseWrapInterceptor;
import com.chelvc.framework.common.util.JacksonUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingBean;
import org.springframework.boot.web.servlet.FilterRegistrationBean;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Primary;
import org.springframework.core.Ordered;
import org.springframework.http.converter.json.Jackson2ObjectMapperBuilder;
import org.springframework.web.cors.CorsConfiguration;
import org.springframework.web.cors.UrlBasedCorsConfigurationSource;
import org.springframework.web.filter.CorsFilter;
import org.springframework.web.method.support.HandlerMethodReturnValueHandler;
import org.springframework.web.servlet.mvc.method.annotation.RequestMappingHandlerAdapter;
import org.springframework.web.servlet.mvc.method.annotation.RequestResponseBodyMethodProcessor;

/**
 * 应用配置
 *
 * @author Woody
 * @date 2024/1/30
 */
@Configuration
public class ApplicationConfigurer {
    @Bean
    @Primary
    public ObjectMapper objectMapper(Jackson2ObjectMapperBuilder builder,
                                     @Value("${spring.jackson.safely:false}") boolean safely) {
        return JacksonUtils.initializeSerializer(builder.build(), safely);
    }

    @Bean
    @ConditionalOnMissingBean(SessionFactory.class)
    public SessionFactory sessionFactory() {
        return new DefaultSessionFactory();
    }

    @Bean
    @ConditionalOnMissingBean(ResponseWrapInterceptor.class)
    public ResponseWrapInterceptor responseWrapInterceptor(RequestMappingHandlerAdapter adapter) {
        List<HandlerMethodReturnValueHandler> handlers =
                ObjectUtils.ifNull(adapter.getReturnValueHandlers(), Lists::newArrayList, Lists::newArrayList);
        HandlerMethodReturnValueHandler processor = handlers.stream()
                .filter(handler -> handler instanceof RequestResponseBodyMethodProcessor).findAny().orElse(null);
        ResponseWrapInterceptor interceptor = new ResponseWrapInterceptor(processor);
        handlers.add(0, interceptor);
        adapter.setReturnValueHandlers(handlers);
        return interceptor;
    }

    @Bean
    public FilterRegistrationBean<Filter> crossDomainAccessRegistration() {
        FilterRegistrationBean<Filter> registration = new FilterRegistrationBean<>();
        CorsConfiguration configuration = new CorsConfiguration();
        configuration.addAllowedOrigin("*");
        configuration.addAllowedHeader("*");
        configuration.addAllowedMethod("*");
        configuration.setAllowCredentials(true);
        UrlBasedCorsConfigurationSource source = new UrlBasedCorsConfigurationSource();
        source.registerCorsConfiguration("/**", configuration);
        registration.setFilter(new CorsFilter(source));
        registration.setOrder(Ordered.HIGHEST_PRECEDENCE);
        return registration;
    }
}
