package com.chelvc.framework.base.config;

import java.util.concurrent.TimeUnit;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import lombok.RequiredArgsConstructor;
import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.impl.client.DefaultConnectionKeepAliveStrategy;
import org.apache.http.impl.client.DefaultHttpRequestRetryHandler;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.impl.conn.PoolingHttpClientConnectionManager;
import org.apache.http.protocol.HttpContext;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.http.client.HttpComponentsClientHttpRequestFactory;
import org.springframework.web.client.RestTemplate;

/**
 * Http请求配置
 *
 * @author Woody
 * @date 2024/1/30
 */
@Configuration
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class HttpClientConfigurer {
    private final HttpClientProperties properties;

    @Bean
    public HttpClient httpClient() {
        HttpClientBuilder builder = HttpClientBuilder.create();
        PoolingHttpClientConnectionManager manager = new PoolingHttpClientConnectionManager();
        manager.setMaxTotal(this.properties.getMaxTotal());
        manager.setDefaultMaxPerRoute(this.properties.getMaxPerRoute());
        builder.setConnectionManager(manager);
        builder.setRetryHandler(new DefaultHttpRequestRetryHandler(this.properties.getRetryCount(), false));
        builder.setKeepAliveStrategy(new DefaultConnectionKeepAliveStrategy() {
            @Override
            public long getKeepAliveDuration(HttpResponse response, HttpContext context) {
                long duration = super.getKeepAliveDuration(response, context);
                return duration < 0 ? properties.getKeepAliveTime() : duration;
            }
        });

        // 设置后台线程剔除失效连接
        builder.evictExpiredConnections();
        builder.evictIdleConnections(this.properties.getMaxIdleTime(), TimeUnit.MILLISECONDS);

        // 禁用Cookie回话保持功能
        builder.disableCookieManagement();
        return builder.build();
    }

    @Bean
    public RestTemplate restTemplate() {
        HttpComponentsClientHttpRequestFactory factory = new HttpComponentsClientHttpRequestFactory(this.httpClient());
        factory.setConnectTimeout(this.properties.getConnectTimeout());
        factory.setReadTimeout(this.properties.getSocketTimeout());
        factory.setConnectionRequestTimeout(this.properties.getConnectionRequestTimeout());
        RestTemplate template = new RestTemplate(factory);
        template.setMessageConverters(ApplicationContextHolder.converters(template.getMessageConverters()));
        return template;
    }
}
