package com.chelvc.framework.base.context;

import java.io.IOException;
import java.io.InputStream;
import java.lang.ref.WeakReference;
import java.nio.charset.StandardCharsets;
import java.time.Duration;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Objects;
import java.util.Properties;
import java.util.Set;
import java.util.function.Consumer;
import java.util.function.Function;
import java.util.function.Predicate;
import java.util.function.Supplier;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import com.chelvc.framework.base.util.SpringUtils;
import com.chelvc.framework.common.model.Paging;
import com.chelvc.framework.common.model.Period;
import com.chelvc.framework.common.model.Range;
import com.chelvc.framework.common.model.Region;
import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.common.util.DateUtils;
import com.chelvc.framework.common.util.JacksonUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.google.common.collect.Sets;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.springframework.aop.framework.AopProxyUtils;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.NoSuchBeanDefinitionException;
import org.springframework.beans.factory.config.YamlPropertiesFactoryBean;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.context.support.GenericApplicationContext;
import org.springframework.core.Ordered;
import org.springframework.core.annotation.AnnotationUtils;
import org.springframework.core.annotation.Order;
import org.springframework.core.env.Environment;
import org.springframework.core.io.Resource;
import org.springframework.core.io.support.PathMatchingResourcePatternResolver;
import org.springframework.core.io.support.ResourcePatternResolver;
import org.springframework.http.MediaType;
import org.springframework.http.converter.HttpMessageConverter;
import org.springframework.http.converter.StringHttpMessageConverter;
import org.springframework.http.converter.cbor.MappingJackson2CborHttpMessageConverter;
import org.springframework.http.converter.json.AbstractJackson2HttpMessageConverter;
import org.springframework.http.converter.json.MappingJackson2HttpMessageConverter;
import org.springframework.http.converter.smile.MappingJackson2SmileHttpMessageConverter;
import org.springframework.http.converter.xml.MappingJackson2XmlHttpMessageConverter;
import org.springframework.stereotype.Component;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RestController;

/**
 * 应用上下文工具类
 *
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
@Component
@Order(Ordered.HIGHEST_PRECEDENCE)
public class ApplicationContextHolder implements ApplicationContextAware, PropertyRefreshListener {
    /**
     * 服务端口属性
     */
    public static final String SERVER_PORT_PROPERTY = "server.port";

    /**
     * 应用名称属性
     */
    public static final String APPLICATION_NAME_PROPERTY = "spring.application.name";

    /**
     * 应用环境属性
     */
    public static final String APPLICATION_PROFILE_PROPERTY = "spring.profiles.active";

    /**
     * bootstrap.yml文件搜索路径
     */
    public static final String BOOTSTRAP_YML_PATH =
            ResourcePatternResolver.CLASSPATH_ALL_URL_PREFIX + "/bootstrap.yml";

    /**
     * bootstrap.properties文件搜索路径
     */
    public static final String BOOTSTRAP_PROPERTIES_PATH =
            ResourcePatternResolver.CLASSPATH_ALL_URL_PREFIX + "/bootstrap.properties";

    /**
     * application.yml文件搜索路径
     */
    public static final String APPLICATION_YML_PATH =
            ResourcePatternResolver.CLASSPATH_ALL_URL_PREFIX + "/application.yml";

    /**
     * application.properties文件搜索路径
     */
    public static final String APPLICATION_PROPERTIES_PATH =
            ResourcePatternResolver.CLASSPATH_ALL_URL_PREFIX + "/application.properties";

    /**
     * 属性名/对象实例映射表
     */
    private static final Map<String, WeakReference<Object>> PROPERTY_OBJECT_MAPPING = Maps.newConcurrentMap();

    /**
     * 应用上下文对象
     */
    private static ApplicationContext APPLICATION_CONTEXT;

    @Override
    public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
        APPLICATION_CONTEXT = applicationContext;
    }

    @Override
    public void refreshProperty(Set<String> properties) {
        if (ObjectUtils.notEmpty(properties)) {
            properties.forEach(PROPERTY_OBJECT_MAPPING::remove);
        }
    }

    /**
     * 获取服务端口
     *
     * @return 服务端口
     */
    public static Integer getPort() {
        return getPort(getApplicationContext());
    }

    /**
     * 根据 *.yml、*.properties文件资源获取服务端口
     *
     * @param resource *.yml、*.properties文件资源对象实例
     * @return 服务端口
     */
    public static Integer getPort(@NonNull Resource resource) {
        Properties properties = getResourceProperties(resource);
        if (properties == null) {
            return null;
        }
        String property = properties.getProperty(SERVER_PORT_PROPERTY);
        return StringUtils.ifEmpty(property, Integer::parseInt);
    }

    /**
     * 获取服务端口
     *
     * @param applicationContext 应用上下文实例
     * @return 服务端口
     */
    public static Integer getPort(@NonNull ApplicationContext applicationContext) {
        String property = applicationContext.getEnvironment().getProperty(SERVER_PORT_PROPERTY);
        return StringUtils.ifEmpty(property, Integer::parseInt);
    }

    /**
     * 获取环境标识
     *
     * @return 环境标识
     */
    public static String getProfile() {
        return getProfile(getApplicationContext());
    }

    /**
     * 根据 *.yml、*.properties文件资源获取环境标识
     *
     * @param resource *.yml、*.properties文件资源对象实例
     * @return 环境标识
     */
    public static String getProfile(@NonNull Resource resource) {
        Properties properties = getResourceProperties(resource);
        return properties == null ? null : properties.getProperty(APPLICATION_PROFILE_PROPERTY);
    }

    /**
     * 获取环境标识
     *
     * @param applicationContext 应用上下文实例
     * @return 环境标识
     */
    public static String getProfile(@NonNull ApplicationContext applicationContext) {
        return applicationContext.getEnvironment().getProperty(APPLICATION_PROFILE_PROPERTY);
    }

    /**
     * 判断是否是生产环境
     *
     * @return true/false
     */
    public static boolean isProduction() {
        return getProperty("environment.production", boolean.class, false);
    }

    /**
     * 获取应用环境对象
     *
     * @return 应用环境对象
     */
    public static Environment getEnvironment() {
        return getEnvironment(true);
    }

    /**
     * 获取应用环境对象
     *
     * @param required 应用环境是否必须
     * @return 应用环境对象
     */
    public static Environment getEnvironment(boolean required) {
        ApplicationContext applicationContext = getApplicationContext(required);
        Environment environment = ObjectUtils.ifNull(applicationContext, ApplicationContext::getEnvironment);
        if (required) {
            return AssertUtils.nonnull(environment, () -> "Application environment has not been initialized");
        }
        return environment;
    }

    /**
     * 获取应用名称
     *
     * @return 应用名称
     */
    public static String getApplicationName() {
        return getApplicationName(getApplicationContext());
    }

    /**
     * 获取应用名称
     *
     * @param applicationContext 应用上下文实例
     * @return 应用名称
     */
    public static String getApplicationName(@NonNull ApplicationContext applicationContext) {
        return applicationContext.getEnvironment().getProperty(APPLICATION_NAME_PROPERTY);
    }

    /**
     * 根据 *.yml、*.properties文件资源获取应用名称
     *
     * @param resource *.yml、*.properties文件资源对象实例
     * @return 应用名称
     */
    public static String getApplicationName(@NonNull Resource resource) {
        Properties properties = getResourceProperties(resource);
        return properties == null ? null : properties.getProperty(APPLICATION_NAME_PROPERTY);
    }

    /**
     * 获取应用上下文对象
     *
     * @return 应用上下文对象
     */
    public static ApplicationContext getApplicationContext() {
        return getApplicationContext(true);
    }

    /**
     * 获取应用上下文对象
     *
     * @param required 应用上下文是否必须
     * @return 应用上下文对象
     */
    public static ApplicationContext getApplicationContext(boolean required) {
        if (required) {
            return AssertUtils.nonnull(APPLICATION_CONTEXT, () -> "Application context has not been initialized");
        }
        return APPLICATION_CONTEXT;
    }

    /**
     * 获取所有application.yml文件资源
     *
     * @return 资源对象数组
     */
    public static List<Resource> getApplicationResources() {
        List<Resource> resources = Lists.newLinkedList();
        ResourcePatternResolver resourcePatternResolver = new PathMatchingResourcePatternResolver();

        // bootstrap.yml配置文件
        try {
            resources.addAll(Arrays.asList(resourcePatternResolver.getResources(BOOTSTRAP_YML_PATH)));
        } catch (IOException e) {
            throw new RuntimeException(e);
        }

        // 查找application.yml配置文件
        try {
            resources.addAll(Arrays.asList(resourcePatternResolver.getResources(APPLICATION_YML_PATH)));
        } catch (IOException e) {
            throw new RuntimeException(e);
        }

        // bootstrap.properties配置文件
        try {
            resources.addAll(Arrays.asList(resourcePatternResolver.getResources(BOOTSTRAP_PROPERTIES_PATH)));
        } catch (IOException e) {
            throw new RuntimeException(e);
        }

        // 查找application.properties配置文件
        try {
            resources.addAll(Arrays.asList(resourcePatternResolver.getResources(APPLICATION_PROPERTIES_PATH)));
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        return resources;
    }

    /**
     * 判断类对象是否属于指定资源
     *
     * @param resource 资源对象
     * @param clazz    类对象
     * @return true/false
     */
    public static boolean isResourceClass(@NonNull Resource resource, @NonNull Class<?> clazz) {
        String file = clazz.getProtectionDomain().getCodeSource().getLocation().getPath();
        try {
            return resource.getURL().getPath().contains(file);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 获取资源文件 .yml、.properties属性
     *
     * @param resource *.yml、*.properties文件资源对象实例
     * @return 应用名称
     */
    public static Properties getResourceProperties(@NonNull Resource resource) {
        String filename = resource.getFilename();
        if (StringUtils.isEmpty(filename) || filename.endsWith(".yml")) {
            // yml
            YamlPropertiesFactoryBean yaml = new YamlPropertiesFactoryBean();
            yaml.setResources(resource);
            return yaml.getObject();
        }
        // properties
        Properties properties = new Properties();
        try (InputStream input = resource.getInputStream()) {
            properties.load(input);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        return properties;
    }

    /**
     * 查找接口对象实例
     *
     * @return 接口对象实例列表
     */
    public static List<Object> lookupControllers() {
        return lookupControllers(getApplicationContext());
    }

    /**
     * 查找接口对象实例
     *
     * @param applicationContext 应用上下文
     * @return 接口对象实例列表
     */
    public static List<Object> lookupControllers(@NonNull ApplicationContext applicationContext) {
        return Stream.of(applicationContext.getBeansWithAnnotation(Controller.class).values(),
                applicationContext.getBeansWithAnnotation(RestController.class).values())
                .flatMap(Collection::stream).distinct().collect(Collectors.toList());
    }

    /**
     * 查找类对象所属资源
     *
     * @param clazz     类对象
     * @param resources 资源对象集合
     * @return 资源对象
     */
    public static Resource lookupClassResource(@NonNull Class<?> clazz, @NonNull Collection<Resource> resources) {
        if (ObjectUtils.isEmpty(resources)) {
            return null;
        }
        return resources.stream().filter(resource -> isResourceClass(resource, clazz)).findAny().orElse(null);
    }

    /**
     * 查找应用中的类对象
     *
     * @param condition 类对象条件判断
     * @return 类对象列表
     */
    public static List<Class<?>> lookupApplicationClasses(@NonNull Predicate<Class<?>> condition) {
        return lookupApplicationClasses(getApplicationContext(), condition);
    }

    /**
     * 查找应用中的类对象
     *
     * @param applicationContext 应用上下文对象
     * @param condition          类对象条件判断
     * @return 类对象列表
     */
    public static List<Class<?>> lookupApplicationClasses(@NonNull ApplicationContext applicationContext,
                                                          @NonNull Predicate<Class<?>> condition) {
        Set<String> packages = Sets.newHashSet();
        applicationContext.getBeansWithAnnotation(SpringBootApplication.class).forEach((key, value) -> {
            Class<?> clazz = value.getClass();

            // 加载@ComponentScan包扫描路径配置
            ComponentScan componentScan = AnnotationUtils.findAnnotation(clazz, ComponentScan.class);
            if (componentScan != null) {
                packages.addAll(Arrays.asList(componentScan.basePackages()));
            }

            // 加载@SpringBootApplication包扫描路径配置
            SpringBootApplication springBootApplication =
                    AnnotationUtils.findAnnotation(clazz, SpringBootApplication.class);
            if (springBootApplication != null) {
                packages.addAll(Arrays.asList(springBootApplication.scanBasePackages()));
            }
        });

        // 根据包扫描路径查找类对象
        return packages.stream().flatMap(pack -> SpringUtils.lookupClasses(pack, condition).stream())
                .distinct().collect(Collectors.toList());
    }

    /**
     * 注册对象实例
     *
     * @param name     对象名称
     * @param type     对象类型
     * @param supplier 对象实例提供函数
     * @param <T>      对象类型泛型
     * @return 对象实例
     */
    public static <T> T register(@NonNull String name, @NonNull Class<T> type, @NonNull Supplier<T> supplier) {
        ApplicationContext applicationContext = getApplicationContext();
        ((GenericApplicationContext) getApplicationContext()).registerBean(name, type, supplier);
        return applicationContext.getBean(name, type);
    }

    /**
     * 自定义消息转换处理器
     *
     * @param converters 消息转换处理器列表
     * @return 消息转换处理器列表
     */
    public static List<HttpMessageConverter<?>> converters(List<HttpMessageConverter<?>> converters) {
        if (ObjectUtils.isEmpty(converters)) {
            return Collections.emptyList();
        }
        return converters.stream().map(converter -> {
            Class<?> clazz = converter.getClass();
            if (clazz == StringHttpMessageConverter.class) {
                converter = new StringHttpMessageConverter(StandardCharsets.UTF_8);
            } else if (converter instanceof AbstractJackson2HttpMessageConverter) {
                ObjectMapper mapper = JacksonUtils.getDefaultSerializer();
                List<MediaType> types = Lists.newArrayList(converter.getSupportedMediaTypes());
                if (!types.contains(MediaType.TEXT_PLAIN)) {
                    types.add(MediaType.TEXT_PLAIN);
                }
                if (clazz == MappingJackson2HttpMessageConverter.class) {
                    converter = new MappingJackson2HttpMessageConverter(mapper);
                } else if (clazz == MappingJackson2XmlHttpMessageConverter.class) {
                    converter = new MappingJackson2XmlHttpMessageConverter(mapper);
                } else if (clazz == MappingJackson2CborHttpMessageConverter.class) {
                    converter = new MappingJackson2CborHttpMessageConverter(mapper);
                } else if (clazz == MappingJackson2SmileHttpMessageConverter.class) {
                    converter = new MappingJackson2SmileHttpMessageConverter(mapper);
                }
                ((AbstractJackson2HttpMessageConverter) converter).setSupportedMediaTypes(types);
            }
            return converter;
        }).collect(Collectors.toList());
    }

    /**
     * 获取对象实例
     *
     * @param type 对象类型
     * @param <T>  对象类型泛型
     * @return 对象实例
     */
    public static <T> T getBean(@NonNull Class<T> type) {
        return getBean(type, true);
    }

    /**
     * 获取对象实例
     *
     * @param applicationContext 应用上下文
     * @param type               对象类型
     * @param <T>                对象类型泛型
     * @return 对象实例
     */
    public static <T> T getBean(@NonNull ApplicationContext applicationContext, @NonNull Class<T> type) {
        return getBean(applicationContext, type, true);
    }

    /**
     * 获取对象实例
     *
     * @param type     对象类型
     * @param required 对象实例是否必须存在
     * @param <T>      对象类型泛型
     * @return 对象实例
     */
    public static <T> T getBean(@NonNull Class<T> type, boolean required) {
        ApplicationContext applicationContext = getApplicationContext(required);
        return applicationContext == null ? null : getBean(applicationContext, type, required);
    }

    /**
     * 获取对象实例
     *
     * @param applicationContext 应用上下文
     * @param type               对象类型
     * @param required           对象实例是否必须存在
     * @param <T>                对象类型泛型
     * @return 对象实例
     */
    public static <T> T getBean(@NonNull ApplicationContext applicationContext, @NonNull Class<T> type,
                                boolean required) {
        try {
            return applicationContext.getBean(type);
        } catch (NoSuchBeanDefinitionException e) {
            if (required) {
                throw e;
            }
            return null;
        }
    }

    /**
     * 获取对象实例
     *
     * @param name 对象名称
     * @param <T>  对象类型泛型
     * @return 对象实例
     */
    public static <T> T getBean(@NonNull String name) {
        return getBean(name, true);
    }

    /**
     * 获取对象实例
     *
     * @param applicationContext 应用上下文
     * @param name               对象名称
     * @param <T>                对象类型泛型
     * @return 对象实例
     */
    public static <T> T getBean(@NonNull ApplicationContext applicationContext, @NonNull String name) {
        return getBean(applicationContext, name, true);
    }

    /**
     * 获取对象实例
     *
     * @param name     对象名称
     * @param required 对象实例是否必须存在
     * @param <T>      对象类型泛型
     * @return 对象实例
     */
    public static <T> T getBean(@NonNull String name, boolean required) {
        ApplicationContext applicationContext = getApplicationContext(required);
        return applicationContext == null ? null : getBean(applicationContext, name, required);
    }

    /**
     * 获取对象实例
     *
     * @param applicationContext 应用上下文
     * @param name               对象名称
     * @param required           对象实例是否必须存在
     * @param <T>                对象类型泛型
     * @return 对象实例
     */
    @SuppressWarnings("unchecked")
    public static <T> T getBean(@NonNull ApplicationContext applicationContext, @NonNull String name,
                                boolean required) {
        try {
            return (T) applicationContext.getBean(name);
        } catch (NoSuchBeanDefinitionException e) {
            if (required) {
                throw e;
            }
            return null;
        }
    }

    /**
     * 获取对象名称/实例映射表
     *
     * @param type 对象类型
     * @param <T>  对象类型
     * @return 对象名称/实例映射表
     */
    public static <T> Map<String, T> getBeans(@NonNull Class<T> type) {
        return getBeans(getApplicationContext(), type);
    }

    /**
     * 获取对象名称/实例映射表
     *
     * @param applicationContext 应用上下文
     * @param type               对象类型
     * @param <T>                对象类型
     * @return 对象名称/实例映射表
     */
    public static <T> Map<String, T> getBeans(@NonNull ApplicationContext applicationContext, @NonNull Class<T> type) {
        return applicationContext.getBeansOfType(type);
    }

    /**
     * 根据@Order注解获取排序Bean实例
     *
     * @param type 对象类型
     * @param <T>  对象类型
     * @return Bean对象实例列表
     */
    public static <T> List<T> getOrderBeans(@NonNull Class<? extends T> type) {
        return getOrderBeans(getApplicationContext(), type);
    }

    /**
     * 根据@Order注解获取排序Bean实例
     *
     * @param applicationContext 应用上下文
     * @param type               对象类型
     * @param <T>                对象类型
     * @return Bean对象实例列表
     */
    public static <T> List<T> getOrderBeans(@NonNull ApplicationContext applicationContext,
                                            @NonNull Class<? extends T> type) {
        Map<String, ? extends T> beans = getBeans(applicationContext, type);
        return ObjectUtils.isEmpty(beans) ? Collections.emptyList() : order(beans.values());
    }

    /**
     * 基于@Order注解对类对象排序
     *
     * @param classes 类对象列表
     */
    public static void order(List<Class<?>> classes) {
        if (ObjectUtils.notEmpty(classes)) {
            classes.sort(Comparator.nullsFirst(Comparator.comparingInt(clazz -> ObjectUtils.ifNull(
                    clazz.getAnnotation(Order.class), Order::value, () -> 0
            ))));
        }
    }

    /**
     * 基于@Order注解对Bean排序
     *
     * @param beans Bean对象实例集合
     * @param <T>   对象类型
     * @return 排序后对象实例列表
     */
    public static <T> List<T> order(Collection<? extends T> beans) {
        if (ObjectUtils.isEmpty(beans)) {
            return Collections.emptyList();
        }
        List<T> orders = Lists.newArrayListWithCapacity(beans.size());
        order(beans, orders::add);
        return orders;
    }

    /**
     * 基于@Order注解对Bean排序
     *
     * @param beans    Bean对象实例集合
     * @param consumer Bean对象实例回调函数
     * @param <T>      对象类型
     */
    public static <T> void order(Collection<? extends T> beans, @NonNull Consumer<T> consumer) {
        if (ObjectUtils.isEmpty(beans)) {
            return;
        }
        beans.stream().sorted(Comparator.nullsFirst(Comparator.comparingInt(bean -> {
            if (bean instanceof Ordered) {
                return ((Ordered) bean).getOrder();
            }
            Order order = AopProxyUtils.ultimateTargetClass(bean).getAnnotation(Order.class);
            return order == null ? 0 : order.value();
        }))).forEach(consumer);
    }

    /**
     * 获取属性值
     *
     * @param key 属性名
     * @return 属性值
     */
    public static String getProperty(@NonNull String key) {
        return getProperty(key, String.class, null);
    }

    /**
     * 获取属性值
     *
     * @param key          属性名
     * @param defaultValue 默认值
     * @return 属性值
     */
    public static String getProperty(@NonNull String key, String defaultValue) {
        return getProperty(key, String.class, defaultValue);
    }

    /**
     * 获取属性值
     *
     * @param key  属性名
     * @param type 值类型
     * @param <T>  数据类型
     * @return 属性值
     */
    public static <T> T getProperty(@NonNull String key, @NonNull Class<T> type) {
        return getProperty(key, type, null);
    }

    /**
     * 获取属性值
     *
     * @param key          属性名
     * @param type         值类型
     * @param defaultValue 默认值
     * @param <T>          数据类型
     * @return 属性值
     */
    @SuppressWarnings("unchecked")
    public static <T> T getProperty(@NonNull String key, @NonNull Class<T> type, T defaultValue) {
        T value = null;
        if (type == String.class || ObjectUtils.isMetaClass(type) || type.isEnum() || type.isArray()
                || Collection.class.isAssignableFrom(type)) {
            Environment environment = getEnvironment(false);
            if (environment != null) {
                try {
                    if (type == String.class) {
                        value = (T) environment.getProperty(key);
                    } else {
                        value = environment.getProperty(key, type);
                    }
                } catch (Exception e) {
                    log.error("Get property value failed: {}", key, e);
                }
            }
        } else if (type == Date.class) {
            value = (T) getProperty(key, DateUtils::parse);
        } else if (type == Range.class) {
            value = (T) getProperty(key, Range::parse);
        } else if (type == Paging.class) {
            value = (T) getProperty(key, Paging::parse);
        } else if (type == Period.class) {
            value = (T) getProperty(key, Period::parse);
        } else if (type == Duration.class) {
            value = (T) getProperty(key, Duration::parse);
        } else if (type == Region.class) {
            value = (T) getProperty(key, code -> Region.of(Integer.valueOf(code)));
        } else {
            value = getProperty(key, json -> JacksonUtils.deserialize(json, type));
        }
        return ObjectUtils.ifNull(value, defaultValue);
    }

    /**
     * 获取属性值
     *
     * @param key      属性名
     * @param function 属性值转换回调函数
     * @param <T>      数据类型
     * @return 属性值
     */
    @SuppressWarnings("unchecked")
    public static <T> T getProperty(@NonNull String key, @NonNull Function<String, T> function) {
        Object value = ObjectUtils.ifNull(PROPERTY_OBJECT_MAPPING.get(key), WeakReference::get);
        if (value == null) {
            Environment environment = getEnvironment(false);
            if (environment != null) {
                try {
                    value = function.apply(environment.getProperty(key));
                } catch (Exception e) {
                    log.error("Get property value failed: {}", key, e);
                }
            }
            value = ObjectUtils.ifNull(value, ObjectUtils.EMPTY);
            PROPERTY_OBJECT_MAPPING.put(key, new WeakReference<>(value));
        }
        return value == ObjectUtils.EMPTY ? null : (T) value;
    }

    /**
     * 获取属性值
     *
     * @param key  属性名
     * @param type 值类型
     * @param <T>  数据类型
     * @return 属性值
     */
    public static <T> T getProperty(@NonNull String key, @NonNull TypeReference<T> type) {
        return getProperty(key, type, null);
    }

    /**
     * 获取属性值
     *
     * @param key          属性名
     * @param type         值类型
     * @param defaultValue 默认值
     * @param <T>          数据类型
     * @return 属性值
     */
    public static <T> T getProperty(@NonNull String key, @NonNull TypeReference<T> type, T defaultValue) {
        T value = getProperty(key, json -> JacksonUtils.deserialize(json, type));
        return ObjectUtils.ifNull(value, defaultValue);
    }

    /**
     * 获取国际化消息
     *
     * @param code 消息标识
     * @param args 消息参数
     * @return 消息内容
     */
    public static String getMessage(@NonNull String code, @NonNull Object... args) {
        return getMessage(code, code, args);
    }

    /**
     * 获取国际化消息
     *
     * @param code     消息标识
     * @param defaults 默认消息
     * @param args     消息参数
     * @return 消息内容
     */
    public static String getMessage(@NonNull String code, String defaults, @NonNull Object... args) {
        return getMessage(LocaleContextHolder.getLocale(), code, defaults, args);
    }

    /**
     * 获取国际化消息
     *
     * @param locale 本地信息
     * @param code   消息标识
     * @param args   消息参数
     * @return 消息内容
     */
    public static String getMessage(@NonNull Locale locale, @NonNull String code, @NonNull Object... args) {
        return getMessage(locale, code, code, args);
    }

    /**
     * 获取国际化消息
     *
     * @param locale   本地信息
     * @param code     消息标识
     * @param defaults 默认消息
     * @param args     消息参数
     * @return 消息内容
     */
    public static String getMessage(@NonNull Locale locale, @NonNull String code, String defaults,
                                    @NonNull Object... args) {
        if (Objects.isNull(defaults)) {
            defaults = code;
        }
        ApplicationContext applicationContext = getApplicationContext(false);
        if (applicationContext != null) {
            try {
                return applicationContext.getMessage(code, args, defaults, locale);
            } catch (Exception e) {
                log.warn("Application message get failed [{}]: {}, {}", locale, code, e.getMessage());
            }
        }
        return defaults;
    }
}
