package com.chelvc.framework.base.context;

import javax.servlet.http.HttpServletRequest;

import ch.qos.logback.classic.Level;
import com.chelvc.framework.common.model.Platform;
import com.chelvc.framework.common.model.Terminal;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import lombok.NonNull;
import org.slf4j.Logger;

/**
 * 日志上下文工具类
 *
 * @author Woody
 * @date 2024/1/30
 */
public final class LoggingContextHolder {
    /**
     * 日志级别配置前缀
     */
    public static final String LOGGING_LEVEL_PREFIX = "logging.level.";

    private LoggingContextHolder() {
    }

    /**
     * 获取日志信息
     *
     * @param t 异常对象实例
     * @return 消息内容
     */
    public static String message(Throwable t) {
        return message(SessionContextHolder.getRequest(), t);
    }

    /**
     * 获取日志信息
     *
     * @param messages 消息内容数组
     * @return 消息内容
     */
    public static String message(@NonNull Object... messages) {
        return message(SessionContextHolder.getRequest(), messages);
    }

    /**
     * 获取日志信息
     *
     * @param request Http请求对象
     * @param t       异常对象实例
     * @return 消息内容
     */
    public static String message(HttpServletRequest request, Throwable t) {
        return message(request, ObjectUtils.ifNull(t, Throwable::getMessage));
    }

    /**
     * 获取日志信息
     *
     * @param request  Http请求对象
     * @param messages 消息内容数组
     * @return 消息内容
     */
    public static String message(HttpServletRequest request, @NonNull Object... messages) {
        String uri = ObjectUtils.ifNull(request, HttpServletRequest::getRequestURI);
        String method = ObjectUtils.ifNull(request, HttpServletRequest::getMethod);
        return message(uri, method, messages);
    }

    /**
     * 获取日志信息
     *
     * @param uri      资源地址
     * @param method   请求方式
     * @param messages 消息内容数组
     * @return 消息内容
     */
    public static String message(String uri, String method, @NonNull Object... messages) {
        Session session = SessionContextHolder.getSession(false);
        Long id = ObjectUtils.ifNull(session, Session::getId);
        String host = ObjectUtils.ifNull(session, Session::getHost);
        String device = ObjectUtils.ifNull(session, Session::getDevice);
        Platform platform = ObjectUtils.ifNull(session, Session::getPlatform);
        Terminal terminal = ObjectUtils.ifNull(session, Session::getTerminal);
        String version = ObjectUtils.ifNull(session, Session::getVersion);
        Long timestamp = ObjectUtils.ifNull(session, Session::getTimestamp);
        StringBuilder buffer = new StringBuilder("[");
        if (timestamp != null) {
            buffer.append(timestamp);
        }
        buffer.append("] [");
        if (StringUtils.notEmpty(host)) {
            buffer.append(host);
        }
        buffer.append("] [");
        if (StringUtils.notEmpty(device)) {
            buffer.append(device);
        }
        buffer.append("] [");
        if (platform != null) {
            buffer.append(platform);
        }
        buffer.append("] [");
        if (terminal != null) {
            buffer.append(terminal);
        }
        buffer.append("] [");
        if (StringUtils.notEmpty(version)) {
            buffer.append(version);
        }
        buffer.append("] [");
        if (id != null) {
            buffer.append(id);
        }
        buffer.append("] [");
        if (StringUtils.notEmpty(method)) {
            buffer.append(method).append(" ");
        }
        if (StringUtils.notEmpty(uri)) {
            buffer.append(uri);
        }
        buffer.append("]:");
        for (Object message : messages) {
            if (StringUtils.notEmpty(message)) {
                buffer.append(" ").append(message);
            }
        }
        return buffer.toString();
    }

    /**
     * 获取日志级别
     *
     * @param target 目标对象名称
     * @return 日志级别
     */
    public static Level level(@NonNull String target) {
        String property = LOGGING_LEVEL_PREFIX + target;
        String level = ApplicationContextHolder.getProperty(property);
        return StringUtils.ifEmpty(level, Level::toLevel);
    }

    /**
     * 获取日志级别
     *
     * @param target 目标对象
     * @return 日志级别
     */
    public static Level level(@NonNull Class<?> target) {
        return level(target.getName());
    }

    /**
     * 打印正常日志
     *
     * @param logger   日志处理器
     * @param messages 消息数组
     */
    public static void info(@NonNull Logger logger, @NonNull Object... messages) {
        logger.info(message(messages));
    }

    /**
     * 打印正常日志
     *
     * @param logger   日志处理器
     * @param request  Http请求对象
     * @param messages 消息数组
     */
    public static void info(@NonNull Logger logger, @NonNull HttpServletRequest request, @NonNull Object... messages) {
        logger.info(message(request, messages));
    }

    /**
     * 打印告警日志
     *
     * @param logger 日志处理器
     * @param t      异常对象实例
     */
    public static void warn(@NonNull Logger logger, @NonNull Throwable t) {
        warn(logger, t, false);
    }

    /**
     * 打印告警日志
     *
     * @param logger  日志处理器
     * @param t       异常对象实例
     * @param tracing 是否打印堆栈
     */
    public static void warn(@NonNull Logger logger, @NonNull Throwable t, boolean tracing) {
        if (tracing) {
            logger.warn(message(t), t);
        } else {
            logger.warn(message(t));
        }
    }

    /**
     * 打印告警日志
     *
     * @param logger   日志处理器
     * @param messages 消息数组
     */
    public static void warn(@NonNull Logger logger, @NonNull Object... messages) {
        logger.warn(message(messages));
    }

    /**
     * 打印告警日志
     *
     * @param logger  日志处理器
     * @param request Http请求对象
     * @param t       异常对象实例
     */
    public static void warn(@NonNull Logger logger, @NonNull HttpServletRequest request, @NonNull Throwable t) {
        warn(logger, request, t, false);
    }

    /**
     * 打印告警日志
     *
     * @param logger  日志处理器
     * @param request Http请求对象
     * @param t       异常对象实例
     * @param tracing 是否打印堆栈
     */
    public static void warn(@NonNull Logger logger, @NonNull HttpServletRequest request, @NonNull Throwable t,
                            boolean tracing) {
        if (tracing) {
            logger.warn(message(request, t), t);
        } else {
            logger.warn(message(request, t));
        }
    }

    /**
     * 打印告警日志
     *
     * @param logger   日志处理器
     * @param request  Http请求对象
     * @param messages 消息数组
     */
    public static void warn(@NonNull Logger logger, @NonNull HttpServletRequest request, @NonNull Object... messages) {
        logger.warn(message(request, messages));
    }

    /**
     * 打印错误日志
     *
     * @param logger 日志处理器
     * @param t      异常对象实例
     */
    public static void error(@NonNull Logger logger, @NonNull Throwable t) {
        logger.error(message(t), t);
    }

    /**
     * 打印错误日志
     *
     * @param logger   日志处理器
     * @param messages 消息数组
     */
    public static void error(@NonNull Logger logger, @NonNull Object... messages) {
        logger.error(message(messages));
    }

    /**
     * 打印错误日志
     *
     * @param logger  日志处理器
     * @param request Http请求对象
     * @param t       异常对象实例
     */
    public static void error(@NonNull Logger logger, @NonNull HttpServletRequest request, @NonNull Throwable t) {
        logger.error(message(request, t), t);
    }

    /**
     * 打印错误日志
     *
     * @param logger   日志处理器
     * @param request  Http请求对象
     * @param messages 消息数组
     */
    public static void error(@NonNull Logger logger, @NonNull HttpServletRequest request, @NonNull Object... messages) {
        logger.error(message(request, messages));
    }

    /**
     * 打印调试日志
     *
     * @param logger 日志处理器
     * @param t      异常对象实例
     */
    public static void debug(@NonNull Logger logger, @NonNull Throwable t) {
        if (logger.isDebugEnabled()) {
            logger.debug(message(t), t);
        }
    }

    /**
     * 打印调试日志
     *
     * @param logger   日志处理器
     * @param messages 消息数组
     */
    public static void debug(@NonNull Logger logger, @NonNull Object... messages) {
        if (logger.isDebugEnabled()) {
            logger.debug(message(messages));
        }
    }

    /**
     * 打印调试日志
     *
     * @param logger  日志处理器
     * @param request Http请求对象
     * @param t       异常对象实例
     */
    public static void debug(@NonNull Logger logger, @NonNull HttpServletRequest request, @NonNull Throwable t) {
        if (logger.isDebugEnabled()) {
            logger.debug(message(request, t), t);
        }
    }

    /**
     * 打印调试日志
     *
     * @param logger   日志处理器
     * @param request  Http请求对象
     * @param messages 消息数组
     */
    public static void debug(@NonNull Logger logger, @NonNull HttpServletRequest request, @NonNull Object... messages) {
        if (logger.isDebugEnabled()) {
            logger.debug(message(request, messages));
        }
    }
}
