package com.chelvc.framework.base.context;

import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayDeque;
import java.util.Collections;
import java.util.Deque;
import java.util.Objects;
import java.util.Set;
import java.util.function.Function;
import javax.servlet.ServletRequestEvent;
import javax.servlet.ServletRequestListener;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.chelvc.framework.common.model.Compare;
import com.chelvc.framework.common.model.Platform;
import com.chelvc.framework.common.model.Terminal;
import com.chelvc.framework.common.model.Version;
import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.fasterxml.jackson.core.JsonEncoding;
import lombok.NonNull;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.env.Environment;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

/**
 * 会话上下文工具类
 *
 * @author Woody
 * @date 2024/1/30
 */
@Component
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class SessionContextHolder implements ServletRequestListener {
    /**
     * 身份标识请求头
     */
    public static final String HEADER_ID = "id";

    /**
     * 使用类别请求头
     */
    public static final String HEADER_USING = "using";

    /**
     * 应用范围请求头
     */
    public static final String HEADER_SCOPE = "scope";

    /**
     * 手机号码请求头
     */
    public static final String HEADER_MOBILE = "mobile";

    /**
     * 设备标识请求头
     */
    public static final String HEADER_DEVICE = "device";

    /**
     * 渠道来源请求头
     */
    public static final String HEADER_CHANNEL = "channel";

    /**
     * 应用平台请求头
     */
    public static final String HEADER_PLATFORM = "platform";

    /**
     * 终端编号请求头
     */
    public static final String HEADER_TERMINAL = "terminal";

    /**
     * 终端版本号请求头
     */
    public static final String HEADER_VERSION = "version";

    /**
     * 请求时间戳请求头
     */
    public static final String HEADER_TIMESTAMP = "timestamp";

    /**
     * 注册时间戳请求头
     */
    public static final String HEADER_REGISTERING = "registering";

    /**
     * 权限标识请求头
     */
    public static final String HEADER_AUTHORITIES = "authorities";

    /**
     * 空会话对象实例
     */
    private static final Session EMPTY_SESSION = new Session();

    /**
     * 用户会话上下文
     */
    private static final ThreadLocal<Deque<Session>> SESSION_CONTEXT = ThreadLocal.withInitial(ArrayDeque::new);

    private final SessionFactory sessionFactory;

    /**
     * 获取会话信息
     *
     * @return 会话信息
     */
    public static Session getSession() {
        return getSession(true);
    }

    /**
     * 获取会话信息
     *
     * @param required 是否必须
     * @return 会话信息
     */
    public static Session getSession(boolean required) {
        Session session = SESSION_CONTEXT.get().peek();
        if (session == EMPTY_SESSION) {
            session = null;
        }
        if (required) {
            return AssertUtils.nonnull(session, () -> "Session has not been initialized");
        }
        return session;
    }

    /**
     * 设置会话信息
     *
     * @param session 会话信息
     */
    public static void setSession(Session session) {
        SESSION_CONTEXT.get().push(ObjectUtils.ifNull(session, EMPTY_SESSION));
    }

    /**
     * 设置会话信息，如果当前已存在会话则更新会话主体信息
     *
     * @param id    主体标识
     * @param scope 应用范围
     * @return 会话信息
     */
    public static Session setSession(@NonNull Long id, @NonNull String scope) {
        return setSession(id, scope, Collections.emptySet());
    }

    /**
     * 设置会话信息，如果当前已存在会话则更新会话主体信息
     *
     * @param id          主体标识
     * @param scope       应用范围
     * @param authorities 权限标识集合
     * @return 会话信息
     */
    public static Session setSession(@NonNull Long id, @NonNull String scope, @NonNull Set<String> authorities) {
        return setSession(id, Using.NORMAL, scope, null, null, authorities);
    }

    /**
     * 设置会话信息，如果当前已存在会话则更新会话主体信息
     *
     * @param id          主体标识
     * @param using       使用类别
     * @param scope       应用范围
     * @param mobile      手机号码
     * @param registering 注册时间戳
     * @param authorities 权限标识集合
     * @return 会话信息
     */
    public static Session setSession(@NonNull Long id, @NonNull Using using, @NonNull String scope, String mobile,
                                     Long registering, @NonNull Set<String> authorities) {
        Deque<Session> deque = SESSION_CONTEXT.get();
        Session session = deque.peek();
        if (session == null || session == EMPTY_SESSION) {
            if (session != null) {
                deque.poll();
            }
            session = Session.builder().id(id).using(using).scope(scope).mobile(mobile).registering(registering)
                    .authorities(Collections.unmodifiableSet(authorities))
                    .timestamp(System.currentTimeMillis()).build();
            deque.push(session);
        } else {
            session.initializePrincipal(id, using, scope, mobile, registering, authorities);
        }
        return session;
    }

    /**
     * 获取用户ID
     *
     * @return 用户ID
     */
    public static Long getId() {
        return ObjectUtils.ifNull(getSession(false), Session::getId);
    }

    /**
     * 获取使用类别
     *
     * @return 使用类别
     */
    public static Using getUsing() {
        return ObjectUtils.ifNull(getSession(false), Session::getUsing);
    }

    /**
     * 获取请求地址
     *
     * @return 请求地址
     */
    public static String getHost() {
        return ObjectUtils.ifNull(getSession(false), Session::getHost);
    }

    /**
     * 获取应用范围
     *
     * @return 应用范围
     */
    public static String getScope() {
        return ObjectUtils.ifNull(getSession(false), Session::getScope);
    }

    /**
     * 获取手机号码
     *
     * @return 手机号码
     */
    public static String getMobile() {
        return ObjectUtils.ifNull(getSession(false), Session::getMobile);
    }

    /**
     * 获取设备标识
     *
     * @return 设备标识
     */
    public static String getDevice() {
        return ObjectUtils.ifNull(getSession(false), Session::getDevice);
    }

    /**
     * 获取渠道来源
     *
     * @return 渠道来源
     */
    public static String getChannel() {
        return ObjectUtils.ifNull(getSession(false), Session::getChannel);
    }

    /**
     * 获取平台信息
     *
     * @return 平台信息
     */
    public static Platform getPlatform() {
        return ObjectUtils.ifNull(getSession(false), Session::getPlatform);
    }

    /**
     * 获取终端信息
     *
     * @return 终端信息
     */
    public static Terminal getTerminal() {
        return ObjectUtils.ifNull(getSession(false), Session::getTerminal);
    }

    /**
     * 获取版本信息
     *
     * @return 版本信息
     */
    public static String getVersion() {
        return ObjectUtils.ifNull(getSession(false), Session::getVersion);
    }

    /**
     * 获取请求时间戳
     *
     * @return 时间戳
     */
    public static Long getTimestamp() {
        return ObjectUtils.ifNull(getSession(false), Session::getTimestamp);
    }

    /**
     * 获取注册时间戳
     *
     * @return 时间戳
     */
    public static Long getRegistering() {
        return ObjectUtils.ifNull(getSession(false), Session::getRegistering);
    }

    /**
     * 判断是否是审核用户
     *
     * @return true/false
     */
    public static boolean isAuditor() {
        String mobile = getMobile();
        if (StringUtils.isEmpty(mobile)) {
            return false;
        }
        Set<?> mobiles = ApplicationContextHolder.getProperty("auditor.mobile", Set.class);
        return ObjectUtils.notEmpty(mobiles) && mobiles.contains(mobile);
    }

    /**
     * 判断是否是匿名用户
     *
     * @return true/false
     */
    public static boolean isAnonymous() {
        return StringUtils.isEmpty(getMobile());
    }

    /**
     * 判断是否是指定应用范围
     *
     * @param scopes 应用范围数组
     * @return true/false
     */
    public static boolean isScope(@NonNull String... scopes) {
        if (ObjectUtils.isEmpty(scopes)) {
            return false;
        }
        String current = getScope();
        if (current != null) {
            for (String scope : scopes) {
                if (Objects.equals(scope, current)) {
                    return true;
                }
            }
        }
        return false;
    }

    /**
     * 判断是否是指定渠道
     *
     * @param channels 渠道数组
     * @return true/false
     */
    public static boolean isChannel(@NonNull String... channels) {
        if (ObjectUtils.isEmpty(channels)) {
            return false;
        }
        String current = getChannel();
        if (StringUtils.notEmpty(current)) {
            for (String channel : channels) {
                if (Objects.equals(channel, current)) {
                    return true;
                }
            }
        }
        return false;
    }

    /**
     * 判断是否是指定平台
     *
     * @param platforms 平台数组
     * @return true/false
     */
    public static boolean isPlatform(@NonNull Platform... platforms) {
        if (ObjectUtils.isEmpty(platforms)) {
            return false;
        }
        Platform current = getPlatform();
        if (current != null) {
            for (Platform platform : platforms) {
                if (platform == current) {
                    return true;
                }
            }
        }
        return false;
    }

    /**
     * 判断是否是指定终端
     *
     * @param terminals 终端数组
     * @return true/false
     */
    public static boolean isTerminal(@NonNull Terminal... terminals) {
        if (ObjectUtils.isEmpty(terminals)) {
            return false;
        }
        Terminal current = getTerminal();
        if (current != null) {
            for (Terminal terminal : terminals) {
                if (terminal == current) {
                    return true;
                }
            }
        }
        return false;
    }

    /**
     * 判断是否是指定版本
     *
     * @param version 版本注解
     * @return true/false
     */
    public static boolean isVersion(@NonNull com.chelvc.framework.base.annotation.Version version) {
        String value = version.value();
        if (StringUtils.isEmpty(value)) {
            return false;
        }
        Environment environment = ApplicationContextHolder.getEnvironment(false);
        if (environment != null) {
            value = environment.resolvePlaceholders(value);
        }
        value = ApplicationContextHolder.getProperty(value, value);
        if (version.compare() == Compare.EQ) {
            return isVersion(version.terminal(), value);
        } else if (version.compare() == Compare.NE) {
            return !isVersion(version.terminal(), value);
        } else if (version.compare() == Compare.GT) {
            return isAfterVersion(version.terminal(), value);
        } else if (version.compare() == Compare.GE) {
            return isAfterVersion(version.terminal(), value, true);
        } else if (version.compare() == Compare.LT) {
            return isBeforeVersion(version.terminal(), value);
        } else if (version.compare() == Compare.LE) {
            return isBeforeVersion(version.terminal(), value, true);
        }
        return false;
    }

    /**
     * 判断是否是指定版本
     *
     * @param versions 版本注解数组
     * @return true/false
     */
    public static boolean isVersion(@NonNull com.chelvc.framework.base.annotation.Version... versions) {
        if (ObjectUtils.notEmpty(versions)) {
            for (com.chelvc.framework.base.annotation.Version version : versions) {
                if (isVersion(version)) {
                    return true;
                }
            }
        }
        return false;
    }

    /**
     * 判断是否是指定版本
     *
     * @param terminal 指定终端
     * @param version  指定版本
     * @return true/false
     */
    public static boolean isVersion(@NonNull Terminal terminal, @NonNull String version) {
        Session session = getSession(false);
        return terminal == ObjectUtils.ifNull(session, Session::getTerminal)
                && version.equals(ObjectUtils.ifNull(session, Session::getVersion));
    }

    /**
     * 判断是否在指定版本之前
     *
     * @param terminal 指定终端
     * @param version  指定版本
     * @return true/false
     */
    public static boolean isBeforeVersion(@NonNull Terminal terminal, @NonNull String version) {
        return isBeforeVersion(terminal, version, false);
    }

    /**
     * 判断是否在指定版本之前
     *
     * @param terminal 指定终端
     * @param version  指定版本
     * @param contains 是否包含当前版本号
     * @return true/false
     */
    public static boolean isBeforeVersion(@NonNull Terminal terminal, @NonNull String version, boolean contains) {
        Session session = getSession(false);
        return terminal == ObjectUtils.ifNull(session, Session::getTerminal)
                && Version.isBefore(ObjectUtils.ifNull(session, Session::getVersion), version, contains);
    }

    /**
     * 判断是否在指定版本之后
     *
     * @param terminal 指定终端
     * @param version  指定版本
     * @return true/false
     */
    public static boolean isAfterVersion(@NonNull Terminal terminal, @NonNull String version) {
        return isAfterVersion(terminal, version, false);
    }

    /**
     * 判断是否在指定版本之后
     *
     * @param terminal 指定终端
     * @param version  指定版本
     * @param contains 是否包含当前版本号
     * @return true/false
     */
    public static boolean isAfterVersion(@NonNull Terminal terminal, @NonNull String version, boolean contains) {
        Session session = getSession(false);
        return terminal == ObjectUtils.ifNull(session, Session::getTerminal)
                && Version.isAfter(ObjectUtils.ifNull(session, Session::getVersion), version, contains);
    }

    /**
     * 获取当前请求对象
     *
     * @return Http请求对象
     */
    public static HttpServletRequest getRequest() {
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        return ObjectUtils.ifNull(attributes, ServletRequestAttributes::getRequest);
    }

    /**
     * 获取当前响应对象
     *
     * @return Http响应对象
     */
    public static HttpServletResponse getResponse() {
        ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        return ObjectUtils.ifNull(attributes, ServletRequestAttributes::getResponse);
    }

    /**
     * 获取请求参数
     *
     * @param name 参数名称
     * @return 参数值
     */
    public static String getParameter(@NonNull String name) {
        return getParameter(name, value -> value);
    }

    /**
     * 获取请求参数
     *
     * @param name    参数名称
     * @param adapter 参数值适配器
     * @return 参数值
     */
    public static <T> T getParameter(@NonNull String name, @NonNull Function<String, T> adapter) {
        return StringUtils.ifEmpty(ObjectUtils.ifNull(getRequest(), request -> request.getParameter(name)), adapter);
    }

    /**
     * 相应请求结果
     *
     * @param response Http响应对象
     * @param result   请求结果
     * @throws IOException I/O异常
     */
    public static void response(@NonNull HttpServletResponse response, @NonNull Result<?> result) throws IOException {
        response.setStatus(result.status().value());
        response.setContentType(MediaType.APPLICATION_JSON_VALUE);
        response.setCharacterEncoding(JsonEncoding.UTF8.getJavaName());
        OutputStream output = response.getOutputStream();
        JacksonContextHolder.write(output, result);
        output.flush();
    }

    /**
     * 清空上下文信息
     */
    public static void clearSessionContext() {
        Deque<Session> sessions = SESSION_CONTEXT.get();
        sessions.poll();
        if (sessions.isEmpty()) {
            SESSION_CONTEXT.remove();
        }
    }

    @Override
    public void requestInitialized(ServletRequestEvent event) {
        setSession(this.sessionFactory.build((HttpServletRequest) event.getServletRequest()));
    }

    @Override
    public void requestDestroyed(ServletRequestEvent event) {
        clearSessionContext();
    }
}
