package com.chelvc.framework.base.context;

import java.util.concurrent.CompletableFuture;
import java.util.concurrent.Executor;
import java.util.concurrent.ForkJoinPool;
import java.util.function.Supplier;

import com.chelvc.framework.common.util.ThreadUtils;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.springframework.core.task.TaskExecutor;
import org.springframework.scheduling.TaskScheduler;

/**
 * 多线程上下文工具类
 *
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
public final class ThreadContextHolder {
    /**
     * 任务执行线程池
     */
    private static volatile TaskExecutor TASK_EXECUTOR;

    /**
     * 任务调度器
     */
    private static volatile TaskScheduler TASK_SCHEDULER;

    /**
     * 默认任务执行器
     */
    private static final Executor DEFAULT_EXECUTOR = ForkJoinPool.getCommonPoolParallelism() > 1 ?
            ForkJoinPool.commonPool() : ThreadUtils::run;

    private ThreadContextHolder() {
    }

    /**
     * 获取默认任务执行器
     *
     * @return 任务执行器实例
     */
    public static Executor getDefaultExecutor() {
        return DEFAULT_EXECUTOR;
    }

    /**
     * 获取任务执行线程池
     *
     * @return 任务执行线程池实例
     */
    public static TaskExecutor getTaskExecutor() {
        if (TASK_EXECUTOR == null) {
            synchronized (TaskExecutor.class) {
                if (TASK_EXECUTOR == null) {
                    TASK_EXECUTOR = ApplicationContextHolder.getBean(TaskExecutor.class);
                }
            }
        }
        return TASK_EXECUTOR;
    }

    /**
     * 获取任务调度器
     *
     * @return 任务调度器实例
     */
    public static TaskScheduler getTaskScheduler() {
        if (TASK_SCHEDULER == null) {
            synchronized (TaskScheduler.class) {
                if (TASK_SCHEDULER == null) {
                    TASK_SCHEDULER = ApplicationContextHolder.getBean(TaskScheduler.class);
                }
            }
        }
        return TASK_SCHEDULER;
    }

    /**
     * 异步调用方法
     *
     * @param runnable Runnable实例
     * @return CompletableFuture实例
     */
    public static CompletableFuture<Void> run(@NonNull Runnable runnable) {
        return run(getDefaultExecutor(), runnable);
    }

    /**
     * 异步调用方法
     *
     * @param executor 线程执行器
     * @param runnable Runnable实例
     * @return CompletableFuture实例
     */
    public static CompletableFuture<Void> run(@NonNull Executor executor, @NonNull Runnable runnable) {
        Session session = SessionContextHolder.getSession(false);
        return CompletableFuture.runAsync(
                () -> {
                    SessionContextHolder.setSession(session);
                    try {
                        runnable.run();
                    } finally {
                        SessionContextHolder.clearSessionContext();
                    }
                },
                executor
        ).exceptionally(e -> {
            log.error(e.getMessage(), e);
            return null;
        });
    }

    /**
     * 异步调用方法
     *
     * @param supplier 回调方法实例
     * @param <T>      任务结果类型
     * @return CompletableFuture实例
     */
    public static <T> CompletableFuture<T> supply(@NonNull Supplier<T> supplier) {
        return supply(getDefaultExecutor(), supplier);
    }

    /**
     * 异步调用方法
     *
     * @param executor 线程执行器
     * @param supplier 回调方法实例
     * @param <T>      任务结果类型
     * @return CompletableFuture实例
     */
    public static <T> CompletableFuture<T> supply(@NonNull Executor executor, @NonNull Supplier<T> supplier) {
        Session session = SessionContextHolder.getSession(false);
        return CompletableFuture.supplyAsync(
                () -> {
                    SessionContextHolder.setSession(session);
                    try {
                        return supplier.get();
                    } finally {
                        SessionContextHolder.clearSessionContext();
                    }
                },
                executor
        ).exceptionally(e -> {
            log.error(e.getMessage(), e);
            return null;
        });
    }

    /**
     * 异步执行方法（等待执行完成）
     *
     * @param array Runnable实例数组
     */
    public static void execute(@NonNull Runnable... array) {
        execute(getDefaultExecutor(), array);
    }

    /**
     * 异步执行方法（等待执行完成）
     *
     * @param executor 线程执行器
     * @param runs     Runnable实例数组
     */
    public static void execute(@NonNull Executor executor, @NonNull Runnable... runs) {
        if (runs != null && runs.length == 1) {
            run(executor, runs[0]).join();
        } else if (runs != null) {
            CompletableFuture<?>[] futures = new CompletableFuture<?>[runs.length];
            for (int i = 0; i < runs.length; i++) {
                futures[i] = run(executor, runs[i]);
            }
            CompletableFuture.allOf(futures).join();
        }
    }
}
