package com.chelvc.framework.base.interceptor;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import javax.servlet.ReadListener;
import javax.servlet.ServletInputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletRequestWrapper;

import com.chelvc.framework.common.util.CodecUtils;
import com.chelvc.framework.common.util.FileUtils;
import lombok.NonNull;

/**
 * 可缓存请求体的请求包装器实现
 *
 * @author Woody
 * @date 2024/1/30
 */
public class BufferedRequestWrapper extends HttpServletRequestWrapper {
    private final byte[] body;

    public BufferedRequestWrapper(@NonNull HttpServletRequest request) throws IOException {
        this(request, getBody(request));
    }

    public BufferedRequestWrapper(@NonNull HttpServletRequest request, @NonNull byte[] body) {
        super(request);
        this.body = body;
    }

    /**
     * 获取请求体
     *
     * @param request Http请求对象
     * @return 请求体字节数组
     * @throws IOException I/O异常
     */
    private static byte[] getBody(HttpServletRequest request) throws IOException {
        try (InputStream input = request.getInputStream()) {
            if (input instanceof BufferedServletInputStream) {
                return ((BufferedServletInputStream) input).body;
            }
            return FileUtils.getBytes(input);
        }
    }

    @Override
    public BufferedReader getReader() throws IOException {
        return new BufferedReader(new InputStreamReader(this.getInputStream()));
    }

    @Override
    public ServletInputStream getInputStream() throws IOException {
        return new BufferedServletInputStream(this.body);
    }

    /**
     * 可缓存的Servlet输入流实现
     */
    public static class BufferedServletInputStream extends ServletInputStream {
        private final byte[] body;
        private final InputStream delegate;

        public BufferedServletInputStream(@NonNull byte[] body) {
            this.body = body;
            this.delegate = new ByteArrayInputStream(body);
        }

        /**
         * 获取请求体
         *
         * @return 请求体字节数组
         */
        public byte[] body() {
            byte[] copy = new byte[this.body.length];
            System.arraycopy(this.body, 0, copy, 0, this.body.length);
            return copy;
        }

        /**
         * 获取请求体base64解码
         *
         * @return 请求体字节数组
         */
        public byte[] decodeBase64() {
            return CodecUtils.decodeBase64(this.body);
        }

        @Override
        public int read() throws IOException {
            return this.delegate.read();
        }

        @Override
        public int available() throws IOException {
            return this.delegate.available();
        }

        @Override
        public boolean isReady() {
            return true;
        }

        @Override
        public boolean isFinished() {
            return false;
        }

        @Override
        public void setReadListener(ReadListener listener) {
            throw new UnsupportedOperationException();
        }
    }
}
