package com.chelvc.framework.common.model;

import java.io.Serializable;
import java.util.Map;
import java.util.Objects;

import com.chelvc.framework.common.util.ObjectUtils;
import lombok.Getter;

/**
 * 键/值对象模型
 *
 * @param <K> 键类型
 * @param <V> 值类型
 * @author Woody
 * @date 2024/6/15
 */
@Getter
public class Pair<K, V> implements Map.Entry<K, V>, Serializable {
    private final K key;
    private V value;

    public Pair() {
        this(null, null);
    }

    public Pair(K key, V value) {
        this.key = key;
        this.value = value;
    }

    @Override
    public V setValue(V value) {
        return this.value = value;
    }

    @Override
    public int hashCode() {
        return (this.key == null ? 0 : this.key.hashCode()) ^ (this.value == null ? 0 : this.value.hashCode());
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == this) {
            return true;
        } else if (!(obj instanceof Pair)) {
            return false;
        }
        Pair<?, ?> other = (Pair<?, ?>) obj;
        return Objects.equals(this.key, other.getKey()) && Objects.equals(this.value, other.getValue());
    }

    @Override
    public String toString() {
        return "(" + this.key + ',' + this.value + ')';
    }

    /**
     * 构建键/值对象实例
     *
     * @param key   键
     * @param value 值
     * @param <K>   键类型
     * @param <V>   值类型
     * @return 键/值对象实例
     */
    public static <K, V> Pair<K, V> of(K key, V value) {
        return new Pair<>(key, value);
    }

    /**
     * 构建键/值对象实例
     *
     * @param entry 键/值对象实例
     * @param <K>   键类型
     * @param <V>   值类型
     * @return 键/值对象实例
     */
    public static <K, V> Pair<K, V> of(Map.Entry<K, V> entry) {
        return of(ObjectUtils.ifNull(entry, Map.Entry::getKey), ObjectUtils.ifNull(entry, Map.Entry::getValue));
    }
}
