package com.chelvc.framework.common.util;

import java.util.Collection;
import java.util.Map;
import java.util.Objects;
import java.util.function.Supplier;

import com.chelvc.framework.common.exception.ParameterInvalidException;
import com.chelvc.framework.common.exception.ResourceUnavailableException;
import com.chelvc.framework.common.function.Getter;
import lombok.NonNull;

/**
 * 断言工具类
 *
 * @author Woody
 * @date 2024/1/30
 */
public final class AssertUtils {
    private AssertUtils() {
    }

    /**
     * 条件真假检测
     *
     * @param expression 真假表达式
     * @param message    异常消息
     */
    public static void check(boolean expression, String message) {
        if (!expression) {
            throw new IllegalArgumentException(message);
        }
    }

    /**
     * 条件真假检测
     *
     * @param expression 真假表达式
     * @param message    异常消息回调函数
     */
    public static void check(boolean expression, @NonNull Supplier<String> message) {
        if (!expression) {
            throw new IllegalArgumentException(message.get());
        }
    }

    /**
     * 参数有效性校验
     *
     * @param expression 验证表达式
     * @param getter     参数Getter函数实例
     * @param message    异常信息
     * @param <T>        函数调用对象类型
     * @param <R>        函数调用返回类型
     */
    public static <T, R> void check(boolean expression, @NonNull Getter<T, R> getter, @NonNull String message) {
        if (!expression) {
            throw new ParameterInvalidException(ObjectUtils.lookupGetterProperty(getter), message);
        }
    }

    /**
     * 参数有效性校验
     *
     * @param expression 验证表达式
     * @param getter     参数Getter函数实例
     * @param message    异常消息回调函数
     * @param <T>        函数调用对象类型
     * @param <R>        函数调用返回类型
     */
    public static <T, R> void check(boolean expression, @NonNull Getter<T, R> getter,
                                    @NonNull Supplier<String> message) {
        if (!expression) {
            throw new ParameterInvalidException(ObjectUtils.lookupGetterProperty(getter), message.get());
        }
    }

    /**
     * 对象非空校验
     *
     * @param object  非空对象
     * @param message 异常信息
     * @param <T>     对象类型
     * @return 非空对象
     */
    public static <T> T nonnull(T object, String message) {
        available(Objects.nonNull(object), message);
        return object;
    }

    /**
     * 对象非空校验
     *
     * @param object  非空对象
     * @param message 异常信息回调函数
     * @param <T>     对象类型
     * @return 非空对象
     */
    public static <T> T nonnull(T object, @NonNull Supplier<String> message) {
        available(Objects.nonNull(object), message);
        return object;
    }

    /**
     * 字符串非空校验
     *
     * @param string  非空字符串
     * @param message 异常信息
     * @param <T>     对象类型
     * @return 非空字符串
     */
    public static <T extends CharSequence> T nonempty(T string, String message) {
        available(StringUtils.notEmpty(string), message);
        return string;
    }

    /**
     * 字符串非空校验
     *
     * @param string  非空字符串
     * @param message 异常信息回调函数
     * @param <T>     对象类型
     * @return 非空字符串
     */
    public static <T extends CharSequence> T nonempty(T string, @NonNull Supplier<String> message) {
        available(StringUtils.notEmpty(string), message);
        return string;
    }

    /**
     * 数组非空校验
     *
     * @param array   字节数组
     * @param message 异常信息
     * @return 非空数组
     */
    public static byte[] nonempty(byte[] array, String message) {
        available(ObjectUtils.notEmpty(array), message);
        return array;
    }

    /**
     * 数组非空校验
     *
     * @param array   字节数组
     * @param message 异常信息回调函数
     * @return 非空数组
     */
    public static byte[] nonempty(byte[] array, @NonNull Supplier<String> message) {
        available(ObjectUtils.notEmpty(array), message);
        return array;
    }

    /**
     * 数组非空校验
     *
     * @param array   短整型数组
     * @param message 异常信息
     * @return 非空数组
     */
    public static short[] nonempty(short[] array, String message) {
        available(ObjectUtils.notEmpty(array), message);
        return array;
    }

    /**
     * 数组非空校验
     *
     * @param array   短整型数组
     * @param message 异常信息回调函数
     * @return 非空数组
     */
    public static short[] nonempty(short[] array, @NonNull Supplier<String> message) {
        available(ObjectUtils.notEmpty(array), message);
        return array;
    }

    /**
     * 数组非空校验
     *
     * @param array   整型数组
     * @param message 异常信息
     * @return 非空数组
     */
    public static int[] nonempty(int[] array, String message) {
        available(ObjectUtils.notEmpty(array), message);
        return array;
    }

    /**
     * 数组非空校验
     *
     * @param array   整型数组
     * @param message 异常信息回调函数
     * @return 非空数组
     */
    public static int[] nonempty(int[] array, @NonNull Supplier<String> message) {
        available(ObjectUtils.notEmpty(array), message);
        return array;
    }

    /**
     * 数组非空校验
     *
     * @param array   长整型数组
     * @param message 异常信息
     * @return 非空数组
     */
    public static long[] nonempty(long[] array, String message) {
        available(ObjectUtils.notEmpty(array), message);
        return array;
    }

    /**
     * 数组非空校验
     *
     * @param array   长整型数组
     * @param message 异常信息回调函数
     * @return 非空数组
     */
    public static long[] nonempty(long[] array, @NonNull Supplier<String> message) {
        available(ObjectUtils.notEmpty(array), message);
        return array;
    }

    /**
     * 数组非空校验
     *
     * @param array   单精度数组
     * @param message 异常信息
     * @return 非空数组
     */
    public static float[] nonempty(float[] array, String message) {
        available(ObjectUtils.notEmpty(array), message);
        return array;
    }

    /**
     * 数组非空校验
     *
     * @param array   单精度数组
     * @param message 异常信息回调函数
     * @return 非空数组
     */
    public static float[] nonempty(float[] array, @NonNull Supplier<String> message) {
        available(ObjectUtils.notEmpty(array), message);
        return array;
    }

    /**
     * 数组非空校验
     *
     * @param array   双精度数组
     * @param message 异常信息
     * @return 非空数组
     */
    public static double[] nonempty(double[] array, String message) {
        available(ObjectUtils.notEmpty(array), message);
        return array;
    }

    /**
     * 数组非空校验
     *
     * @param array   双精度数组
     * @param message 异常信息回调函数
     * @return 非空数组
     */
    public static double[] nonempty(double[] array, @NonNull Supplier<String> message) {
        available(ObjectUtils.notEmpty(array), message);
        return array;
    }

    /**
     * 数组非空校验
     *
     * @param array   字符数组
     * @param message 异常信息
     * @return 非空数组
     */
    public static char[] nonempty(char[] array, String message) {
        available(ObjectUtils.notEmpty(array), message);
        return array;
    }

    /**
     * 数组非空校验
     *
     * @param array   字符数组
     * @param message 异常信息回调函数
     * @return 非空数组
     */
    public static char[] nonempty(char[] array, @NonNull Supplier<String> message) {
        available(ObjectUtils.notEmpty(array), message);
        return array;
    }

    /**
     * 数组非空校验
     *
     * @param array   布尔数组
     * @param message 异常信息
     * @return 非空数组
     */
    public static boolean[] nonempty(boolean[] array, String message) {
        available(ObjectUtils.notEmpty(array), message);
        return array;
    }

    /**
     * 数组非空校验
     *
     * @param array   布尔数组
     * @param message 异常信息回调函数
     * @return 非空数组
     */
    public static boolean[] nonempty(boolean[] array, @NonNull Supplier<String> message) {
        available(ObjectUtils.notEmpty(array), message);
        return array;
    }

    /**
     * 数组非空校验
     *
     * @param array   非空数组
     * @param message 异常信息
     * @param <T>     对象类型
     * @return 非空数组
     */
    public static <T> T[] nonempty(T[] array, String message) {
        available(ObjectUtils.notEmpty(array), message);
        return array;
    }

    /**
     * 数组非空校验
     *
     * @param array   非空数组
     * @param message 异常信息回调函数
     * @param <T>     对象类型
     * @return 非空数组
     */
    public static <T> T[] nonempty(T[] array, @NonNull Supplier<String> message) {
        available(ObjectUtils.notEmpty(array), message);
        return array;
    }

    /**
     * 字典非空校验
     *
     * @param mapping 非空字典
     * @param message 异常信息
     * @param <K>     字典键类型
     * @param <V>     字典值类型
     * @param <M>     字典类型
     * @return 非空字典
     */
    public static <K, V, M extends Map<K, V>> M nonempty(M mapping, String message) {
        available(ObjectUtils.notEmpty(mapping), message);
        return mapping;
    }

    /**
     * 字典非空校验
     *
     * @param mapping 非空字典
     * @param message 异常信息回调函数
     * @param <K>     字典键类型
     * @param <V>     字典值类型
     * @param <M>     字典类型
     * @return 非空字典
     */
    public static <K, V, M extends Map<K, V>> M nonempty(M mapping, @NonNull Supplier<String> message) {
        available(ObjectUtils.notEmpty(mapping), message);
        return mapping;
    }

    /**
     * 集合非空校验
     *
     * @param collection 非空集合
     * @param message    异常信息
     * @param <T>        集合元素类型
     * @param <C>        集合类型
     */
    public static <T, C extends Collection<T>> C nonempty(C collection, String message) {
        available(ObjectUtils.notEmpty(collection), message);
        return collection;
    }

    /**
     * 集合非空校验
     *
     * @param collection 非空集合
     * @param message    异常信息回调函数
     * @param <T>        集合元素类型
     * @param <C>        集合类型
     */
    public static <T, C extends Collection<T>> C nonempty(C collection, @NonNull Supplier<String> message) {
        available(ObjectUtils.notEmpty(collection), message);
        return collection;
    }

    /**
     * 对象可用性校验
     *
     * @param expression 验证表达式
     * @param message    异常信息
     */
    public static void available(boolean expression, String message) {
        available(expression, null, message);
    }

    /**
     * 对象可用性校验
     *
     * @param expression 验证表达式
     * @param data       异常数据
     * @param message    异常信息
     */
    public static void available(boolean expression, Object data, String message) {
        if (!expression) {
            throw new ResourceUnavailableException(data, message);
        }
    }

    /**
     * 对象可用性校验
     *
     * @param expression 真假表达式
     * @param message    异常消息回调函数
     */
    public static void available(boolean expression, @NonNull Supplier<String> message) {
        available(expression, null, message);
    }

    /**
     * 对象可用性校验
     *
     * @param expression 真假表达式
     * @param data       异常数据
     * @param message    异常消息回调函数
     */
    public static void available(boolean expression, Object data, @NonNull Supplier<String> message) {
        if (!expression) {
            throw new ResourceUnavailableException(data, message.get());
        }
    }
}
