package com.chelvc.framework.common.util;

import java.lang.management.ManagementFactory;
import java.net.InetAddress;
import java.net.NetworkInterface;
import java.util.concurrent.ThreadLocalRandom;
import java.util.regex.Pattern;

import lombok.extern.slf4j.Slf4j;

/**
 * 唯一ID工具类（基于雪花算法）
 *
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
public final class IdentityUtils {
    /**
     * 默认ID生成器实例
     */
    private static volatile Generator DEFAULT_GENERATOR;

    /**
     * 主键匹配模式
     */
    private static final Pattern IDENTITY_PATTERN = Pattern.compile("^[0-9]{1,19}$");

    private IdentityUtils() {
    }

    /**
     * 初始化机器标识ID
     *
     * @param datacenterId 数据标识ID
     * @return 机器标识ID
     */
    private static long initializeWorkerId(long datacenterId) {
        String pid = String.valueOf(datacenterId);
        String name = ManagementFactory.getRuntimeMXBean().getName();
        if (StringUtils.notBlank(name)) {
            /*
             * GET jvmPid
             */
            pid = pid + name.split("@")[0];
        }
        /*
         * MAC + PID 的 hashcode 获取16个低位
         */
        return (pid.hashCode() & 0xffff) % (Generator.MAX_WORKER_ID + 1);
    }

    /**
     * 初始化数据标识ID
     *
     * @return 数据标识ID
     */
    private static long initializeDatacenterId() {
        long id = 0L;
        try {
            InetAddress ip = InetAddress.getLocalHost();
            NetworkInterface network = NetworkInterface.getByInetAddress(ip);
            if (network == null) {
                id = 1L;
            } else {
                byte[] mac = network.getHardwareAddress();
                if (null != mac) {
                    id = ((0x000000FF & (long) mac[mac.length - 2])
                            | (0x0000FF00 & (((long) mac[mac.length - 1]) << 8))) >> 6;
                    id = id % (Generator.MAX_DATACENTER_ID + 1);
                }
            }
        } catch (Exception e) {
            log.warn("Snowflake datacenter id initialize failed: {}", e.getMessage());
        }
        return id;
    }

    /**
     * 获取默认ID生成器
     *
     * @return ID生成器实例
     */
    public static Generator getDefaultGenerator() {
        if (DEFAULT_GENERATOR == null) {
            synchronized (Generator.class) {
                if (DEFAULT_GENERATOR == null) {
                    long datacenterId = initializeDatacenterId();
                    long workerId = initializeWorkerId(datacenterId);
                    log.info("Using default snowflake id configuration: {}, {}", workerId, datacenterId);
                    DEFAULT_GENERATOR = new Generator(workerId, datacenterId);
                }
            }
        }
        return DEFAULT_GENERATOR;
    }


    /**
     * 生成唯一ID
     *
     * @return 唯一ID
     */
    public static long generate() {
        return getDefaultGenerator().next();
    }

    /**
     * 判断字符串是否是主键
     *
     * @param identity 主键字符串
     * @return true/false
     */
    public static boolean isIdentity(String identity) {
        return StringUtils.notEmpty(identity) && IDENTITY_PATTERN.matcher(identity).matches();
    }

    /**
     * ID生成器实现
     */
    public final static class Generator {
        /**
         * 时间起始标记点，作为基准，一般取系统的最近时间（一旦确定不能变动）
         */
        private static final long BENCHMARK = 1288834974657L;

        /**
         * 机器标识位数
         */
        private static final long WORKER_ID_BITS = 5L;
        private static final long DATACENTER_ID_BITS = 5L;
        private static final long MAX_WORKER_ID = ~(-1L << WORKER_ID_BITS);
        private static final long MAX_DATACENTER_ID = ~(-1L << DATACENTER_ID_BITS);

        /**
         * 毫秒内自增位
         */
        private static final long SEQUENCE_BITS = 12L;
        private static final long WORKER_ID_SHIFT = SEQUENCE_BITS;
        private static final long DATACENTER_ID_SHIFT = SEQUENCE_BITS + WORKER_ID_BITS;

        /**
         * 时间戳左移动位
         */
        private static final long TIMESTAMP_LEFT_SHIFT = SEQUENCE_BITS + WORKER_ID_BITS + DATACENTER_ID_BITS;
        private static final long SEQUENCE_MASK = ~(-1L << SEQUENCE_BITS);

        /**
         * 时间回拨偏移量（毫秒）
         */
        private static final long CLOCK_MOVED_BACKWARD_OFFSET = 5;

        /**
         * 机器标识
         */
        private final long workerId;

        /**
         * 数据标识 ID 部分
         */
        private final long datacenterId;

        /**
         * 并发控制
         */
        private long sequence = 0L;

        /**
         * 上次生产 ID 时间戳
         */
        private long lastTimestamp = -1L;

        /**
         * 有参构造器
         *
         * @param workerId     工作机器 ID
         * @param datacenterId 序列号
         */
        public Generator(long workerId, long datacenterId) {
            AssertUtils.check(workerId >= 0 && workerId <= MAX_WORKER_ID,
                    () -> String.format("worker Id can't be greater than %d or less than 0", MAX_WORKER_ID));
            AssertUtils.check(datacenterId >= 0 && datacenterId <= MAX_DATACENTER_ID,
                    () -> String.format("datacenter Id can't be greater than %d or less than 0", MAX_DATACENTER_ID));
            this.workerId = workerId;
            this.datacenterId = datacenterId;
        }

        /**
         * 获取当前时间戳
         *
         * @return 时间戳
         */
        private long now() {
            return System.currentTimeMillis();
        }

        /**
         * 直到下一毫秒
         *
         * @param timestamp 下一目标时间戳
         * @return 当前时间戳
         */
        private long untilNextMillis(long timestamp) {
            long now = this.now();
            while (now <= timestamp) {
                now = this.now();
            }
            return now;
        }

        /**
         * 获取下一个 ID
         *
         * @return 下一个 ID
         */
        public synchronized long next() {
            long timestamp = this.now();
            //闰秒
            if (timestamp < this.lastTimestamp) {
                // 校验时间偏移回拨量
                long offset = this.lastTimestamp - timestamp;
                if (offset > CLOCK_MOVED_BACKWARD_OFFSET) {
                    throw new RuntimeException("Clock moved backwards, refusing to generate id for [" + offset + "ms]");
                }

                // 时间回退timeOffset毫秒内，则允许等待2倍的偏移量后重新获取，解决小范围的时间回拨问题
                try {
                    this.wait(offset << 1);
                } catch (InterruptedException e) {
                    Thread.currentThread().interrupt();
                }

                // 再次获取并校验
                if ((timestamp = this.now()) < this.lastTimestamp) {
                    throw new RuntimeException("Clock moved backwards, refusing to generate id for [" + offset + "ms]");
                }
            }

            if (this.lastTimestamp == timestamp) {
                // 相同毫秒内，序列号自增
                this.sequence = (this.sequence + 1) & SEQUENCE_MASK;
                if (this.sequence == 0) {
                    // 同一毫秒的序列数已经达到最大
                    timestamp = this.untilNextMillis(this.lastTimestamp);
                }
            } else {
                // 不同毫秒内，序列号置为 1 - 3 随机数
                this.sequence = ThreadLocalRandom.current().nextLong(1, 3);
            }

            this.lastTimestamp = timestamp;

            // 时间戳部分 | 数据中心部分 | 机器标识部分 | 序列号部分
            return ((timestamp - BENCHMARK) << TIMESTAMP_LEFT_SHIFT) | (this.datacenterId << DATACENTER_ID_SHIFT)
                    | (this.workerId << WORKER_ID_SHIFT) | this.sequence;
        }
    }
}
