package com.chelvc.framework.common.util;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.lang.reflect.Field;
import java.lang.reflect.Type;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.time.Duration;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.function.BiFunction;
import java.util.stream.Collectors;

import com.chelvc.framework.common.model.Paging;
import com.chelvc.framework.common.model.Period;
import com.chelvc.framework.common.model.Range;
import com.chelvc.framework.common.model.Region;
import com.fasterxml.jackson.annotation.JsonAutoDetect;
import com.fasterxml.jackson.annotation.PropertyAccessor;
import com.fasterxml.jackson.core.JsonGenerator;
import com.fasterxml.jackson.core.JsonParser;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.JsonToken;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.BeanDescription;
import com.fasterxml.jackson.databind.DeserializationContext;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.JavaType;
import com.fasterxml.jackson.databind.JsonDeserializer;
import com.fasterxml.jackson.databind.JsonSerializer;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.SerializationConfig;
import com.fasterxml.jackson.databind.SerializationFeature;
import com.fasterxml.jackson.databind.SerializerProvider;
import com.fasterxml.jackson.databind.module.SimpleModule;
import com.fasterxml.jackson.databind.ser.BeanPropertyWriter;
import com.fasterxml.jackson.databind.ser.BeanSerializerModifier;
import com.fasterxml.jackson.databind.ser.SerializerFactory;
import lombok.NonNull;

/**
 * Jackson工具类
 *
 * @author Woody
 * @date 2024/1/30
 */
public final class JacksonUtils {
    /**
     * 字符串数组类型引用
     */
    public static final TypeReference<String[]> STRING_ARRAY_TYPE = new TypeReference<String[]>() {
    };

    /**
     * 字符串集合类型引用
     */
    public static final TypeReference<Set<String>> STRING_SET_TYPE = new TypeReference<Set<String>>() {
    };

    /**
     * 字符串列表类型引用
     */
    public static final TypeReference<List<String>> STRING_LIST_TYPE = new TypeReference<List<String>>() {
    };

    /**
     * 字符串键/值映射类型引用
     */
    public static final TypeReference<Map<String, String>> STRING_MAP_TYPE = new TypeReference<Map<String, String>>() {
    };

    /**
     * JavaScript最大整型数（2的53次方）
     */
    private static final long JAVA_SCRIPT_MAX_NUMBER = BigInteger.valueOf(2).pow(53).longValue();

    /**
     * JavaScript最大整型数（2的53次方）
     */
    private static final BigInteger JAVA_SCRIPT_MAX_INTEGER = BigInteger.valueOf(JAVA_SCRIPT_MAX_NUMBER);

    /**
     * JavaScript最大小数（2的53次方）
     */
    private static final BigDecimal JAVA_SCRIPT_MAX_DECIMAL = BigDecimal.valueOf(JAVA_SCRIPT_MAX_NUMBER);

    /**
     * 默认对象序列化处理器实例
     */
    private static final ObjectMapper DEFAULT_SERIALIZER = initializeSerializer();

    /**
     * 安全的对象序列化处理器实例
     */
    private static final ObjectMapper SAFELY_SERIALIZER = initializeSerializer(true);

    /**
     * Jackson处理基础模块
     */
    private static volatile SimpleModule BASE_MODULE;

    /**
     * Jackson安全处理模块
     */
    private static volatile SimpleModule SAFETY_MODULE;

    /**
     * Jackson数字处理模块
     */
    private static volatile SimpleModule NUMBER_MODULE;

    /**
     * Jackson null对象序列化处理器更新处理实现
     */
    private static volatile BeanSerializerModifier NULL_MODIFIER;

    private JacksonUtils() {
    }

    /**
     * 获取JSON长整形值
     *
     * @param parser Json Parser
     * @return 长整形数字
     * @throws IOException I/O异常
     */
    private static Long getLongValue(JsonParser parser) throws IOException {
        JsonToken token = parser.getCurrentToken();
        if (token.isNumeric()) {
            JsonParser.NumberType type = parser.getNumberType();
            if (type == JsonParser.NumberType.FLOAT || type == JsonParser.NumberType.DOUBLE) {
                return new BigDecimal(parser.getValueAsString()).longValue();
            }
        } else if (token == JsonToken.VALUE_STRING) {
            return StringUtils.ifEmpty(parser.getText(), text -> new BigDecimal(text).longValue());
        }
        return parser.getLongValue();
    }

    /**
     * 获取JSON短整形值
     *
     * @param parser Json Parser
     * @return 短整形数字
     * @throws IOException I/O异常
     */
    private static Short getShortValue(JsonParser parser) throws IOException {
        JsonToken token = parser.getCurrentToken();
        if (token.isNumeric()) {
            JsonParser.NumberType type = parser.getNumberType();
            if (type == JsonParser.NumberType.FLOAT || type == JsonParser.NumberType.DOUBLE) {
                return new BigDecimal(parser.getValueAsString()).shortValue();
            }
        } else if (token == JsonToken.VALUE_STRING) {
            return StringUtils.ifEmpty(parser.getText(), text -> new BigDecimal(text).shortValue());
        }
        return parser.getShortValue();
    }

    /**
     * 获取JSON整形值
     *
     * @param parser Json Parser
     * @return 整形数字
     * @throws IOException I/O异常
     */
    private static Integer getIntegerValue(JsonParser parser) throws IOException {
        JsonToken token = parser.getCurrentToken();
        if (token.isNumeric()) {
            JsonParser.NumberType type = parser.getNumberType();
            if (type == JsonParser.NumberType.FLOAT || type == JsonParser.NumberType.DOUBLE) {
                return new BigDecimal(parser.getValueAsString()).intValue();
            }
        } else if (token == JsonToken.VALUE_STRING) {
            return StringUtils.ifEmpty(parser.getText(), text -> new BigDecimal(text).intValue());
        }
        return parser.getIntValue();
    }

    /**
     * 构建序列化处理器基础模块
     *
     * @return 模块实例
     */
    private static SimpleModule generateBaseModule() {
        SimpleModule module = new SimpleModule();

        // 设置本地日期序列化处理器
        module.addSerializer(new JsonSerializer<LocalDate>() {
            @Override
            public Class<LocalDate> handledType() {
                return LocalDate.class;
            }

            @Override
            public void serialize(LocalDate date, JsonGenerator generator, SerializerProvider provider)
                    throws IOException {
                generator.writeObject(DateUtils.convert(date));
            }
        });

        // 设置本地日期时间处理器
        module.addSerializer(new JsonSerializer<LocalDateTime>() {
            @Override
            public Class<LocalDateTime> handledType() {
                return LocalDateTime.class;
            }

            @Override
            public void serialize(LocalDateTime datetime, JsonGenerator generator, SerializerProvider provider)
                    throws IOException {
                generator.writeObject(DateUtils.convert(datetime));
            }
        });

        // 设置数字范围序列化处理器
        module.addSerializer(new JsonSerializer<Range>() {
            @Override
            public Class<Range> handledType() {
                return Range.class;
            }

            @Override
            public void serialize(Range range, JsonGenerator generator, SerializerProvider provider)
                    throws IOException {
                generator.writeString(range.toString());
            }
        });

        // 设置时间周期序列化处理器
        module.addSerializer(new JsonSerializer<Period>() {
            @Override
            public Class<Period> handledType() {
                return Period.class;
            }

            @Override
            public void serialize(Period period, JsonGenerator generator, SerializerProvider provider)
                    throws IOException {
                generator.writeString(period.toString());
            }
        });

        // 设置数据分页序列化处理器
        module.addSerializer(new JsonSerializer<Paging>() {
            @Override
            public Class<Paging> handledType() {
                return Paging.class;
            }

            @Override
            public void serialize(Paging paging, JsonGenerator generator, SerializerProvider provider)
                    throws IOException {
                generator.writeString(paging.toString());
            }
        });

        // 设置地区序列化处理器
        module.addSerializer(new JsonSerializer<Region>() {
            @Override
            public Class<Region> handledType() {
                return Region.class;
            }

            @Override
            public void serialize(Region region, JsonGenerator generator, SerializerProvider provider)
                    throws IOException {
                generator.writeNumber(region.getCode());
            }
        });

        // 设置时间周期序列化处理器
        module.addSerializer(new JsonSerializer<Duration>() {
            @Override
            public Class<Duration> handledType() {
                return Duration.class;
            }

            @Override
            public void serialize(Duration duration, JsonGenerator generator, SerializerProvider provider)
                    throws IOException {
                generator.writeString(duration.toString());
            }
        });

        // 设置时间反序列化处理器
        module.addDeserializer(Date.class, new JsonDeserializer<Date>() {
            @Override
            public Class<?> handledType() {
                return Date.class;
            }

            @Override
            public Date deserialize(JsonParser parser, DeserializationContext context) throws IOException {
                return DateUtils.parse(parser.getValueAsString());
            }
        });

        // 设置本地日期反序列化处理器
        module.addDeserializer(LocalDate.class, new JsonDeserializer<LocalDate>() {
            @Override
            public Class<?> handledType() {
                return LocalDate.class;
            }

            @Override
            public LocalDate deserialize(JsonParser parser, DeserializationContext context) throws IOException {
                Date date = DateUtils.parse(parser.getValueAsString());
                return ObjectUtils.ifNull(DateUtils.convert(date), LocalDateTime::toLocalDate);
            }
        });

        // 设置本地日期时间反序列化处理器
        module.addDeserializer(LocalDateTime.class, new JsonDeserializer<LocalDateTime>() {
            @Override
            public Class<?> handledType() {
                return LocalDateTime.class;
            }

            @Override
            public LocalDateTime deserialize(JsonParser parser, DeserializationContext context) throws IOException {
                return DateUtils.convert(DateUtils.parse(parser.getValueAsString()));
            }
        });

        // 设置数字范围反序列化处理器
        module.addDeserializer(Range.class, new JsonDeserializer<Range>() {
            @Override
            public Class<?> handledType() {
                return Range.class;
            }

            @Override
            public Range deserialize(JsonParser parser, DeserializationContext context) throws IOException {
                return Range.parse(parser.getValueAsString());
            }
        });

        // 设置时间周期反序列化处理器
        module.addDeserializer(Period.class, new JsonDeserializer<Period>() {
            @Override
            public Class<?> handledType() {
                return Period.class;
            }

            @Override
            public Period deserialize(JsonParser parser, DeserializationContext context) throws IOException {
                return Period.parse(parser.getValueAsString());
            }
        });

        // 设置数据分页反序列化处理器
        module.addDeserializer(Paging.class, new JsonDeserializer<Paging>() {
            @Override
            public Class<?> handledType() {
                return Paging.class;
            }

            @Override
            public Paging deserialize(JsonParser parser, DeserializationContext context) throws IOException {
                return Paging.parse(parser.getValueAsString());
            }
        });

        // 设置地区反序列化处理器
        module.addDeserializer(Region.class, new JsonDeserializer<Region>() {
            @Override
            public Class<?> handledType() {
                return Region.class;
            }

            @Override
            public Region deserialize(JsonParser parser, DeserializationContext context) throws IOException {
                return Region.of(parser.getIntValue());
            }
        });

        // 设置时间周期反序列化处理器
        module.addDeserializer(Duration.class, new JsonDeserializer<Duration>() {
            @Override
            public Class<?> handledType() {
                return Duration.class;
            }

            @Override
            public Duration deserialize(JsonParser parser, DeserializationContext context) throws IOException {
                return Duration.parse(parser.getValueAsString());
            }
        });
        return module;
    }

    /**
     * 构建序列化处理器安全模块
     *
     * @return 模块实例
     */
    private static SimpleModule generateSafetyModule() {
        SimpleModule module = new SimpleModule();

        // 设置整数序列化处理器
        module.addSerializer(new JsonSerializer<Long>() {
            @Override
            public Class<Long> handledType() {
                return Long.class;
            }

            @Override
            public void serialize(Long number, JsonGenerator generator, SerializerProvider provider)
                    throws IOException {
                if (isSafelyNumber(number)) {
                    generator.writeNumber(number);
                } else {
                    generator.writeString(number.toString());
                }
            }
        });

        // 设置浮点数序列化处理器
        module.addSerializer(new JsonSerializer<Double>() {
            @Override
            public Class<Double> handledType() {
                return Double.class;
            }

            @Override
            public void serialize(Double number, JsonGenerator generator, SerializerProvider provider)
                    throws IOException {
                if (isSafelyNumber(number)) {
                    generator.writeNumber(number);
                } else {
                    generator.writeString(number.toString());
                }
            }
        });

        // 设置大整数序列化处理器
        module.addSerializer(new JsonSerializer<BigInteger>() {
            @Override
            public Class<BigInteger> handledType() {
                return BigInteger.class;
            }

            @Override
            public void serialize(BigInteger number, JsonGenerator generator, SerializerProvider provider)
                    throws IOException {
                if (isSafelyNumber(number)) {
                    generator.writeNumber(number);
                } else {
                    generator.writeString(number.toString());
                }
            }
        });

        // 设置大浮点数序列化处理器
        module.addSerializer(new JsonSerializer<BigDecimal>() {
            @Override
            public Class<BigDecimal> handledType() {
                return BigDecimal.class;
            }

            @Override
            public void serialize(BigDecimal number, JsonGenerator generator, SerializerProvider provider)
                    throws IOException {
                if (isSafelyNumber(number)) {
                    generator.writeNumber(number);
                } else {
                    generator.writeString(number.toPlainString());
                }
            }
        });
        return module;
    }

    /**
     * 构建序列化处理器配置模块
     *
     * @return 模块实例
     */
    private static SimpleModule generateConfigureModule() {
        SimpleModule module = new SimpleModule();
        module.addDeserializer(long.class, new JsonDeserializer<Long>() {
            @Override
            public Long deserialize(JsonParser parser, DeserializationContext context) throws IOException {
                return getLongValue(parser);
            }
        });
        module.addDeserializer(Long.class, new JsonDeserializer<Long>() {
            @Override
            public Long deserialize(JsonParser parser, DeserializationContext context) throws IOException {
                return getLongValue(parser);
            }
        });
        module.addDeserializer(short.class, new JsonDeserializer<Short>() {
            @Override
            public Short deserialize(JsonParser parser, DeserializationContext context) throws IOException {
                return getShortValue(parser);
            }
        });
        module.addDeserializer(Short.class, new JsonDeserializer<Short>() {
            @Override
            public Short deserialize(JsonParser parser, DeserializationContext context) throws IOException {
                return getShortValue(parser);
            }
        });
        module.addDeserializer(int.class, new JsonDeserializer<Integer>() {
            @Override
            public Integer deserialize(JsonParser parser, DeserializationContext context) throws IOException {
                return getIntegerValue(parser);
            }
        });
        module.addDeserializer(Integer.class, new JsonDeserializer<Integer>() {
            @Override
            public Integer deserialize(JsonParser parser, DeserializationContext context) throws IOException {
                return getIntegerValue(parser);
            }
        });
        return module;
    }

    /**
     * 构建null对象序列化处理器更新对象
     *
     * @return null对象序列化处理更新实例
     */
    private static BeanSerializerModifier generateNullModifier() {
        return new BeanSerializerModifier() {
            /**
             * 空字符串序列化处理
             */
            private final JsonSerializer<Object> stringSerializer = new JsonSerializer<Object>() {
                @Override
                public void serialize(Object value, JsonGenerator generator, SerializerProvider provider)
                        throws IOException {
                    if (value == null) {
                        generator.writeString(StringUtils.EMPTY);
                    } else {
                        generator.writeObject(value);
                    }
                }
            };

            /**
             * 空字典序列化处理器
             */
            private final JsonSerializer<Object> mappingSerializer = new JsonSerializer<Object>() {
                @Override
                public void serialize(Object value, JsonGenerator generator, SerializerProvider provider)
                        throws IOException {
                    if (value == null) {
                        generator.writeStartObject();
                        generator.writeEndObject();
                    } else {
                        generator.writeObject(value);
                    }
                }
            };

            /**
             * 空集合序列化处理器
             */
            private final JsonSerializer<Object> collectionSerializer = new JsonSerializer<Object>() {
                @Override
                public void serialize(Object value, JsonGenerator generator, SerializerProvider provider)
                        throws IOException {
                    if (value == null) {
                        generator.writeStartArray();
                        generator.writeEndArray();
                    } else {
                        generator.writeObject(value);
                    }
                }
            };

            @Override
            public List<BeanPropertyWriter> changeProperties(SerializationConfig config, BeanDescription bean,
                                                             List<BeanPropertyWriter> writers) {
                writers.forEach(writer -> {
                    Class<?> clazz = writer.getType().getRawClass();
                    if (CharSequence.class.isAssignableFrom(clazz)) {
                        writer.assignNullSerializer(this.stringSerializer);
                    } else if (Map.class.isAssignableFrom(clazz)) {
                        writer.assignNullSerializer(this.mappingSerializer);
                    } else if (clazz.isArray() || Collection.class.isAssignableFrom(clazz)) {
                        writer.assignNullSerializer(this.collectionSerializer);
                    }
                });
                return writers;
            }
        };
    }

    /**
     * 构建序列化处理器基础模块
     *
     * @return 模块实例
     */
    private static SimpleModule initializeBaseModule() {
        if (BASE_MODULE == null) {
            synchronized (SimpleModule.class) {
                if (BASE_MODULE == null) {
                    BASE_MODULE = generateBaseModule();
                }
            }
        }
        return BASE_MODULE;
    }

    /**
     * 初始化序列化处理器安全模块
     *
     * @return 模块实例
     */
    private static SimpleModule initializeSafetyModule() {
        if (SAFETY_MODULE == null) {
            synchronized (SimpleModule.class) {
                if (SAFETY_MODULE == null) {
                    SAFETY_MODULE = generateSafetyModule();
                }
            }
        }
        return SAFETY_MODULE;
    }

    /**
     * 初始化序列化处理器数字模块
     *
     * @return 模块实例
     */
    private static SimpleModule initializeNumberModule() {
        if (NUMBER_MODULE == null) {
            synchronized (SimpleModule.class) {
                if (NUMBER_MODULE == null) {
                    NUMBER_MODULE = generateConfigureModule();
                }
            }
        }
        return NUMBER_MODULE;
    }

    /**
     * 初始化null对象序列化处理器更新对象
     *
     * @return null对象序列化处理更新实例
     */
    private static BeanSerializerModifier initializeNullModifier() {
        if (NULL_MODIFIER == null) {
            synchronized (BeanSerializerModifier.class) {
                if (NULL_MODIFIER == null) {
                    NULL_MODIFIER = generateNullModifier();
                }
            }
        }
        return NULL_MODIFIER;
    }

    /**
     * 对象序列化处理器基础配置
     *
     * @param mapper 对象序列化处理器实例
     * @return 对象序列化处理器实例
     */
    public static ObjectMapper configure(@NonNull ObjectMapper mapper) {
        mapper = mapper.setVisibility(PropertyAccessor.ALL, JsonAutoDetect.Visibility.ANY)
                .configure(SerializationFeature.INDENT_OUTPUT, false)
                .configure(JsonParser.Feature.ALLOW_SINGLE_QUOTES, true)
                .configure(SerializationFeature.FAIL_ON_EMPTY_BEANS, false)
                .configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, false)
                .enable(DeserializationFeature.USE_BIG_DECIMAL_FOR_FLOATS);
        mapper.registerModule(initializeNumberModule());
        return mapper;
    }

    /**
     * 初始化基础的对象序列化处理器
     *
     * @return 对象序列化处理器实例
     */
    public static ObjectMapper initializeSerializer() {
        return initializeSerializer(false);
    }

    /**
     * 初始化基础的对象序列化处理器
     *
     * @param safely 是否注册安全模块
     * @return 对象序列化处理器实例
     */
    public static ObjectMapper initializeSerializer(boolean safely) {
        return initializeSerializer(new ObjectMapper(), safely);
    }

    /**
     * 初始化基础的对象序列化处理器
     *
     * @param mapper 对象序列化处理器实例
     * @return 对象序列化处理器实例
     */
    public static ObjectMapper initializeSerializer(@NonNull ObjectMapper mapper) {
        return initializeSerializer(mapper, false);
    }

    /**
     * 初始化基础的对象序列化处理器
     *
     * @param mapper 对象序列化处理器实例
     * @param safely 是否注册安全模块
     * @return 对象序列化处理器实例
     */
    public static ObjectMapper initializeSerializer(@NonNull ObjectMapper mapper, boolean safely) {
        // 基本配置
        mapper = configure(mapper);

        // 注册基础模型
        mapper.registerModule(initializeBaseModule());

        // 注册安全模型
        if (safely) {
            mapper.registerModule(initializeSafetyModule());
            SerializerFactory factory = mapper.getSerializerFactory();
            mapper.setSerializerFactory(factory.withSerializerModifier(initializeNullModifier()));
        }
        return mapper;
    }

    /**
     * 判断数字是否安全
     *
     * @param number 数字
     * @return true/false
     */
    public static boolean isSafelyNumber(Number number) {
        if (number instanceof Long) {
            return (long) number <= JAVA_SCRIPT_MAX_NUMBER;
        } else if (number instanceof Double) {
            return (double) number <= JAVA_SCRIPT_MAX_NUMBER;
        } else if (number instanceof BigInteger) {
            return ((BigInteger) number).compareTo(JAVA_SCRIPT_MAX_INTEGER) <= 0;
        } else if (number instanceof BigDecimal) {
            return ((BigDecimal) number).compareTo(JAVA_SCRIPT_MAX_DECIMAL) <= 0;
        }
        return true;
    }

    /**
     * 获取默认的对象序列化处理器
     *
     * @return 对象序列化处理器实例
     */
    public static ObjectMapper getDefaultSerializer() {
        return DEFAULT_SERIALIZER;
    }

    /**
     * 获取安全的对象序列化处理器
     *
     * @return 对象序列化处理器实例
     */
    public static ObjectMapper getSafelySerializer() {
        return SAFELY_SERIALIZER;
    }

    /**
     * 将对象写入输出流
     *
     * @param output 输出流对象
     * @param value  对象实例
     * @throws IOException I/O异常
     */
    public static void write(@NonNull OutputStream output, Object value) throws IOException {
        write(getDefaultSerializer(), output, value);
    }

    /**
     * 将对象写入输出流
     *
     * @param mapper 对象序列化处理器实例
     * @param output 输出流对象
     * @param value  对象实例
     * @throws IOException I/O异常
     */
    public static void write(@NonNull ObjectMapper mapper, @NonNull OutputStream output, Object value)
            throws IOException {
        mapper.writeValue(output, value);
    }

    /**
     * 对象序列化
     *
     * @param object 目标对象实例
     * @return JSON字符串
     */
    public static String serialize(Object object) {
        return serialize(object, false);
    }

    /**
     * 对象序列化
     *
     * @param object 目标对象实例
     * @param safely 是否是安全模式
     * @return JSON字符串
     */
    public static String serialize(Object object, boolean safely) {
        return safely ? serialize(getSafelySerializer(), object) : serialize(getDefaultSerializer(), object);
    }

    /**
     * 对象序列化
     *
     * @param mapper 对象序列化处理器实例
     * @param object 目标对象实例
     * @return JSON字符串
     */
    public static String serialize(@NonNull ObjectMapper mapper, Object object) {
        if (object == null) {
            return null;
        }
        try {
            return mapper.writeValueAsString(object);
        } catch (JsonProcessingException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 对象反序列化
     *
     * @param json JSON字符串
     * @return 对象实例
     */
    public static Object deserialize(String json) {
        return deserialize(getDefaultSerializer(), json, Object.class);
    }

    /**
     * 对象反序列化
     *
     * @param json JSON字符串
     * @param type 类型对象
     * @return 对象实例
     */
    public static Object deserialize(String json, @NonNull Type type) {
        return deserialize(getDefaultSerializer(), json, type);
    }

    /**
     * 对象反序列化
     *
     * @param json JSON字符串
     * @param type 类型对象
     * @param <T>  对象类型
     * @return 对象实例
     */
    public static <T> T deserialize(String json, @NonNull Class<T> type) {
        return deserialize(getDefaultSerializer(), json, type);
    }

    /**
     * 对象反序列化
     *
     * @param json JSON字符串
     * @param type 类型引用对象
     * @param <T>  对象类型
     * @return 对象实例
     */
    public static <T> T deserialize(String json, @NonNull TypeReference<T> type) {
        return deserialize(getDefaultSerializer(), json, type);
    }

    /**
     * 对象反序列化
     *
     * @param mapper 对象序列化处理器实例
     * @param json   JSON字符串
     * @return 对象实例
     */
    public static Object deserialize(@NonNull ObjectMapper mapper, String json) {
        return deserialize(mapper, json, Object.class);
    }

    /**
     * 对象反序列化
     *
     * @param mapper 对象序列化处理器实例
     * @param json   JSON字符串
     * @param type   类型对象
     * @return 对象实例
     */
    public static Object deserialize(@NonNull ObjectMapper mapper, String json, @NonNull Type type) {
        if (type instanceof Class) {
            return deserialize(mapper, json, (Class<?>) type);
        } else if (json == null) {
            return null;
        }
        JavaType javaType = mapper.getTypeFactory().constructType(type);
        try {
            return mapper.readValue(json, javaType);
        } catch (JsonProcessingException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 对象反序列化
     *
     * @param mapper 对象序列化处理器实例
     * @param json   JSON字符串
     * @param type   类型对象
     * @param <T>    对象类型
     * @return 对象实例
     */
    public static <T> T deserialize(@NonNull ObjectMapper mapper, String json, @NonNull Class<T> type) {
        if (json == null) {
            return null;
        }
        try {
            return mapper.readValue(json, type);
        } catch (JsonProcessingException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 对象反序列化
     *
     * @param mapper 对象序列化处理器实例
     * @param json   JSON字符串
     * @param type   类型引用对象
     * @param <T>    对象类型
     * @return 对象实例
     */
    public static <T> T deserialize(@NonNull ObjectMapper mapper, String json, @NonNull TypeReference<T> type) {
        if (json == null) {
            return null;
        }
        try {
            return mapper.readValue(json, type);
        } catch (JsonProcessingException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 对象反序列化
     *
     * @param bytes JSON字节数组
     * @return 对象实例
     */
    public static Object deserialize(byte[] bytes) {
        return deserialize(getDefaultSerializer(), bytes, Object.class);
    }

    /**
     * 对象反序列化
     *
     * @param bytes JSON字节数组
     * @param type  类型对象
     * @return 对象实例
     */
    public static Object deserialize(byte[] bytes, @NonNull Type type) {
        return deserialize(getDefaultSerializer(), bytes, type);
    }

    /**
     * 对象反序列化
     *
     * @param bytes JSON字节数组
     * @param type  类型对象
     * @param <T>   对象类型
     * @return 对象实例
     */
    public static <T> T deserialize(byte[] bytes, @NonNull Class<T> type) {
        return deserialize(getDefaultSerializer(), bytes, type);
    }

    /**
     * 对象反序列化
     *
     * @param bytes JSON字节数组
     * @param type  类型引用对象
     * @param <T>   对象类型
     * @return 对象实例
     */
    public static <T> T deserialize(byte[] bytes, @NonNull TypeReference<T> type) {
        return deserialize(getDefaultSerializer(), bytes, type);
    }

    /**
     * 对象反序列化
     *
     * @param mapper 对象序列化处理器实例
     * @param bytes  JSON字节数组
     * @return 对象实例
     */
    public static Object deserialize(@NonNull ObjectMapper mapper, byte[] bytes) {
        return deserialize(mapper, bytes, Object.class);
    }

    /**
     * 对象反序列化
     *
     * @param mapper 对象序列化处理器实例
     * @param bytes  JSON字节数组
     * @param type   类型对象
     * @return 对象实例
     */
    public static Object deserialize(@NonNull ObjectMapper mapper, byte[] bytes, @NonNull Type type) {
        if (type instanceof Class) {
            return deserialize(mapper, bytes, (Class<?>) type);
        } else if (bytes == null) {
            return null;
        }
        JavaType javaType = mapper.getTypeFactory().constructType(type);
        try {
            return mapper.readValue(bytes, javaType);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 对象反序列化
     *
     * @param mapper 对象序列化处理器实例
     * @param bytes  JSON字节数组
     * @param type   类型对象
     * @param <T>    对象类型
     * @return 对象实例
     */
    public static <T> T deserialize(@NonNull ObjectMapper mapper, byte[] bytes, @NonNull Class<T> type) {
        if (bytes == null) {
            return null;
        }
        try {
            return mapper.readValue(bytes, type);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 对象反序列化
     *
     * @param mapper 对象序列化处理器实例
     * @param bytes  JSON字节数组
     * @param type   类型引用对象
     * @param <T>    对象类型
     * @return 对象实例
     */
    public static <T> T deserialize(@NonNull ObjectMapper mapper, byte[] bytes, @NonNull TypeReference<T> type) {
        if (bytes == null) {
            return null;
        }
        try {
            return mapper.readValue(bytes, type);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 对象反序列化
     *
     * @param input JSON字节流
     * @return 对象实例
     * @throws IOException I/O异常
     */
    public static Object deserialize(InputStream input) throws IOException {
        return deserialize(getDefaultSerializer(), input, Object.class);
    }

    /**
     * 对象反序列化
     *
     * @param input JSON字节流
     * @param type  类型引用对象
     * @return 对象实例
     * @throws IOException I/O异常
     */
    public static Object deserialize(InputStream input, @NonNull Type type) throws IOException {
        return deserialize(getDefaultSerializer(), input, type);
    }

    /**
     * 对象反序列化
     *
     * @param input JSON字节流
     * @param type  类型引用对象
     * @param <T>   对象类型
     * @return 对象实例
     * @throws IOException I/O异常
     */
    public static <T> T deserialize(InputStream input, @NonNull Class<T> type) throws IOException {
        return deserialize(getDefaultSerializer(), input, type);
    }

    /**
     * 对象反序列化
     *
     * @param input JSON字节流
     * @param type  类型引用对象
     * @param <T>   对象类型
     * @return 对象实例
     * @throws IOException I/O异常
     */
    public static <T> T deserialize(InputStream input, @NonNull TypeReference<T> type) throws IOException {
        return deserialize(getDefaultSerializer(), input, type);
    }

    /**
     * 对象反序列化
     *
     * @param mapper 对象序列化处理器实例
     * @param input  JSON字节流
     * @return 对象实例
     * @throws IOException I/O异常
     */
    public static Object deserialize(@NonNull ObjectMapper mapper, InputStream input) throws IOException {
        return deserialize(mapper, input, Object.class);
    }

    /**
     * 对象反序列化
     *
     * @param mapper 对象序列化处理器实例
     * @param input  JSON字节流
     * @param type   类型引用对象
     * @return 对象实例
     * @throws IOException I/O异常
     */
    public static Object deserialize(@NonNull ObjectMapper mapper, InputStream input, @NonNull Type type)
            throws IOException {
        if (type instanceof Class) {
            return deserialize(mapper, input, (Class<?>) type);
        }
        return input == null ? null : mapper.readValue(input, mapper.getTypeFactory().constructType(type));
    }

    /**
     * 对象反序列化
     *
     * @param mapper 对象序列化处理器实例
     * @param input  JSON字节流
     * @param type   类型引用对象
     * @param <T>    对象类型
     * @return 对象实例
     * @throws IOException I/O异常
     */
    public static <T> T deserialize(@NonNull ObjectMapper mapper, InputStream input, @NonNull Class<T> type)
            throws IOException {
        return input == null ? null : mapper.readValue(input, type);
    }

    /**
     * 对象反序列化
     *
     * @param mapper 对象序列化处理器实例
     * @param input  JSON字节流
     * @param type   类型引用对象
     * @param <T>    对象类型
     * @return 对象实例
     * @throws IOException I/O异常
     */
    public static <T> T deserialize(@NonNull ObjectMapper mapper, InputStream input, @NonNull TypeReference<T> type)
            throws IOException {
        return input == null ? null : mapper.readValue(input, type);
    }

    /**
     * 对象反序列化
     *
     * @param map  对象属性/值映射表
     * @param type 类型对象
     * @param <T>  对象类型
     * @return 对象实例
     */
    public static <T> T deserialize(Map<String, ?> map, @NonNull Class<T> type) {
        return deserialize(map, type, (field, value) -> value);
    }

    /**
     * 对象反序列化
     *
     * @param map      对象属性/值映射表
     * @param type     类型对象
     * @param function 字段值处理回调函数
     * @param <T>      对象类型
     * @return 对象实例
     */
    public static <T> T deserialize(Map<String, ?> map, @NonNull Class<T> type,
                                    @NonNull BiFunction<Field, Object, Object> function) {
        return deserialize(getDefaultSerializer(), map, type, function);
    }

    /**
     * 对象反序列化
     *
     * @param mapper 对象序列化处理器实例
     * @param map    对象属性/值映射表
     * @param type   类型对象
     * @param <T>    对象类型
     * @return 对象实例
     */
    public static <T> T deserialize(@NonNull ObjectMapper mapper, Map<String, ?> map, @NonNull Class<T> type) {
        return deserialize(mapper, map, type, (field, value) -> value);
    }

    /**
     * 对象反序列化
     *
     * @param mapper   对象序列化处理器实例
     * @param map      对象属性/值映射表
     * @param type     类型对象
     * @param function 字段值处理回调函数
     * @param <T>      对象类型
     * @return 对象实例
     */
    public static <T> T deserialize(@NonNull ObjectMapper mapper, Map<String, ?> map, @NonNull Class<T> type,
                                    @NonNull BiFunction<Field, Object, Object> function) {
        if (map == null) {
            return null;
        }
        T instance = ObjectUtils.instance(type);
        if (ObjectUtils.notEmpty(map)) {
            Map<String, Field> fields = ObjectUtils.getClassFields(type);
            if (ObjectUtils.notEmpty(fields)) {
                for (Map.Entry<String, ?> entry : map.entrySet()) {
                    Field field = fields.get(entry.getKey());
                    if (field != null) {
                        Object value = entry.getValue();
                        if (field.getType() == String.class) {
                            value = ObjectUtils.ifNull(value, Object::toString);
                        } else if (value instanceof String
                                || (value != null && ObjectUtils.isMetaClass(field.getType())
                                && !ObjectUtils.isSameClass(field.getType(), value.getClass()))) {
                            value = deserialize(mapper, value.toString(), field.getGenericType());
                        }
                        ObjectUtils.setObjectValue(instance, field, function.apply(field, value));
                    }
                }
            }
        }
        return instance;
    }

    /**
     * 对象反序列化
     *
     * @param maps 对象属性/值映射表集合
     * @param type 类型对象
     * @param <T>  对象类型
     * @return 对象实例
     */
    public static <T, M extends Map<String, ?>> List<T> deserialize(Collection<M> maps, @NonNull Class<T> type) {
        return deserialize(maps, type, (field, value) -> value);
    }

    /**
     * 对象反序列化
     *
     * @param maps     对象属性/值映射表集合
     * @param type     类型对象
     * @param function 字段值处理回调函数
     * @param <T>      对象类型
     * @return 对象实例
     */
    public static <T, M extends Map<String, ?>> List<T> deserialize(Collection<M> maps, @NonNull Class<T> type,
                                                                    @NonNull BiFunction<Field, Object, Object> function) {
        return deserialize(getDefaultSerializer(), maps, type, function);
    }

    /**
     * 对象反序列化
     *
     * @param mapper 对象序列化处理器实例
     * @param maps   对象属性/值映射表集合
     * @param type   类型对象
     * @param <T>    对象类型
     * @return 对象实例
     */
    public static <T, M extends Map<String, ?>> List<T> deserialize(@NonNull ObjectMapper mapper, Collection<M> maps,
                                                                    @NonNull Class<T> type) {
        return deserialize(mapper, maps, type, (field, value) -> value);
    }

    /**
     * 对象反序列化
     *
     * @param mapper   对象序列化处理器实例
     * @param maps     对象属性/值映射表集合
     * @param type     类型对象
     * @param function 字段值处理回调函数
     * @param <T>      对象类型
     * @return 对象实例
     */
    public static <T, M extends Map<String, ?>> List<T> deserialize(@NonNull ObjectMapper mapper, Collection<M> maps,
                                                                    @NonNull Class<T> type,
                                                                    @NonNull BiFunction<Field, Object, Object> function) {
        if (ObjectUtils.isEmpty(maps)) {
            return Collections.emptyList();
        }
        return maps.stream().map(map -> deserialize(mapper, map, type, function)).collect(Collectors.toList());
    }

    /**
     * 对象类型转换
     *
     * @param object 原对象实例
     * @param type   目标对象类型
     * @param <T>    数据类型
     * @return 对象实例
     */
    public static <T> T convert(Object object, @NonNull Class<T> type) {
        return convert(getDefaultSerializer(), object, type);
    }

    /**
     * 对象类型转换
     *
     * @param object 原对象实例
     * @param type   目标对象类型
     * @param <T>    数据类型
     * @return 对象实例
     */
    public static <T> T convert(Object object, @NonNull TypeReference<T> type) {
        return convert(getDefaultSerializer(), object, type);
    }

    /**
     * 对象类型转换
     *
     * @param mapper 对象序列化处理器实例
     * @param object 原对象实例
     * @param type   目标对象类型
     * @param <T>    数据类型
     * @return 对象实例
     */
    public static <T> T convert(@NonNull ObjectMapper mapper, Object object, @NonNull Class<T> type) {
        return object == null ? null : mapper.convertValue(object, type);
    }

    /**
     * 对象类型转换
     *
     * @param mapper 对象序列化处理器实例
     * @param object 原对象实例
     * @param type   目标对象类型
     * @param <T>    数据类型
     * @return 对象实例
     */
    public static <T> T convert(@NonNull ObjectMapper mapper, Object object, @NonNull TypeReference<T> type) {
        return object == null ? null : mapper.convertValue(object, type);
    }
}
