package com.chelvc.framework.common.util;

import java.beans.BeanInfo;
import java.beans.IntrospectionException;
import java.beans.Introspector;
import java.beans.PropertyDescriptor;
import java.io.Serializable;
import java.lang.invoke.SerializedLambda;
import java.lang.ref.WeakReference;
import java.lang.reflect.Array;
import java.lang.reflect.Field;
import java.lang.reflect.GenericArrayType;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.nio.ByteBuffer;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Set;
import java.util.function.BiConsumer;
import java.util.function.BiPredicate;
import java.util.function.Consumer;
import java.util.function.Function;
import java.util.function.Predicate;
import java.util.function.Supplier;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import com.chelvc.framework.common.function.Getter;
import com.chelvc.framework.common.function.Setter;
import com.chelvc.framework.common.model.Modification;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import io.protostuff.LinkedBuffer;
import io.protostuff.ProtobufIOUtil;
import io.protostuff.Schema;
import io.protostuff.runtime.RuntimeSchema;
import lombok.NonNull;
import ma.glasnost.orika.MapperFactory;
import ma.glasnost.orika.MappingContext;
import ma.glasnost.orika.converter.BidirectionalConverter;
import ma.glasnost.orika.impl.DefaultMapperFactory;
import org.apache.commons.lang3.reflect.TypeUtils;

/**
 * 对象工具类
 *
 * @author Woody
 * @date 2024/1/30
 */
public final class ObjectUtils {
    /**
     * 空对象标识（空字节数组）
     */
    public static final byte[] EMPTY = new byte[0];

    /**
     * CPU数量
     */
    public static final int CPU_COUNT = Runtime.getRuntime().availableProcessors();

    /**
     * 对象转换映射工厂
     */
    private static final MapperFactory MAPPER_FACTORY = new DefaultMapperFactory.Builder().build();

    /**
     * 对象字段映射表
     */
    private static final Map<Class<?>, WeakReference<Map<String, Field>>> CLASS_FIELD_MAPPING = Maps.newConcurrentMap();

    /**
     * Protostuff对象类型/Schema映射表
     */
    private static final Map<Class<?>, WeakReference<Schema<?>>> PROTOSTUFF_SCHEMA_MAPPING = Maps.newConcurrentMap();

    /**
     * SerializedLambda 反序列化缓存
     */
    private static final Map<Class<?>, WeakReference<SerializedLambda>> LAMBDA_FUNCTION_MAPPING =
            Maps.newConcurrentMap();

    /**
     * 类对象/属性描述映射表
     */
    private static final Map<Class<?>, WeakReference<List<PropertyDescriptor>>> PROPERTY_DESCRIPTOR_MAPPING =
            Maps.newConcurrentMap();

    static {
        // 注册简单的对象拷贝转换器
        MAPPER_FACTORY.getConverterFactory().registerConverter("simple",
                new BidirectionalConverter<Object, Object>() {
                    @Override
                    public Object convertTo(Object object, ma.glasnost.orika.metadata.Type type,
                                            MappingContext mappingContext) {
                        return object;
                    }

                    @Override
                    public Object convertFrom(Object object, ma.glasnost.orika.metadata.Type type,
                                              MappingContext mappingContext) {
                        return object;
                    }
                });

        // 注册Modification属性转换器
        MAPPER_FACTORY.classMap(Modification.class, Modification.class)
                .fieldMap("before", "before").converter("simple").add()
                .fieldMap("after", "after").converter("simple").add().byDefault().register();
    }

    private ObjectUtils() {
    }

    /**
     * 判断数组是否为空
     *
     * @param array 字节数组
     * @return true/false
     */
    public static boolean isEmpty(byte[] array) {
        return array == null || array.length == 0;
    }

    /**
     * 判断数组是否为空
     *
     * @param array 短整型数组
     * @return true/false
     */
    public static boolean isEmpty(short[] array) {
        return array == null || array.length == 0;
    }

    /**
     * 判断数组是否为空
     *
     * @param array 整型数组
     * @return true/false
     */
    public static boolean isEmpty(int[] array) {
        return array == null || array.length == 0;
    }

    /**
     * 判断数组是否为空
     *
     * @param array 长整型数组
     * @return true/false
     */
    public static boolean isEmpty(long[] array) {
        return array == null || array.length == 0;
    }

    /**
     * 判断数组是否为空
     *
     * @param array 单精度数组
     * @return true/false
     */
    public static boolean isEmpty(float[] array) {
        return array == null || array.length == 0;
    }

    /**
     * 判断数组是否为空
     *
     * @param array 双精度数组
     * @return true/false
     */
    public static boolean isEmpty(double[] array) {
        return array == null || array.length == 0;
    }

    /**
     * 判断数组是否为空
     *
     * @param array 字符数组
     * @return true/false
     */
    public static boolean isEmpty(char[] array) {
        return array == null || array.length == 0;
    }

    /**
     * 判断数组是否为空
     *
     * @param array 布尔数组
     * @return true/false
     */
    public static boolean isEmpty(boolean[] array) {
        return array == null || array.length == 0;
    }

    /**
     * 判断数组是否为空
     *
     * @param array 对象数组
     * @param <T>   对象类型
     * @return true/false
     */
    public static <T> boolean isEmpty(T[] array) {
        return array == null || array.length == 0;
    }

    /**
     * 判断字典是否为空
     *
     * @param map 键/值映射表
     * @param <K> 键类型
     * @param <V> 值类型
     * @return true/false
     */
    public static <K, V> boolean isEmpty(Map<K, V> map) {
        return map == null || map.isEmpty();
    }

    /**
     * 判断集合是否为空
     *
     * @param collection 对象集合
     * @param <T>        对象类型
     * @return true/false
     */
    public static <T> boolean isEmpty(Collection<T> collection) {
        return collection == null || collection.isEmpty();
    }

    /**
     * 判断对象是否为空
     *
     * @param object 对象实例
     * @return true/false
     */
    public static boolean isBlank(Object object) {
        return StringUtils.isEmpty(object)
                || (object instanceof byte[] && ((byte[]) object).length == 0)
                || (object instanceof short[] && ((short[]) object).length == 0)
                || (object instanceof int[] && ((int[]) object).length == 0)
                || (object instanceof long[] && ((long[]) object).length == 0)
                || (object instanceof float[] && ((float[]) object).length == 0)
                || (object instanceof double[] && ((double[]) object).length == 0)
                || (object instanceof char[] && ((char[]) object).length == 0)
                || (object instanceof boolean[] && ((boolean[]) object).length == 0)
                || (object instanceof Object[] && ((Object[]) object).length == 0)
                || (object instanceof Map && ((Map<?, ?>) object).isEmpty())
                || (object instanceof Collection && ((Collection<?>) object).isEmpty());
    }

    /**
     * 判断数组是否非空
     *
     * @param array 字节数组
     * @return true/false
     */
    public static boolean notEmpty(byte[] array) {
        return !isEmpty(array);
    }

    /**
     * 判断数组是否非空
     *
     * @param array 短整型数组
     * @return true/false
     */
    public static boolean notEmpty(short[] array) {
        return !isEmpty(array);
    }

    /**
     * 判断数组是否非空
     *
     * @param array 整型数组
     * @return true/false
     */
    public static boolean notEmpty(int[] array) {
        return !isEmpty(array);
    }

    /**
     * 判断数组是否非空
     *
     * @param array 长整型数组
     * @return true/false
     */
    public static boolean notEmpty(long[] array) {
        return !isEmpty(array);
    }

    /**
     * 判断数组是否非空
     *
     * @param array 单精度数组
     * @return true/false
     */
    public static boolean notEmpty(float[] array) {
        return !isEmpty(array);
    }

    /**
     * 判断数组是否非空
     *
     * @param array 双精度数组
     * @return true/false
     */
    public static boolean notEmpty(double[] array) {
        return !isEmpty(array);
    }

    /**
     * 判断数组是否非空
     *
     * @param array 字符数组
     * @return true/false
     */
    public static boolean notEmpty(char[] array) {
        return !isEmpty(array);
    }

    /**
     * 判断数组是否非空
     *
     * @param array 布尔数组
     * @return true/false
     */
    public static boolean notEmpty(boolean[] array) {
        return !isEmpty(array);
    }

    /**
     * 判断数组是否非空
     *
     * @param array 对象数组
     * @param <T>   对象类型
     * @return true/false
     */
    public static <T> boolean notEmpty(T[] array) {
        return !isEmpty(array);
    }

    /**
     * 判断字典是否非空
     *
     * @param map 键/值映射表
     * @param <K> 键类型
     * @param <V> 值类型
     * @return true/false
     */
    public static <K, V> boolean notEmpty(Map<K, V> map) {
        return !isEmpty(map);
    }

    /**
     * 判断集合是否非空
     *
     * @param collection 对象集合
     * @param <T>        对象类型
     * @return true/false
     */
    public static <T> boolean notEmpty(Collection<T> collection) {
        return !isEmpty(collection);
    }

    /**
     * 判断对象是否非空
     *
     * @param object 对象实例
     * @return true/false
     */
    public static boolean notBlank(Object object) {
        return !isBlank(object);
    }

    /**
     * 返回不为空的对象（如果第一个对象为空，则返回第二个对象）
     *
     * @param object 目标对象
     * @param value  替换对象
     * @param <T>    对象类型泛型
     * @return 返回对象
     */
    public static <T> T ifNull(T object, T value) {
        return object == null ? value : object;
    }

    /**
     * 返回不为空的对象（如果第一个对象为空，则返回第二个对象）
     *
     * @param object   目标对象
     * @param supplier 替换对象提供函数
     * @param <T>      对象类型泛型
     * @return 返回对象
     */
    public static <T> T ifNull(T object, @NonNull Supplier<T> supplier) {
        return object == null ? supplier.get() : object;
    }

    /**
     * 返回不为空的对象（如果第一个对象为空，则返回第二个对象）
     *
     * @param object   目标对象
     * @param function 对象处理函数
     * @param <T>      目标对象类型泛型
     * @param <R>      返回对象类型泛型
     * @return 返回对象
     */
    public static <T, R> R ifNull(T object, @NonNull Function<T, R> function) {
        return object == null ? null : function.apply(object);
    }

    /**
     * 返回不为空的对象（如果第一个对象为空，则返回第二个对象）
     *
     * @param object   目标对象
     * @param function 对象处理函数
     * @param supplier 替换对象提供函数
     * @param <T>      目标对象类型泛型
     * @param <R>      返回对象类型泛型
     * @return 返回对象
     */
    public static <T, R> R ifNull(T object, @NonNull Function<T, R> function, @NonNull Supplier<R> supplier) {
        return object == null ? supplier.get() : ifNull(function.apply(object), supplier);
    }

    /**
     * 获取对象实例
     *
     * @param clazz 类对象
     * @param <T>   数据类型
     * @return 对象实例
     */
    public static <T> T instance(@NonNull Class<T> clazz) {
        try {
            return clazz.newInstance();
        } catch (InstantiationException | IllegalAccessException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 获取对象字段
     *
     * @param clazz 对象
     * @return 字段名称/实例映射表
     */
    public static Map<String, Field> getClassFields(@NonNull Class<?> clazz) {
        Map<String, Field> fields = ObjectUtils.ifNull(CLASS_FIELD_MAPPING.get(clazz), WeakReference::get);
        if (fields == null) {
            fields = findClassFields(clazz);
            CLASS_FIELD_MAPPING.put(clazz, new WeakReference<>(fields));
        }
        return fields;
    }

    /**
     * 查找对象字段
     *
     * @param clazz 对象
     * @return 字段名称/实例映射表
     */
    public static Map<String, Field> findClassFields(@NonNull Class<?> clazz) {
        Map<String, Field> fields = Maps.newHashMap();
        findClassField(clazz, field -> {
            fields.put(field.getName(), field);
            return false;
        });
        return fields.isEmpty() ? Collections.emptyMap() : Collections.unmodifiableMap(fields);
    }

    /**
     * 获取对象字段
     *
     * @param clazz 对象
     * @param name  字段名称
     * @return 字段实例
     */
    public static Field getClassField(@NonNull Class<?> clazz, @NonNull String name) {
        Field field = findClassField(clazz, name);
        AssertUtils.check(Objects.nonNull(field), () -> "No such field: " + clazz.getName() + "." + name);
        return field;
    }

    /**
     * 查找对象字段
     *
     * @param clazz 对象
     * @param name  字段名称
     * @return 字段实例
     */
    public static Field findClassField(@NonNull Class<?> clazz, @NonNull String name) {
        return findClassField(clazz, field -> Objects.equals(field.getName(), name));
    }

    /**
     * 查找对象字段
     *
     * @param clazz  对象
     * @param filter 字段过滤函数
     * @return 字段实例
     */
    public static Field findClassField(@NonNull Class<?> clazz, @NonNull Predicate<Field> filter) {
        if (!Modifier.isInterface(clazz.getModifiers())) {
            do {
                Field[] fields = clazz.getDeclaredFields();
                if (notEmpty(fields)) {
                    boolean enumerable = Enum.class.isAssignableFrom(clazz);
                    for (Field field : fields) {
                        if (!field.isSynthetic() && ((enumerable && field.isEnumConstant())
                                || (!enumerable && !Modifier.isStatic(field.getModifiers())))) {
                            field.setAccessible(true);
                            if (filter.test(field)) {
                                return field;
                            }
                        }
                    }
                }
            } while ((clazz = clazz.getSuperclass()) != null && clazz != Enum.class && clazz != Object.class);
        }
        return null;
    }

    /**
     * 查找对象字段
     *
     * @param clazz    对象
     * @param consumer 字段回调函数
     */
    public static void foreachClassFields(@NonNull Class<?> clazz, @NonNull Consumer<Field> consumer) {
        findClassField(clazz, field -> {
            consumer.accept(field);
            return false;
        });
    }

    /**
     * 获取对象字段值
     *
     * @param object 对象实例
     * @param field  字段对象
     * @return 字段值
     */
    public static Object getObjectValue(Object object, @NonNull Field field) {
        if (object == null) {
            return null;
        } else if (!field.isAccessible()) {
            field.setAccessible(true);
        }
        try {
            return field.get(object);
        } catch (IllegalAccessException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 获取对象属性值
     *
     * @param object   对象实例
     * @param property 属性名
     * @return 属性值
     */
    public static Object getObjectValue(Object object, @NonNull String property) {
        return object == null ? null : getObjectValue(object, getClassField(object.getClass(), property));
    }

    /**
     * 设置对象字段值
     *
     * @param object 对象实例
     * @param field  属性字段对象
     * @param value  字段值
     */
    public static void setObjectValue(Object object, @NonNull Field field, Object value) {
        if (object == null) {
            return;
        } else if (!field.isAccessible()) {
            field.setAccessible(true);
        }
        try {
            field.set(object, value);
        } catch (IllegalAccessException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 设置对象属性值
     *
     * @param object   对象实例
     * @param property 属性名
     * @param value    属性值
     */
    public static void setObjectValue(Object object, @NonNull String property, Object value) {
        if (object != null) {
            setObjectValue(object, getClassField(object.getClass(), property), value);
        }
    }

    /**
     * 将对象转换成属性名/值映射表
     *
     * @param object 对象实例
     * @return 对象属性名/值映射表
     */
    public static Map<String, Object> object2map(Object object) {
        return object == null ? null : object2map(object.getClass(), object);
    }

    /**
     * 将对象转换成属性名/值映射表
     *
     * @param target 目标类型
     * @param object 对象实例
     * @return 对象属性名/值映射表
     */
    public static Map<String, Object> object2map(@NonNull Class<?> target, Object object) {
        if (object == null) {
            return null;
        }
        Map<String, Field> targets = getClassFields(target);
        if (isEmpty(targets)) {
            return Collections.emptyMap();
        }
        Class<?> original = object.getClass();
        boolean identical = original == target;
        Map<String, Field> originals = identical ? targets : getClassFields(original);
        if (isEmpty(originals)) {
            return Collections.emptyMap();
        }
        Map<String, Object> map = Maps.newHashMapWithExpectedSize(targets.size());
        targets.forEach((name, field) -> {
            if (identical || (field = originals.get(name)) != null) {
                map.put(name, getObjectValue(object, field));
            }
        });
        return map.isEmpty() ? Collections.emptyMap() : map;
    }

    /**
     * 将属性名/值映射表转换成对象实例
     *
     * @param map  属性名/值映射表
     * @param type 目标对象类型
     * @param <T>  数据类型
     * @return 对象属性名/值映射表
     */
    public static <T> T map2object(Map<String, Object> map, @NonNull Class<T> type) {
        if (map == null) {
            return null;
        }
        T instance = instance(type);
        if (notEmpty(map)) {
            Map<String, Field> fields = getClassFields(type);
            if (notEmpty(fields)) {
                map.forEach((name, value) -> {
                    Field field = fields.get(name);
                    if (field != null) {
                        setObjectValue(instance, field, value);
                    }
                });
            }
        }
        return instance;
    }

    /**
     * 获取对象属性复制处理器工厂
     *
     * @return 对象属性复制处理器工厂
     */
    public static MapperFactory getCopierBuilder() {
        return MAPPER_FACTORY;
    }

    /**
     * 对象属性复制
     *
     * @param source 源对象实例
     * @param <T>    源对象类型
     * @return 目标对象实例
     */
    @SuppressWarnings("unchecked")
    public static <T> T copying(T source) {
        return source == null ? null : (T) copying(source, source.getClass());
    }

    /**
     * 对象属性复制
     *
     * @param source 源对象实例
     * @param target 目标对象实例
     * @param <T>    源对象类型
     * @param <R>    目标对象类型
     * @return 目标对象实例
     */
    public static <T, R> R copying(T source, R target) {
        if (source != null && target != null) {
            getCopierBuilder().getMapperFacade().map(source, target);
        }
        return target;
    }

    /**
     * 对象属性复制
     *
     * @param source 源对象实例
     * @param target 目标对象
     * @param <T>    源对象类型
     * @param <R>    目标对象类型
     * @return 目标对象实例
     */
    public static <T, R> R copying(T source, @NonNull Class<R> target) {
        return getCopierBuilder().getMapperFacade().map(source, target);
    }

    /**
     * 对象属性复制
     *
     * @param sources 源对象实例数组
     * @param target  目标对象
     * @param <T>     源对象类型
     * @param <R>     目标对象类型
     * @return 目标对象实例列表
     */
    public static <T, R> List<R> copying(T[] sources, @NonNull Class<R> target) {
        return isEmpty(sources) ? Collections.emptyList() :
                getCopierBuilder().getMapperFacade().mapAsList(sources, target);
    }

    /**
     * 对象属性复制
     *
     * @param sources 源对象实例列表
     * @param target  目标对象
     * @param <T>     源对象类型
     * @param <R>     目标对象类型
     * @return 目标对象实例列表
     */
    public static <T, R> List<R> copying(Collection<T> sources, @NonNull Class<R> target) {
        return isEmpty(sources) ? Collections.emptyList() :
                getCopierBuilder().getMapperFacade().mapAsList(sources, target);
    }

    /**
     * 判断字典是否为空，如果字典为空则调用适配函数并返回
     *
     * @param map      字典
     * @param supplier 默认字典提供函数
     * @param <K>      健类型
     * @param <V>      值类型
     * @param <M>      字典类型
     * @return 字典
     */
    public static <K, V, M extends Map<K, V>> M ifEmpty(M map, @NonNull Supplier<M> supplier) {
        return isEmpty(map) ? supplier.get() : map;
    }

    /**
     * 判断字典是否为空，如果字典为空则调用适配函数并返回
     *
     * @param map      字典
     * @param function 字典处理函数
     * @param <K>      健类型
     * @param <V>      值类型
     * @param <M>      字典类型
     * @return 字典
     */
    public static <K, V, M extends Map<K, V>> Map<K, V> ifEmpty(M map, @NonNull Function<M, Map<K, V>> function) {
        return ifEmpty(map, function, Collections::emptyMap);
    }

    /**
     * 判断字典是否为空，如果字典为空则调用适配函数并返回
     *
     * @param map      字典
     * @param function 字典处理函数
     * @param supplier 默认字典提供函数
     * @param <K>      健类型
     * @param <V>      值类型
     * @param <R>      目标类型
     * @param <M>      字典类型
     * @return 字典
     */
    public static <K, V, R, M extends Map<K, V>> R ifEmpty(M map, @NonNull Function<M, R> function,
                                                           @NonNull Supplier<R> supplier) {
        return isEmpty(map) ? supplier.get() : ifNull(function.apply(map), supplier);
    }

    /**
     * 判断集合是否为空，如果集合为空则调用适配函数并返回
     *
     * @param collection 对象集合
     * @param supplier   默认集合提供函数
     * @param <T>        对象类型
     * @param <C>        集合类型
     * @return 对象集合
     */
    public static <T, C extends Collection<T>> C ifEmpty(C collection, @NonNull Supplier<C> supplier) {
        return isEmpty(collection) ? supplier.get() : collection;
    }

    /**
     * 判断集合是否为空，如果集合非空则使用适配函数
     *
     * @param collection 对象集合
     * @param function   集合处理函数
     * @param <T>        对象类型
     * @param <R>        目标类型
     * @param <C>        集合类型
     * @return 对象集合
     */
    public static <T, R, C extends Collection<T>> List<R> ifEmpty(C collection,
                                                                  @NonNull Function<C, List<R>> function) {
        return ifEmpty(collection, function, Collections::emptyList);
    }

    /**
     * 判断集合是否为空，如果集合非空则使用适配函数
     *
     * @param collection 对象集合
     * @param function   集合处理函数
     * @param supplier   默认集合提供函数
     * @param <T>        对象类型
     * @param <R>        目标类型
     * @param <C>        集合类型
     * @return 对象集合
     */
    public static <T, R, C extends Collection<T>> R ifEmpty(C collection, @NonNull Function<C, R> function,
                                                            @NonNull Supplier<R> supplier) {
        return isEmpty(collection) ? supplier.get() : ifNull(function.apply(collection), supplier);
    }

    /**
     * 判断数组是否为空，如果数组非空则使用适配函数
     *
     * @param array    对象数组
     * @param function 数组处理函数
     * @param supplier 默认数组提供函数
     * @param <T>      对象类型
     * @param <R>      目标类型
     * @return 对象数组
     */
    public static <T, R> R ifEmpty(T[] array, @NonNull Function<T[], R> function, @NonNull Supplier<R> supplier) {
        return isEmpty(array) ? supplier.get() : ifNull(function.apply(array), supplier);
    }

    /**
     * 合并集合
     *
     * @param collections 集合数组
     * @param <T>         对象类型
     * @return 对象列表
     */
    @SafeVarargs
    public static <T> List<T> merge(Collection<T>... collections) {
        if (collections == null || collections.length == 0) {
            return Collections.emptyList();
        }
        return Stream.of(collections).filter(Objects::nonNull).flatMap(Collection::stream).filter(Objects::nonNull)
                .distinct().collect(Collectors.toList());
    }

    /**
     * 获取Protostuff schema
     *
     * @param clazz 目标对象
     * @param <M>   数据模型类型
     * @param <T>   目标对象类型
     * @return Schema对象实例
     */
    @SuppressWarnings("unchecked")
    private static <M, T extends Schema<M>> T getProtostuffSchema(@NonNull Class<M> clazz) {
        T schema = (T) ObjectUtils.ifNull(PROTOSTUFF_SCHEMA_MAPPING.get(clazz), WeakReference::get);
        if (schema == null) {
            schema = (T) RuntimeSchema.getSchema(clazz);
            PROTOSTUFF_SCHEMA_MAPPING.put(clazz, new WeakReference<>(schema));
        }
        return schema;
    }

    /**
     * 序列化方法，把指定对象序列化成字节数组
     *
     * @param object 被序列化对象实例
     * @return 字节数组
     */
    @SuppressWarnings("unchecked")
    public static <T> byte[] serialize(T object) {
        if (object == null) {
            return null;
        }
        Schema<T> schema = getProtostuffSchema((Class<T>) object.getClass());
        LinkedBuffer buffer = LinkedBuffer.allocate(LinkedBuffer.DEFAULT_BUFFER_SIZE);
        try {
            return ProtobufIOUtil.toByteArray(object, schema, buffer);
        } finally {
            buffer.clear();
        }
    }

    /**
     * 反序列化方法，将字节数组反序列化成指定Class类型
     *
     * @param bytes 字节数组
     * @param clazz 目标对象类型
     * @return 目标对象实例
     */
    public static <T> T deserialize(byte[] bytes, @NonNull Class<T> clazz) {
        if (bytes == null) {
            return null;
        }
        Schema<T> schema = getProtostuffSchema(clazz);
        T object = schema.newMessage();
        ProtobufIOUtil.mergeFrom(bytes, object, schema);
        return object;
    }

    /**
     * 获取对象属性描述列表
     *
     * @param clazz 目标对象
     * @return 属性描述列表
     */
    public static List<PropertyDescriptor> getPropertyDescriptors(@NonNull Class<?> clazz) {
        List<PropertyDescriptor> descriptors =
                ObjectUtils.ifNull(PROPERTY_DESCRIPTOR_MAPPING.get(clazz), WeakReference::get);
        if (descriptors == null) {
            BeanInfo bean;
            try {
                bean = Introspector.getBeanInfo(clazz, Object.class);
            } catch (IntrospectionException e) {
                throw new RuntimeException(e);
            }
            descriptors = Arrays.asList(bean.getPropertyDescriptors());
            PROPERTY_DESCRIPTOR_MAPPING.put(clazz, new WeakReference<>(descriptors));
        }
        return descriptors;
    }

    /**
     * 查找属性对应的Getter方法
     *
     * @param clazz    目标对象
     * @param property 属性名称
     * @return 方法对象
     */
    public static Method lookupPropertyGetter(@NonNull Class<?> clazz, @NonNull String property) {
        return getPropertyDescriptors(clazz).stream().filter(descriptor -> descriptor.getName().equals(property))
                .findAny().map(PropertyDescriptor::getReadMethod).orElse(null);
    }

    /**
     * 查找函数Lambda对象
     *
     * @param function 可序列化函数对象实例
     * @return Lambda对象序列化对象实例
     */
    private static SerializedLambda lookupFunctionLambda(Serializable function) {
        Class<?> clazz = function.getClass();
        SerializedLambda lambda = ObjectUtils.ifNull(LAMBDA_FUNCTION_MAPPING.get(clazz), WeakReference::get);
        if (lambda == null) {
            try {
                Method method = clazz.getDeclaredMethod("writeReplace");
                method.setAccessible(true);
                lambda = (SerializedLambda) method.invoke(function);
            } catch (ReflectiveOperationException e) {
                throw new RuntimeException(e);
            }
            LAMBDA_FUNCTION_MAPPING.put(clazz, new WeakReference<>(lambda));
        }
        return lambda;
    }

    /**
     * 查找Getter函数Lambda对象
     *
     * @param getter Getter函数实例
     * @param <T>    函数调用对象类型
     * @param <R>    函数调用返回类型
     * @return Lambda对象序列化对象实例
     */
    public static <T, R> SerializedLambda lookupGetterLambda(@NonNull Getter<T, R> getter) {
        return lookupFunctionLambda(getter);
    }

    /**
     * 查找Setter函数Lambda对象
     *
     * @param setter Setter函数实例
     * @param <T>    函数调用对象类型
     * @param <U>    函数调用参数类型
     * @return Lambda对象序列化对象实例
     */
    public static <T, U> SerializedLambda lookupSetterLambda(@NonNull Setter<T, U> setter) {
        return lookupFunctionLambda(setter);
    }

    /**
     * 查找Getter函数对应属性名
     *
     * @param getter Getter函数实例
     * @param <T>    函数调用对象类型
     * @param <R>    函数调用返回类型
     * @return 属性名
     */
    public static <T, R> String lookupGetterProperty(@NonNull Getter<T, R> getter) {
        SerializedLambda lambda = lookupGetterLambda(getter);
        String method = lambda.getImplMethodName();
        if (method.startsWith("is")) {
            return StringUtils.hump(method.substring(2));
        } else if (method.startsWith("get")) {
            return StringUtils.hump(method.substring(3));
        }
        throw new RuntimeException("Invalid getter method: " + method);
    }

    /**
     * 查找Setter函数对应属性名
     *
     * @param setter Setter函数实例
     * @param <T>    函数调用对象类型
     * @param <U>    函数调用参数类型
     * @return 属性名
     */
    public static <T, U> String lookupSetterProperty(@NonNull Setter<T, U> setter) {
        SerializedLambda lambda = lookupSetterLambda(setter);
        String method = lambda.getImplMethodName();
        if (method.startsWith("set")) {
            return StringUtils.hump(method.substring(3));
        }
        throw new RuntimeException("Invalid setter method: " + method);
    }

    /**
     * 对象类型转换
     *
     * @param type 对象类型
     * @return 对象类型
     */
    public static Class<?> type2class(Type type) {
        if (type instanceof Class) {
            return (Class<?>) type;
        } else if (type instanceof ParameterizedType) {
            return (Class<?>) ((ParameterizedType) type).getRawType();
        } else if (type instanceof GenericArrayType) {
            Type component = ((GenericArrayType) type).getGenericComponentType();
            if (component instanceof Class) {
                return Array.newInstance((Class<?>) component, 0).getClass();
            }
            Class<?> clazz = type2class(component);
            return Array.newInstance(clazz, 0).getClass();
        }
        return Object.class;
    }

    /**
     * 判断是否是元对象
     *
     * @param clazz 目标对象
     * @return true/false
     */
    public static boolean isMetaClass(Class<?> clazz) {
        return clazz == byte.class || clazz == Byte.class
                || clazz == char.class || clazz == Character.class
                || clazz == short.class || clazz == Short.class
                || clazz == float.class || clazz == Float.class
                || clazz == int.class || clazz == Integer.class
                || clazz == long.class || clazz == Long.class
                || clazz == double.class || clazz == Double.class
                || clazz == boolean.class || clazz == Boolean.class;
    }

    /**
     * 判断对象是否相同
     *
     * @param a 比较对象
     * @param b 比较对象
     * @return true/false
     */
    public static boolean isSameClass(Class<?> a, Class<?> b) {
        if (a == b) {
            return true;
        } else if (a == byte.class || a == Byte.class) {
            return b == byte.class || b == Byte.class;
        } else if (a == char.class || a == Character.class) {
            return b == char.class || b == Character.class;
        } else if (a == short.class || a == Short.class) {
            return b == short.class || b == Short.class;
        } else if (a == float.class || a == Float.class) {
            return b == float.class || b == Float.class;
        } else if (a == int.class || a == Integer.class) {
            return b == int.class || b == Integer.class;
        } else if (a == long.class || a == Long.class) {
            return b == long.class || b == Long.class;
        } else if (a == double.class || a == Double.class) {
            return b == double.class || b == Double.class;
        } else if (a == boolean.class || a == Boolean.class) {
            return b == boolean.class || b == Boolean.class;
        }
        return false;
    }

    /**
     * 判断是否只是Map类型（无有效泛型类型）
     *
     * @param type 对象类型
     * @return true/false
     */
    public static boolean isOnlyMap(Type type) {
        if (type == Map.class) {
            return true;
        } else if (type instanceof ParameterizedType) {
            ParameterizedType parameterized = (ParameterizedType) type;
            if (parameterized.getRawType() == Map.class) {
                Type[] args = parameterized.getActualTypeArguments();
                Type arg1 = args[0], arg2 = args[1];
                return !(arg1 instanceof Class || arg1 instanceof ParameterizedType)
                        && !(arg2 instanceof Class || arg2 instanceof ParameterizedType);
            }
        }
        return false;
    }

    /**
     * 判断是否只是Set类型（无有效泛型类型）
     *
     * @param type 对象类型
     * @return true/false
     */
    public static boolean isOnlySet(Type type) {
        if (type == Set.class) {
            return true;
        } else if (type instanceof ParameterizedType) {
            ParameterizedType parameterized = (ParameterizedType) type;
            if (parameterized.getRawType() == Set.class) {
                Type arg = parameterized.getActualTypeArguments()[0];
                return !(arg instanceof Class || arg instanceof ParameterizedType);
            }
        }
        return false;
    }

    /**
     * 判断是否只是List类型（无有效泛型类型）
     *
     * @param type 对象类型
     * @return true/false
     */
    public static boolean isOnlyList(Type type) {
        if (type == List.class) {
            return true;
        } else if (type instanceof ParameterizedType) {
            ParameterizedType parameterized = (ParameterizedType) type;
            if (parameterized.getRawType() == List.class) {
                Type arg = parameterized.getActualTypeArguments()[0];
                return !(arg instanceof Class || arg instanceof ParameterizedType);
            }
        }
        return false;
    }

    /**
     * 判断是否只是Collection类型（无有效泛型类型）
     *
     * @param type 对象类型
     * @return true/false
     */
    public static boolean isOnlyCollection(Type type) {
        if (type == Collection.class) {
            return true;
        } else if (type instanceof ParameterizedType) {
            ParameterizedType parameterized = (ParameterizedType) type;
            if (parameterized.getRawType() == Collection.class) {
                Type arg = parameterized.getActualTypeArguments()[0];
                return !(arg instanceof Class || arg instanceof ParameterizedType);
            }
        }
        return false;
    }

    /**
     * 对象类型解析
     *
     * @param type 对象类型实例
     * @return 对象类型解析字符串
     */
    public static String analyse(@NonNull Type type) {
        if (type instanceof ParameterizedType) {
            ParameterizedType parameterized = (ParameterizedType) type;
            StringBuilder buffer = new StringBuilder();
            buffer.append(TypeUtils.class.getName()).append(".parameterize(");
            buffer.append(analyse(parameterized.getRawType()));
            Type[] args = parameterized.getActualTypeArguments();
            buffer.append(", new ").append(Type.class.getName()).append("[]{");
            if (notEmpty(args)) {
                for (int i = 0; i < args.length; i++) {
                    if (i > 0) {
                        buffer.append(", ");
                    }
                    buffer.append(analyse(args[i]));
                }
            }
            return buffer.append("})").toString();
        }
        return type.getTypeName() + ".class";
    }

    /**
     * 获取对象所有父类/接口类型
     *
     * @param clazz 对象类型
     * @return 父类/接口类型列表
     */
    public static List<Type> getGenericSuperclasses(@NonNull Class<?> clazz) {
        Type superclass = clazz.getGenericSuperclass();
        Type[] interfaces = clazz.getGenericInterfaces();
        if (superclass == null && isEmpty(interfaces)) {
            return Collections.emptyList();
        } else if (superclass == null) {
            return Arrays.asList(interfaces);
        } else if (isEmpty(interfaces)) {
            return Collections.singletonList(superclass);
        }
        List<Type> superclasses = Lists.newArrayList(interfaces);
        superclasses.add(superclass);
        return superclasses;
    }

    /**
     * 查找对象父类/接口
     *
     * @param clazz      对象类型
     * @param superclass 父类/接口类型
     * @return 父类/接口类型
     */
    public static Type lookupGenericSuperclass(@NonNull Class<?> clazz, @NonNull Class<?> superclass) {
        if (clazz == superclass) {
            return clazz;
        }
        for (Type type : getGenericSuperclasses(clazz)) {
            Class<?> classed = type2class(type);
            if (classed == null) {
                continue;
            } else if (Objects.equals(classed, superclass)) {
                return type;
            }
            Type found = lookupGenericSuperclass(classed, superclass);
            if (found != null) {
                return found;
            }
        }
        return null;
    }

    /**
     * 查找对象父类/接口泛型
     *
     * @param clazz      对象类型
     * @param superclass 父类/接口类型
     * @return 泛型类型
     */
    public static Type lookupSuperclassParameterized(@NonNull Class<?> clazz, @NonNull Class<?> superclass) {
        return lookupSuperclassParameterized(clazz, superclass, null);
    }

    /**
     * 查找对象父类/接口泛型
     *
     * @param clazz       对象类型
     * @param superclass  父类/接口类型
     * @param defaultType 默认类型
     * @return 泛型类型
     */
    public static Type lookupSuperclassParameterized(@NonNull Class<?> clazz, @NonNull Class<?> superclass,
                                                     Type defaultType) {
        Type[] types = lookupSuperclassParameterizes(clazz, superclass);
        return isEmpty(types) ? defaultType : types[0];
    }

    /**
     * 查找对象父类/接口泛型
     *
     * @param clazz      对象类型
     * @param superclass 父类/接口类型
     * @return 泛型类型数组
     */
    public static Type[] lookupSuperclassParameterizes(@NonNull Class<?> clazz, @NonNull Class<?> superclass) {
        for (Type type : getGenericSuperclasses(clazz)) {
            if (Objects.equals(type2class(type), superclass)) {
                if (type instanceof ParameterizedType) {
                    return ((ParameterizedType) type).getActualTypeArguments();
                }
                return new Type[0];
            }
        }
        return new Type[0];
    }

    /**
     * 判断对象是否相同
     *
     * @param object 对象实例
     * @param other  对象实例
     * @return true/false
     */
    public static boolean equals(Object object, Object other) {
        if (Objects.equals(object, other)
                || (object instanceof Map && ((Map<?, ?>) object).isEmpty() && Objects.isNull(other))
                || (other instanceof Map && ((Map<?, ?>) other).isEmpty() && Objects.isNull(object))
                || (object instanceof Object[] && ((Object[]) object).length == 0 && Objects.isNull(other))
                || (other instanceof Object[] && ((Object[]) other).length == 0 && Objects.isNull(object))
                || (object instanceof Collection && ((Collection<?>) object).isEmpty() && Objects.isNull(other))
                || (other instanceof Collection && ((Collection<?>) other).isEmpty() && Objects.isNull(object))) {
            return true;
        } else if (object instanceof Map && other instanceof Map) {
            if (((Map<?, ?>) object).size() != ((Map<?, ?>) other).size()) {
                return false;
            }
            for (Map.Entry<?, ?> entry : ((Map<?, ?>) object).entrySet()) {
                if (!equals(entry.getValue(), ((Map<?, ?>) other).get(entry.getKey()))) {
                    return false;
                }
            }
            for (Map.Entry<?, ?> entry : ((Map<?, ?>) other).entrySet()) {
                if (!equals(entry.getValue(), ((Map<?, ?>) object).get(entry.getKey()))) {
                    return false;
                }
            }
            return true;
        } else if (object instanceof Object[] && other instanceof Object[]) {
            if (((Object[]) object).length != ((Object[]) other).length) {
                return false;
            }
            for (Object value : (Object[]) object) {
                if (Stream.of((Object[]) other).noneMatch(o -> equals(o, value))) {
                    return false;
                }
            }
            for (Object value : (Object[]) other) {
                if (Stream.of((Object[]) object).noneMatch(o -> equals(o, value))) {
                    return false;
                }
            }
            return true;
        } else if (object instanceof Object[] && other instanceof Collection) {
            if (((Object[]) object).length != ((Collection<?>) other).size()) {
                return false;
            }
            for (Object value : (Object[]) object) {
                if (((Collection<?>) other).stream().noneMatch(o -> equals(o, value))) {
                    return false;
                }
            }
            for (Object value : (Collection<?>) other) {
                if (Stream.of((Object[]) object).noneMatch(o -> equals(o, value))) {
                    return false;
                }
            }
            return true;
        } else if (object instanceof Collection && other instanceof Collection) {
            if (((Collection<?>) object).size() != ((Collection<?>) other).size()) {
                return false;
            }
            for (Object value : (Collection<?>) object) {
                if (((Collection<?>) other).stream().noneMatch(o -> equals(o, value))) {
                    return false;
                }
            }
            for (Object value : (Collection<?>) other) {
                if (((Collection<?>) object).stream().noneMatch(o -> equals(o, value))) {
                    return false;
                }
            }
            return true;
        } else if (object instanceof Collection && other instanceof Object[]) {
            if (((Collection<?>) object).size() != ((Object[]) other).length) {
                return false;
            }
            for (Object value : (Collection<?>) object) {
                if (Stream.of((Object[]) other).noneMatch(o -> equals(o, value))) {
                    return false;
                }
            }
            for (Object value : (Object[]) other) {
                if (((Collection<?>) object).stream().noneMatch(o -> equals(o, value))) {
                    return false;
                }
            }
            return true;
        }
        return false;
    }

    /**
     * 获取两个对象实例不同属性值
     *
     * @param <T>    数据类型
     * @param object 对象实例
     * @param other  对象实例
     * @return 不同属性值
     */
    public static <T> List<Modification> compare(T object, T other) {
        return compare(object, other, field -> true);
    }

    /**
     * 获取两个对象实例不同属性值
     *
     * @param <T>    数据类型
     * @param object 对象实例
     * @param other  对象实例
     * @param filter 字段过滤函数
     * @return 不同属性值
     */
    public static <T> List<Modification> compare(T object, T other, @NonNull Predicate<Field> filter) {
        if (object == null || other == null) {
            return Collections.emptyList();
        }
        Map<String, Field> fields = getClassFields(object.getClass());
        if (isEmpty(fields)) {
            return Collections.emptyList();
        }
        return fields.entrySet().stream().filter(entry -> filter.test(entry.getValue())).map(entry -> {
            String name = entry.getKey();
            Field field = entry.getValue();
            Object before = getObjectValue(object, field);
            Object after = getObjectValue(other, field);
            return equals(before, after) ? null : Modification.builder().name(name).before(before).after(after).build();
        }).filter(Objects::nonNull).collect(Collectors.toList());
    }

    /**
     * 循环获取对象
     *
     * @param supplier 对象提供函数
     * @param filter   对象过滤函数（返回false则中断循环）
     * @param <T>      对象类型
     * @return 对象实例
     */
    public static <T> T loop(@NonNull Supplier<T> supplier, @NonNull BiPredicate<Integer, T> filter) {
        T t;
        int count = 1;
        do {
            t = supplier.get();
        } while (filter.test(count++, t));
        return t;
    }

    /**
     * 数字分组
     *
     * @param total 总数
     * @param size  分组大小
     * @return 分组数量
     */
    public static int group(int total, int size) {
        if (size < 1 || size >= total) {
            return 1;
        }
        return total % size == 0 ? total / size : total / size + 1;
    }

    /**
     * 对象列表拆分
     *
     * @param objects 对象列表
     * @param size    分组大小
     * @param <T>     对象类型
     * @return 对象分组列表
     */
    public static <T> List<List<T>> split(List<T> objects, int size) {
        return isEmpty(objects) ? Collections.emptyList() : group(objects, group(objects.size(), size));
    }

    /**
     * 对象列表拆分
     *
     * @param objects  对象列表
     * @param size     分组大小
     * @param consumer 对象分组处理器
     * @param <T>      对象类型
     */
    public static <T> void split(List<T> objects, int size, @NonNull BiConsumer<Integer, List<T>> consumer) {
        if (notEmpty(objects)) {
            group(objects, group(objects.size(), size), consumer);
        }
    }

    /**
     * 对象列表分组
     *
     * @param objects 对象列表
     * @param group   分组数量
     * @param <T>     对象类型
     * @return 对象分组列表
     */
    public static <T> List<List<T>> group(List<T> objects, int group) {
        if (isEmpty(objects)) {
            return Collections.emptyList();
        } else if (group < 2) {
            return Collections.singletonList(objects);
        }
        List<List<T>> groups = Lists.newArrayListWithCapacity(group);
        group(objects, group, (index, list) -> groups.add(list));
        return groups;
    }

    /**
     * 对象列表分组
     *
     * @param objects  对象列表
     * @param group    分组数量
     * @param consumer 对象分组处理器
     * @param <T>      对象类型
     */
    public static <T> void group(List<T> objects, int group, @NonNull BiConsumer<Integer, List<T>> consumer) {
        if (isEmpty(objects)) {
            return;
        }
        if (group < 2) {
            consumer.accept(0, objects);
        } else {
            int total = objects.size(), size = group(total, group);
            for (int i = 0; i < group; i++) {
                consumer.accept(i, objects.subList(i * size, Math.min((i + 1) * size, total)));
            }
        }
    }

    /**
     * 获取从缓存中获取字节数组
     *
     * @param buffer 直接缓存
     * @return 字节数组
     */
    public static byte[] getBytes(ByteBuffer buffer) {
        if (buffer == null) {
            return null;
        }
        byte[] bytes = new byte[buffer.remaining()];
        buffer.get(bytes, 0, bytes.length);
        return bytes;
    }

    /**
     * 将对象转换为对象流
     *
     * @param object 对象实例
     * @return 对象流实例
     */
    public static Stream<?> stream(Object object) {
        if (object instanceof Stream) {
            return (Stream<?>) object;
        } else if (object instanceof Object[]) {
            return Stream.of((Object[]) object);
        } else if (object instanceof Collection) {
            return ((Collection<?>) object).stream();
        }
        return Stream.of(object);
    }

    /**
     * 将对象转换为集合
     *
     * @param object 对象实例
     * @return 对象集合
     */
    public static Collection<?> collection(Object object) {
        if (object instanceof Stream) {
            return ((Stream<?>) object).collect(Collectors.toList());
        } else if (object instanceof Object[]) {
            return Arrays.asList((Object[]) object);
        } else if (object instanceof Collection) {
            return (Collection<?>) object;
        }
        return Collections.singletonList(object);
    }

    /**
     * 获取字符串长度
     *
     * @param string 字符序列对象实例
     * @return 字符串长度
     */
    public static int size(CharSequence string) {
        return string == null ? 0 : string.length();
    }

    /**
     * 获取字典元素个数
     *
     * @param map 字典对象实例
     * @return 元素个数
     */
    public static int size(Map<?, ?> map) {
        return map == null ? 0 : map.size();
    }

    /**
     * 获取集合元素个数
     *
     * @param collection 集合对象实例
     * @return 元素个数
     */
    public static int size(Collection<?> collection) {
        return collection == null ? 0 : collection.size();
    }

    /**
     * 获取列表指定下标元素
     *
     * @param list  元素列表
     * @param index 元素下标
     * @param <T>   元素类型
     * @return 元素
     */
    @SuppressWarnings("unchecked")
    public static <T> T get(List<?> list, int index) {
        return (T) get(list, index, value -> value);
    }

    /**
     * 获取列表指定下标元素
     *
     * @param list     元素列表
     * @param index    元素下标
     * @param function 对象转换回调函数
     * @param <T>      元素类型
     * @param <R>      返回类型
     * @return 元素
     */
    public static <T, R> R get(List<T> list, int index, @NonNull Function<T, R> function) {
        AssertUtils.check(index > -1, () -> "index must be greater than -1");
        return size(list) > index ? ifNull(list.get(index), function) : null;
    }
}