package com.chelvc.framework.common.util;

import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Queue;
import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.TimeUnit;
import java.util.function.Consumer;
import java.util.function.Predicate;
import java.util.stream.Collectors;
import java.util.stream.IntStream;

import com.google.common.collect.Lists;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;

/**
 * 线程工具类
 *
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
public final class ThreadUtils {
    private ThreadUtils() {
    }

    /**
     * 休眠线程
     *
     * @param millis 毫秒数
     * @return true/false
     */
    public static boolean sleep(long millis) {
        try {
            Thread.sleep(millis);
        } catch (InterruptedException e) {
            Thread.currentThread().interrupt();
            return false;
        }
        return true;
    }

    /**
     * 异步调用方法
     *
     * @param runnable 回调接口
     * @return 线程对象
     */
    public static Thread run(@NonNull Runnable runnable) {
        Thread thread = new Thread(runnable);
        thread.start();
        return thread;
    }

    /**
     * 异步调用方法
     *
     * @param runs 回调接口数组
     * @return 线程对象数组
     */
    public static Thread[] run(@NonNull Runnable... runs) {
        if (runs.length == 0) {
            return new Thread[0];
        }
        Thread[] threads = new Thread[runs.length];
        for (int i = 0; i < runs.length; i++) {
            threads[i] = run(runs[i]);
        }
        return threads;
    }

    /**
     * 阻塞线程
     *
     * @param threads 线程对象数组
     */
    public static void join(@NonNull Thread... threads) {
        for (Thread thread : threads) {
            try {
                thread.join();
            } catch (InterruptedException e) {
                Thread.currentThread().interrupt();
                break;
            }
        }
    }

    /**
     * 等待线程执行完成
     *
     * @param threads 线程对象集合
     */
    public static void join(@NonNull Collection<Thread> threads) {
        for (Thread thread : threads) {
            try {
                thread.join();
            } catch (InterruptedException e) {
                Thread.currentThread().interrupt();
                break;
            }
        }
    }

    /**
     * 停止线程
     *
     * @param threads 线程对象数组
     */
    public static void stop(@NonNull Thread... threads) {
        interrupt(threads);
        join(threads);
    }

    /**
     * 停止线程
     *
     * @param threads 线程对象集合
     */
    public static void stop(@NonNull Collection<Thread> threads) {
        interrupt(threads);
        join(threads);
    }

    /**
     * 中断线程
     *
     * @param threads 线程对象数组
     */
    public static void interrupt(@NonNull Thread... threads) {
        for (Thread thread : threads) {
            thread.interrupt();
        }
    }

    /**
     * 中断线程
     *
     * @param threads 线程对象集合
     */
    public static void interrupt(@NonNull Collection<Thread> threads) {
        for (Thread thread : threads) {
            thread.interrupt();
        }
    }

    /**
     * 异步消费消息
     *
     * @param queue    消息队列
     * @param consumer 消息消费者
     * @param <T>      消息类型
     * @return 消费线程
     */
    public static <T> Thread consume(@NonNull Queue<T> queue, @NonNull Consumer<T> consumer) {
        return run(() -> {
            while (!Thread.currentThread().isInterrupted()) {
                // 从队列中获取消息
                T message = queue.poll();
                if (message == null) {
                    sleep(1000);
                    continue;
                }

                // 消费消息
                try {
                    consumer.accept(message);
                } catch (Exception e) {
                    log.error(e.getMessage(), e);
                }
            }
        });
    }

    /**
     * 异步消费消息
     *
     * @param queue    消息队列
     * @param consumer 消息消费者
     * @param <T>      消息类型
     * @return 消费线程
     */
    public static <T> Thread consume(@NonNull Queue<T> queue, @NonNull Predicate<T> consumer) {
        return run(() -> {
            while (!Thread.currentThread().isInterrupted()) {
                // 从队列中获取消息
                T message = queue.peek();
                if (message == null) {
                    sleep(1000);
                    continue;
                }

                // 消费消息
                try {
                    if (consumer.test(message)) {
                        // 处理成功后移除消息
                        queue.remove();
                    }
                } catch (Exception e) {
                    log.error(e.getMessage(), e);
                }
            }
        });
    }

    /**
     * 异步消费消息
     *
     * @param queue    消息队列
     * @param consumer 消息消费者
     * @param <T>      消息类型
     * @return 消费线程列表
     */
    public static <T> List<Thread> consume(@NonNull BlockingQueue<T> queue, @NonNull Consumer<T> consumer) {
        return consume(queue, consumer, 3);
    }

    /**
     * 异步消费消息
     *
     * @param queue       消息队列
     * @param consumer    消息消费者
     * @param concurrency 并发数量
     * @param <T>         消息类型
     * @return 消费线程列表
     */
    public static <T> List<Thread> consume(@NonNull BlockingQueue<T> queue, @NonNull Consumer<T> consumer,
                                           int concurrency) {
        return IntStream.range(0, concurrency).mapToObj(i -> run(() -> {
            while (!Thread.currentThread().isInterrupted()) {
                // 从队列中获取消息
                T message = null;
                try {
                    message = queue.poll(1, TimeUnit.SECONDS);
                } catch (InterruptedException e) {
                    Thread.currentThread().interrupt();
                }

                // 消费消息
                if (message != null) {
                    try {
                        consumer.accept(message);
                    } catch (Exception e) {
                        log.error(e.getMessage(), e);
                    }
                }
            }
        })).collect(Collectors.toList());
    }

    /**
     * 异步批量消费消息
     *
     * @param queue       消息队列
     * @param consumer    消息消费者
     * @param concurrency 并发数量
     * @param batchSize   批处理数量
     * @param <T>         消息类型
     * @return 消费线程列表
     */
    public static <T> List<Thread> consume(@NonNull BlockingQueue<T> queue, @NonNull Consumer<List<T>> consumer,
                                           int concurrency, int batchSize) {
        return IntStream.range(0, concurrency).mapToObj(i -> run(() -> {
            List<T> buffer = Lists.newArrayListWithCapacity(batchSize);
            List<T> unmodifiable = Collections.unmodifiableList(buffer);
            while (!Thread.currentThread().isInterrupted()) {
                // 从队列中获取消息
                T message = null;
                try {
                    message = queue.poll(3, TimeUnit.SECONDS);
                } catch (InterruptedException e) {
                    Thread.currentThread().interrupt();
                }

                // 将消息加入缓冲区
                if (message != null) {
                    buffer.add(message);
                }

                // 如果获取消息超时或缓冲区消息数量达到batchSize则执行消费
                if (buffer.size() > 0 && (message == null || buffer.size() % batchSize == 0)) {
                    try {
                        consumer.accept(unmodifiable);
                    } catch (Exception e) {
                        log.error(e.getMessage(), e);
                    } finally {
                        buffer.clear();
                    }
                }
            }
            if (ObjectUtils.notEmpty(buffer)) {
                consumer.accept(unmodifiable);
            }
        })).collect(Collectors.toList());
    }

    /**
     * 异步消费消息
     *
     * @param consumer 消息消费者
     * @param <T>      消息类型
     * @return 消息队列
     */
    public static <T> BlockingQueue<T> queue(@NonNull Consumer<T> consumer) {
        return queue(1000, consumer);
    }

    /**
     * 异步消费消息
     *
     * @param consumer 消息消费者
     * @param <T>      消息类型
     * @return 消息队列
     */
    public static <T> BlockingQueue<T> queue(@NonNull Predicate<T> consumer) {
        return queue(1000, consumer);
    }

    /**
     * 异步消费消息
     *
     * @param capacity 队列大小
     * @param consumer 消息消费者
     * @param <T>      消息类型
     * @return 消息队列
     */
    public static <T> BlockingQueue<T> queue(int capacity, @NonNull Consumer<T> consumer) {
        return queue(capacity, consumer, 3);
    }

    /**
     * 异步消费消息
     *
     * @param consumer    消息消费者
     * @param concurrency 并发数量
     * @param <T>         消息类型
     * @return 消息队列
     */
    public static <T> BlockingQueue<T> queue(@NonNull Consumer<T> consumer, int concurrency) {
        return queue(1000, consumer, concurrency);
    }

    /**
     * 异步消费消息
     *
     * @param capacity 队列大小
     * @param consumer 消息消费者
     * @param <T>      消息类型
     * @return 消息队列
     */
    public static <T> BlockingQueue<T> queue(int capacity, @NonNull Predicate<T> consumer) {
        BlockingQueue<T> queue = new ArrayBlockingQueue<>(Math.max(capacity, 1));
        consume(queue, consumer);
        return queue;
    }

    /**
     * 异步消费消息
     *
     * @param capacity    队列大小
     * @param consumer    消息消费者
     * @param concurrency 并发数量
     * @param <T>         消息类型
     * @return 消息队列
     */
    public static <T> BlockingQueue<T> queue(int capacity, @NonNull Consumer<T> consumer, int concurrency) {
        BlockingQueue<T> queue = new ArrayBlockingQueue<>(Math.max(capacity, 1));
        consume(queue, consumer, concurrency);
        return queue;
    }

    /**
     * 异步消费消息
     *
     * @param consumer    消息消费者
     * @param concurrency 并发数量
     * @param batchSize   批处理数量
     * @param <T>         消息类型
     * @return 消息队列
     */
    public static <T> BlockingQueue<T> queue(@NonNull Consumer<List<T>> consumer, int concurrency, int batchSize) {
        return queue(1000, consumer, concurrency, batchSize);
    }

    /**
     * 异步批量消费消息
     *
     * @param capacity    队列大小
     * @param consumer    消息消费者
     * @param concurrency 并发数量
     * @param batchSize   批处理数量
     * @param <T>         消息类型
     * @return 消息队列
     */
    public static <T> BlockingQueue<T> queue(int capacity, @NonNull Consumer<List<T>> consumer, int concurrency,
                                             int batchSize) {
        BlockingQueue<T> queue = new ArrayBlockingQueue<>(Math.max(capacity, 1));
        consume(queue, consumer, concurrency, batchSize);
        return queue;
    }
}