package com.chelvc.framework.database.config;

import java.time.Duration;
import java.util.function.Supplier;

import com.baomidou.mybatisplus.autoconfigure.ConfigurationCustomizer;
import com.baomidou.mybatisplus.core.injector.ISqlInjector;
import com.baomidou.mybatisplus.extension.plugins.MybatisPlusInterceptor;
import com.baomidou.mybatisplus.extension.plugins.inner.InnerInterceptor;
import com.baomidou.mybatisplus.extension.plugins.inner.PaginationInnerInterceptor;
import com.baomidou.mybatisplus.extension.plugins.inner.TenantLineInnerInterceptor;
import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.common.function.Executor;
import com.chelvc.framework.common.function.Handler;
import com.chelvc.framework.common.function.Provider;
import com.chelvc.framework.common.model.File;
import com.chelvc.framework.common.model.Modification;
import com.chelvc.framework.common.model.Period;
import com.chelvc.framework.common.model.Range;
import com.chelvc.framework.common.model.Region;
import com.chelvc.framework.database.context.Transactor;
import com.chelvc.framework.database.crypto.CacheableDatabaseCipherHandler;
import com.chelvc.framework.database.crypto.DatabaseCipherHandler;
import com.chelvc.framework.database.crypto.DefaultDatabaseCipherHandler;
import com.chelvc.framework.database.handler.FileTypeHandler;
import com.chelvc.framework.database.handler.ModificationTypeHandler;
import com.chelvc.framework.database.handler.PeriodTypeHandler;
import com.chelvc.framework.database.handler.RangeTypeHandler;
import com.chelvc.framework.database.handler.RegionTypeHandler;
import com.chelvc.framework.database.interceptor.DeletedIsolateInterceptor;
import com.chelvc.framework.database.interceptor.EnvIsolateInterceptor;
import com.chelvc.framework.database.support.EnhanceSqlInjector;
import lombok.NonNull;
import org.apache.ibatis.type.JdbcType;
import org.apache.ibatis.type.TypeHandlerRegistry;
import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingBean;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.context.ApplicationContext;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.transaction.annotation.Transactional;

/**
 * 数据库配置
 *
 * @author Woody
 * @date 2024/1/30
 */
@Configuration
public class DatabaseConfigurer {
    @Bean
    @ConditionalOnMissingBean(Transactor.class)
    public Transactor transactor() {
        return new Transactor() {
            @Override
            @Transactional(rollbackFor = Exception.class)
            public void execute(@NonNull Executor executor) {
                executor.execute();
            }

            @Override
            @Transactional(rollbackFor = Exception.class)
            public <T> T execute(@NonNull Supplier<T> executor) {
                return executor.get();
            }

            @Override
            @Transactional(rollbackFor = Exception.class)
            public void execute(@NonNull Handler executor) throws Exception {
                executor.handle();
            }

            @Override
            @Transactional(rollbackFor = Exception.class)
            public <T> T execute(@NonNull Provider<T> executor) throws Exception {
                return executor.get();
            }
        };
    }

    @Bean
    public ISqlInjector sqlInjector() {
        return new EnhanceSqlInjector();
    }

    @Bean
    public ConfigurationCustomizer configurationCustomizer() {
        return configuration -> {
            // 初始化Mybatis配置拦截器
            MybatisConfigurer.initialize(configuration);

            // 注册自定义字段类型处理器
            TypeHandlerRegistry handlerRegistry = configuration.getTypeHandlerRegistry();
            handlerRegistry.register(File.class, JdbcType.VARCHAR, new FileTypeHandler());
            handlerRegistry.register(Range.class, JdbcType.VARCHAR, new RangeTypeHandler());
            handlerRegistry.register(Period.class, JdbcType.VARCHAR, new PeriodTypeHandler());
            handlerRegistry.register(Region.class, JdbcType.VARCHAR, new RegionTypeHandler());
            handlerRegistry.register(Modification.class, JdbcType.VARCHAR, new ModificationTypeHandler());
        };
    }

    @Bean
    @ConditionalOnProperty(name = "database.crypto.secret")
    @ConditionalOnMissingBean(DatabaseCipherHandler.class)
    public DatabaseCipherHandler databaseCipherHandler(DatabaseProperties properties) {
        DatabaseProperties.Crypto crypto = properties.getCrypto();
        DatabaseCipherHandler handler = new DefaultDatabaseCipherHandler(crypto);
        if (crypto.getCache().isEnabled()) {
            Duration duration = Duration.ofSeconds(crypto.getCache().getDuration());
            return new CacheableDatabaseCipherHandler(handler, crypto.getCache().getCapacity(), duration);
        }
        return handler;
    }

    @Bean
    public MybatisPlusInterceptor mybatisPlusInterceptor(ApplicationContext applicationContext) {
        MybatisPlusInterceptor interceptor = new MybatisPlusInterceptor();

        // 设置自定义租户隔离拦截器
        ApplicationContextHolder.getOrderBeans(applicationContext, InnerInterceptor.class)
                .forEach(interceptor::addInnerInterceptor);

        // 设置数据环境隔离过滤拦截器
        interceptor.addInnerInterceptor(new TenantLineInnerInterceptor() {
            {
                this.setTenantLineHandler(new EnvIsolateInterceptor());
            }
        });

        // 设置数据逻辑删除过滤拦截器
        interceptor.addInnerInterceptor(new TenantLineInnerInterceptor() {
            {
                this.setTenantLineHandler(new DeletedIsolateInterceptor());
            }
        });

        // 设置分页拦截器，数据过滤条件与拦截器设置顺序有关，分页拦截器最后设置才能在分页时同时使用数据隔离过滤条件
        interceptor.addInnerInterceptor(new PaginationInnerInterceptor());
        return interceptor;
    }
}