package com.chelvc.framework.database.config;

import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.sql.CallableStatement;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Set;
import java.util.function.Consumer;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.core.metadata.TableFieldInfo;
import com.baomidou.mybatisplus.core.metadata.TableInfo;
import com.baomidou.mybatisplus.extension.handlers.JacksonTypeHandler;
import com.chelvc.framework.common.model.File;
import com.chelvc.framework.common.model.Modification;
import com.chelvc.framework.common.model.Period;
import com.chelvc.framework.common.model.Range;
import com.chelvc.framework.common.model.Region;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.database.annotation.Sensitive;
import com.chelvc.framework.database.context.DatabaseContextHolder;
import com.chelvc.framework.database.handler.BooleanArrayTypeHandler;
import com.chelvc.framework.database.handler.BooleanSetTypeHandler;
import com.chelvc.framework.database.handler.BooleansTypeHandler;
import com.chelvc.framework.database.handler.DoubleArrayTypeHandler;
import com.chelvc.framework.database.handler.DoubleSetTypeHandler;
import com.chelvc.framework.database.handler.DoublesTypeHandler;
import com.chelvc.framework.database.handler.FileArrayTypeHandler;
import com.chelvc.framework.database.handler.FileSetTypeHandler;
import com.chelvc.framework.database.handler.FilesTypeHandler;
import com.chelvc.framework.database.handler.FloatArrayTypeHandler;
import com.chelvc.framework.database.handler.FloatSetTypeHandler;
import com.chelvc.framework.database.handler.FloatsTypeHandler;
import com.chelvc.framework.database.handler.IntegerArrayTypeHandler;
import com.chelvc.framework.database.handler.IntegerSetTypeHandler;
import com.chelvc.framework.database.handler.IntegersTypeHandler;
import com.chelvc.framework.database.handler.JsonTypeHandler;
import com.chelvc.framework.database.handler.ListTypeHandler;
import com.chelvc.framework.database.handler.ListsTypeHandler;
import com.chelvc.framework.database.handler.LongArrayTypeHandler;
import com.chelvc.framework.database.handler.LongSetTypeHandler;
import com.chelvc.framework.database.handler.LongsTypeHandler;
import com.chelvc.framework.database.handler.MapTypeHandler;
import com.chelvc.framework.database.handler.MapsTypeHandler;
import com.chelvc.framework.database.handler.ModificationArrayTypeHandler;
import com.chelvc.framework.database.handler.ModificationSetTypeHandler;
import com.chelvc.framework.database.handler.ModificationsTypeHandler;
import com.chelvc.framework.database.handler.PeriodArrayTypeHandler;
import com.chelvc.framework.database.handler.PeriodSetTypeHandler;
import com.chelvc.framework.database.handler.PeriodsTypeHandler;
import com.chelvc.framework.database.handler.RangeArrayTypeHandler;
import com.chelvc.framework.database.handler.RangeSetTypeHandler;
import com.chelvc.framework.database.handler.RangesTypeHandler;
import com.chelvc.framework.database.handler.RegionArrayTypeHandler;
import com.chelvc.framework.database.handler.RegionSetTypeHandler;
import com.chelvc.framework.database.handler.RegionsTypeHandler;
import com.chelvc.framework.database.handler.SensitiveArrayTypeHandler;
import com.chelvc.framework.database.handler.SensitiveListTypeHandler;
import com.chelvc.framework.database.handler.SensitiveSetTypeHandler;
import com.chelvc.framework.database.handler.SensitiveTypeHandler;
import com.chelvc.framework.database.handler.SetTypeHandler;
import com.chelvc.framework.database.handler.SetsTypeHandler;
import com.chelvc.framework.database.handler.ShortArrayTypeHandler;
import com.chelvc.framework.database.handler.ShortSetTypeHandler;
import com.chelvc.framework.database.handler.ShortsTypeHandler;
import com.chelvc.framework.database.handler.StringArrayTypeHandler;
import com.chelvc.framework.database.handler.StringSetTypeHandler;
import com.chelvc.framework.database.handler.StringsTypeHandler;
import com.chelvc.framework.database.sql.CallableStringDecrypter;
import com.chelvc.framework.database.sql.ResultStringDecrypter;
import com.chelvc.framework.database.sql.WriteStringEncryptor;
import com.google.common.collect.Maps;
import com.google.common.collect.Sets;
import javassist.CannotCompileException;
import javassist.ClassPool;
import javassist.CtClass;
import javassist.CtConstructor;
import javassist.CtNewMethod;
import javassist.NotFoundException;
import lombok.extern.slf4j.Slf4j;
import org.apache.ibatis.parsing.XNode;
import org.apache.ibatis.reflection.TypeParameterResolver;
import org.apache.ibatis.session.Configuration;
import org.apache.ibatis.type.JdbcType;
import org.apache.ibatis.type.TypeAliasRegistry;
import org.apache.ibatis.type.TypeHandler;
import org.apache.ibatis.type.UnknownTypeHandler;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * TypeHandler配置类
 *
 * @author Woody
 * @date 2024/4/6
 */
@Slf4j
public class TypeHandlerConfigurer extends MybatisConfigurer {
    /**
     * 内置的别名/对象类型映射表
     */
    private final Map<String, Class<?>> aliases = new TypeAliasRegistry().getTypeAliases();

    /**
     * JSON类型/处理器映射表
     */
    private final Map<Type, Class<? extends TypeHandler<?>>> handlers = Maps.newConcurrentMap();

    public TypeHandlerConfigurer(Configuration configuration) {
        super(configuration);
    }

    /**
     * 将对象名称转换成类对象
     *
     * @param name 对象名称
     * @return 类对象
     */
    private Class<?> name2class(String name) {
        if (StringUtils.notEmpty(name)) {
            try {
                return Class.forName(name);
            } catch (Exception e) {
                log.error("Class load failed: {}", name, e);
            }
        }
        return null;
    }

    /**
     * 将对象别名转换成类对象
     *
     * @param alias 对象别名
     * @return 类对象
     */
    private Class<?> alias2class(String alias) {
        if (StringUtils.notEmpty(alias)) {
            Class<?> clazz = this.aliases.get(alias.toLowerCase());
            if (clazz != null || (clazz = this.name2class(alias)) != null) {
                return clazz;
            }
        }
        return null;
    }

    /**
     * 判断对象类型是否是元类型
     *
     * @param type 对象类型
     * @return true/false
     */
    private boolean isMetaType(Class<?> type) {
        return DatabaseContextHolder.isTypeHandlerRegistered(type) || Map.class.isAssignableFrom(type)
                || Collection.class.isAssignableFrom(type);
    }

    /**
     * 判断对象是否可配置类型处理器
     *
     * @param clazz 对象类型
     * @return true/false
     */
    private boolean isTypeHandlerConfigurable(Class<?> clazz) {
        return !this.isMetaType(clazz) && Objects.nonNull(ObjectUtils.findClassField(clazz, field -> {
            TableField annotation = field.getAnnotation(TableField.class);
            if (annotation != null && !annotation.exist()) {
                return false;
            } else if (field.isAnnotationPresent(Sensitive.class)) {
                // 敏感字段
                return true;
            }

            // JSON字段
            Type type = TypeParameterResolver.resolveFieldType(field, clazz);
            return !DatabaseContextHolder.isTypeHandlerRegistered(ObjectUtils.type2class(type));
        }));
    }

    /**
     * 绑定字段类型处理器，成功绑定返回true，否则返回false
     *
     * @param clazz 对象类型
     * @param field 表字段信息
     * @return true/false
     */
    private boolean bindTypeHandler(Class<?> clazz, TableFieldInfo field) {
        // 查找字段类型处理器
        Class<? extends TypeHandler<?>> handler =
                this.lookupTypeHandlerClass(clazz, field.getField(), field.getTypeHandler());
        if (handler == null) {
            return false;
        }

        // 绑定并注册字段类型处理器
        ObjectUtils.setObjectValue(field, "typeHandler", handler);
        String el = (String) ObjectUtils.getObjectValue(field, "el");
        int delimiter = el.indexOf(',');
        if (delimiter > 0) {
            el = el.substring(0, delimiter);
        }
        el = el + ",typeHandler=" + handler.getName();
        ObjectUtils.setObjectValue(field, "el", el);
        return true;
    }

    /**
     * 查找字段JSON类型处理器
     *
     * @param type 对象类型
     * @return 类型处理器对象
     */
    private Class<? extends TypeHandler<?>> lookupJsonHandlerClass(Type type) {
        if (ObjectUtils.isOnlyMap(type)) {
            return MapTypeHandler.class;
        } else if (ObjectUtils.isOnlySet(type)) {
            return SetTypeHandler.class;
        } else if (ObjectUtils.isOnlyList(type)) {
            return ListTypeHandler.class;
        } else if (type instanceof ParameterizedType) {
            Type raw = ((ParameterizedType) type).getRawType();
            Type arg = ((ParameterizedType) type).getActualTypeArguments()[0];
            if (raw == Set.class) {
                if (arg == int.class || arg == Integer.class) {
                    return IntegerSetTypeHandler.class;
                } else if (arg == short.class || arg == Short.class) {
                    return ShortSetTypeHandler.class;
                } else if (arg == long.class || arg == Long.class) {
                    return LongSetTypeHandler.class;
                } else if (arg == float.class || arg == Float.class) {
                    return FloatSetTypeHandler.class;
                } else if (arg == double.class || arg == Double.class) {
                    return DoubleSetTypeHandler.class;
                } else if (arg == boolean.class || arg == Boolean.class) {
                    return BooleanSetTypeHandler.class;
                } else if (arg == String.class) {
                    return StringSetTypeHandler.class;
                } else if (arg == File.class) {
                    return FileSetTypeHandler.class;
                } else if (arg == Range.class) {
                    return RangeSetTypeHandler.class;
                } else if (arg == Period.class) {
                    return PeriodSetTypeHandler.class;
                } else if (arg == Region.class) {
                    return RegionSetTypeHandler.class;
                } else if (arg == Modification.class) {
                    return ModificationSetTypeHandler.class;
                }
            } else if (raw == List.class) {
                if (arg == int.class || arg == Integer.class) {
                    return IntegersTypeHandler.class;
                } else if (arg == short.class || arg == Short.class) {
                    return ShortsTypeHandler.class;
                } else if (arg == long.class || arg == Long.class) {
                    return LongsTypeHandler.class;
                } else if (arg == float.class || arg == Float.class) {
                    return FloatsTypeHandler.class;
                } else if (arg == double.class || arg == Double.class) {
                    return DoublesTypeHandler.class;
                } else if (arg == boolean.class || arg == Boolean.class) {
                    return BooleansTypeHandler.class;
                } else if (arg == String.class) {
                    return StringsTypeHandler.class;
                } else if (arg == File.class) {
                    return FilesTypeHandler.class;
                } else if (arg == Range.class) {
                    return RangesTypeHandler.class;
                } else if (arg == Period.class) {
                    return PeriodsTypeHandler.class;
                } else if (arg == Region.class) {
                    return RegionsTypeHandler.class;
                } else if (arg == Modification.class) {
                    return ModificationsTypeHandler.class;
                } else if (ObjectUtils.isOnlyMap(arg)) {
                    return MapsTypeHandler.class;
                } else if (ObjectUtils.isOnlySet(arg)) {
                    return SetsTypeHandler.class;
                } else if (ObjectUtils.isOnlyList(arg)) {
                    return ListsTypeHandler.class;
                }
            }
        } else if (type instanceof Class && ((Class<?>) type).isArray()) {
            Class<?> component = ((Class<?>) type).getComponentType();
            if (component == int.class || component == Integer.class) {
                return IntegerArrayTypeHandler.class;
            } else if (component == short.class || component == Short.class) {
                return ShortArrayTypeHandler.class;
            } else if (component == long.class || component == Long.class) {
                return LongArrayTypeHandler.class;
            } else if (component == float.class || component == Float.class) {
                return FloatArrayTypeHandler.class;
            } else if (component == double.class || component == Double.class) {
                return DoubleArrayTypeHandler.class;
            } else if (component == boolean.class || component == Boolean.class) {
                return BooleanArrayTypeHandler.class;
            } else if (component == String.class) {
                return StringArrayTypeHandler.class;
            } else if (component == File.class) {
                return FileArrayTypeHandler.class;
            } else if (component == Range.class) {
                return RangeArrayTypeHandler.class;
            } else if (component == Period.class) {
                return PeriodArrayTypeHandler.class;
            } else if (component == Region.class) {
                return RegionArrayTypeHandler.class;
            } else if (component == Modification.class) {
                return ModificationArrayTypeHandler.class;
            }
        }
        Class<? extends TypeHandler<?>> clazz = this.handlers.get(type);
        return clazz != null ? clazz : this.handlers.computeIfAbsent(type, this::initializeJsonHandlerClass);
    }

    /**
     * 查找字段类型处理器
     *
     * @param clazz   对象类型
     * @param field   字段实例
     * @param handler 当前类型处理器对象
     * @return 类型处理器对象
     */
    @SuppressWarnings("unchecked")
    private Class<? extends TypeHandler<?>> lookupTypeHandlerClass(Class<?> clazz, Field field,
                                                                   Class<? extends TypeHandler<?>> handler) {
        // 排除忽略字段
        TableField annotation = field.getAnnotation(TableField.class);
        if (annotation != null && !annotation.exist()) {
            return null;
        }

        // 如果当前类型处理器不存在，则继续查找@TableField注解中的类型处理器，如果仍不存在则尝试使用JSON类型处理器
        Type type = TypeParameterResolver.resolveFieldType(field, clazz);
        Class<?> model = ObjectUtils.type2class(type);
        if (((handler == null && (handler = (Class<? extends TypeHandler<?>>)
                ObjectUtils.ifNull(annotation, TableField::typeHandler)) == null)
                || handler == UnknownTypeHandler.class || handler == JacksonTypeHandler.class)
                && !DatabaseContextHolder.isTypeHandlerRegistered(model)) {
            handler = this.lookupJsonHandlerClass(type);
        }

        // 如果是敏感字段，则初始化敏感字段类型处理器
        if (field.isAnnotationPresent(Sensitive.class)) {
            if (handler == StringsTypeHandler.class) {
                return SensitiveListTypeHandler.class;
            } else if (handler == StringSetTypeHandler.class) {
                return SensitiveSetTypeHandler.class;
            } else if (handler == StringArrayTypeHandler.class) {
                return SensitiveArrayTypeHandler.class;
            } else if (model == String.class
                    && (handler == null || !SensitiveTypeHandler.class.isAssignableFrom(handler))) {
                return this.initializeSensitiveHandlerClass(model, handler);
            }
        }
        return handler == UnknownTypeHandler.class ? null : handler;
    }

    /**
     * 初始化JSON类型处理器对象
     *
     * @param type 对象类型
     * @return 类型处理器对象
     */
    @SuppressWarnings("unchecked")
    private Class<? extends TypeHandler<?>> initializeJsonHandlerClass(Type type) {
        ClassPool pool = ClassPool.getDefault();
        CtClass handler = pool.makeClass(StringUtils.uuid());
        try {
            handler.setSuperclass(pool.get(JsonTypeHandler.Simple.class.getName()));
            CtConstructor constructor = new CtConstructor(new CtClass[]{}, handler);
            constructor.setBody(String.format("{super(%s);}", ObjectUtils.analyse(type)));
            handler.addConstructor(constructor);
            return (Class<? extends TypeHandler<?>>) handler.toClass(this.getClass().getClassLoader(), null);
        } catch (NotFoundException | CannotCompileException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 初始化敏感字段类型处理器
     *
     * @param model      数据模型对象
     * @param superclass 处理器父类
     * @return 类型处理器对象
     */
    @SuppressWarnings("unchecked")
    private Class<? extends TypeHandler<?>> initializeSensitiveHandlerClass(Class<?> model, Class<?> superclass) {
        if (superclass == null) {
            return SensitiveTypeHandler.class;
        }

        ClassPool pool = ClassPool.getDefault();
        CtClass handler = pool.makeClass(StringUtils.uuid());
        Set<String> methods = ObjectUtils.ifNull(
                superclass.getDeclaredMethods(),
                _methods -> Stream.of(_methods).map(Method::getName).collect(Collectors.toSet())
        );
        try {
            handler.setSuperclass(pool.get(superclass.getName()));

            // 重写setNonNullParameter或setParameter方法
            String method = methods.contains("setNonNullParameter") ? "setNonNullParameter" : "setParameter";
            handler.addMethod(CtNewMethod.make(String.format(
                    "public void %s(%s ps, int i, %s param, %s type) throws %s {\n " +
                            "super.%s(new %s(ps), i, param, type);\n}",
                    method, PreparedStatement.class.getName(), model.getName(), JdbcType.class.getName(),
                    SQLException.class.getName(), method, WriteStringEncryptor.class.getName()
            ), handler));

            // 重写getNullableResult或getResult方法
            method = methods.contains("getNullableResult") ? "getNullableResult" : "getResult";
            handler.addMethod(CtNewMethod.make(String.format(
                    "public %s %s(%s rs, String columnName) throws %s {\n " +
                            "return super.%s(new %s(rs), columnName);\n}",
                    model.getName(), method, ResultSet.class.getName(), SQLException.class.getName(),
                    method, ResultStringDecrypter.class.getName()
            ), handler));
            handler.addMethod(CtNewMethod.make(String.format(
                    "public %s %s(%s rs, int columnIndex) throws %s {\n " +
                            "return super.%s(new %s(rs), columnIndex);\n}",
                    model.getName(), method, ResultSet.class.getName(), SQLException.class.getName(),
                    method, ResultStringDecrypter.class.getName()
            ), handler));
            handler.addMethod(CtNewMethod.make(String.format(
                    "public %s %s(%s cs, int columnIndex) throws %s {\n " +
                            "return super.%s(new %s(cs), columnIndex);\n}",
                    model.getName(), method, CallableStatement.class.getName(), SQLException.class.getName(),
                    method, CallableStringDecrypter.class.getName()
            ), handler));
            return (Class<? extends TypeHandler<?>>) handler.toClass(this.getClass().getClassLoader(), null);
        } catch (NotFoundException | CannotCompileException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 初始化resultMap
     *
     * @param document xml文档对象
     * @param clazz    对象类型
     * @return 节点元素
     */
    private Element initializeResultMapping(Document document, Class<?> clazz) {
        // 构建resultMap节点元素
        Element element = document.createElement("resultMap");
        element.setAttribute("id", StringUtils.uuid());
        element.setAttribute("type", clazz.getName());

        // 添加resultMap字段映射
        ObjectUtils.foreachClassFields(clazz, field -> initializeResultMapping(
                document, clazz, field, element::appendChild
        ));
        return element;
    }

    /**
     * 初始化resultMap元素
     *
     * @param document xml文档对象
     * @param clazz    对象类型
     * @param field    对象字段
     * @param consumer resultMap元素回调函数
     */
    private void initializeResultMapping(Document document, Class<?> clazz, Field field, Consumer<Element> consumer) {
        // 排除主键字段
        if (field.isAnnotationPresent(TableId.class)) {
            return;
        }

        // 排除不需要TypeHandler的字段
        Class<? extends TypeHandler<?>> handler = this.lookupTypeHandlerClass(clazz, field, null);
        if (handler == null) {
            return;
        }

        // 获取可能的字段别名集合
        String property = field.getName();
        Set<String> aliases = Sets.newHashSetWithExpectedSize(3);
        aliases.add(property);
        aliases.add(StringUtils.hump2underscore(property));
        aliases.add(StringUtils.underscore2hump(property));
        String column = ObjectUtils.ifNull(field.getAnnotation(TableField.class), TableField::value);
        if (StringUtils.notEmpty(column = DatabaseContextHolder.unquote(column))) {
            aliases.add(column);
            aliases.add(StringUtils.hump2underscore(column));
            aliases.add(StringUtils.underscore2hump(column));
        }

        // 构建字段/属性映射元素
        aliases.forEach(alias -> {
            Element child = document.createElement("result");
            child.setAttribute("column", alias);
            child.setAttribute("property", property);
            child.setAttribute("typeHandler", handler.getName());
            consumer.accept(child);
        });
    }

    @Override
    @SuppressWarnings("unchecked")
    public void intercept(XNode mapper) {
        // 设置已有resultMap的类型处理器
        Document document = mapper.getNode().getOwnerDocument();
        List<XNode> maps = mapper.evalNodes("resultMap");
        if (ObjectUtils.notEmpty(maps)) {
            for (XNode map : maps) {
                Class<?> clazz = this.alias2class(map.getStringAttribute("type"));
                if (clazz == null || this.isMetaType(clazz)) {
                    continue;
                }
                Set<String> properties = Sets.newHashSet();
                for (XNode child : map.getChildren()) {
                    String property = child.getStringAttribute("property");
                    if (StringUtils.isEmpty(property)) {
                        continue;
                    }
                    properties.add(property);
                    if ("id".equalsIgnoreCase(child.getName())) {
                        continue;
                    }
                    Field field = ObjectUtils.findClassField(clazz, property);
                    if (field == null || field.isAnnotationPresent(TableId.class)) {
                        continue;
                    }

                    // 重置字段类型处理器
                    Class<? extends TypeHandler<?>> handler =
                            (Class<? extends TypeHandler<?>>) this.name2class(child.getStringAttribute("typeHandler"));
                    if ((handler = this.lookupTypeHandlerClass(clazz, field, handler)) != null) {
                        ((Element) child.getNode()).setAttribute("typeHandler", handler.getName());
                    }
                    if (StringUtils.isEmpty(child.getStringAttribute("typeHandler"))) {
                        continue;
                    }

                    // 如果column为空则设置为当前属性名
                    String column = child.getStringAttribute("column");
                    if (StringUtils.isEmpty(column)) {
                        ((Element) child.getNode()).setAttribute("column", property);
                    }

                    // 补齐下划线格式字段映射
                    if (!Objects.equals(column = StringUtils.hump2underscore(property), property)) {
                        Element element = document.createElement("result");
                        element.setAttribute("column", column);
                        element.setAttribute("property", property);
                        element.setAttribute("typeHandler", child.getStringAttribute("typeHandler"));
                        map.getNode().appendChild(element);
                    }
                }

                // 补齐缺失字段映射
                ObjectUtils.foreachClassFields(clazz, field -> {
                    if (!properties.contains(field.getName())) {
                        initializeResultMapping(document, clazz, field, map.getNode()::appendChild);
                    }
                });
            }
        }

        // 设置resultType对应的resultMap，并将原来的resultType替换成resultMap
        List<XNode> selects = mapper.evalNodes("select");
        if (ObjectUtils.notEmpty(selects)) {
            Map<Class<?>, Element> cache = Maps.newHashMap();
            for (XNode select : selects) {
                Class<?> clazz = this.alias2class(select.getStringAttribute("resultType"));
                if (clazz != null && this.isTypeHandlerConfigurable(clazz)) {
                    // 将resultType替换成resultMap
                    Element map = cache.computeIfAbsent(clazz, c -> this.initializeResultMapping(document, clazz));
                    NodeList children = map.getChildNodes();
                    if (children != null && children.getLength() > 0) {
                        mapper.getNode().appendChild(map);
                        Element element = (Element) select.getNode();
                        element.removeAttribute("resultType");
                        element.setAttribute("resultMap", map.getAttribute("id"));
                    }
                }
            }
        }
    }

    @Override
    public void intercept(TableInfo table) {
        // 自动绑定字段类型处理器
        List<TableFieldInfo> fields = table.getFieldList();
        if (ObjectUtils.notEmpty(fields)) {
            boolean bound = false;
            for (TableFieldInfo field : fields) {
                bound |= this.bindTypeHandler(table.getEntityType(), field);
            }
            if (bound && !table.isAutoInitResultMap()) {
                ObjectUtils.setObjectValue(table, "autoInitResultMap", true);
            }
        }
    }
}
