package com.chelvc.framework.database.context;

import java.io.InputStream;
import java.io.Reader;
import java.lang.reflect.Modifier;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.time.Instant;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.Month;
import java.time.OffsetDateTime;
import java.time.OffsetTime;
import java.time.Year;
import java.time.YearMonth;
import java.time.ZonedDateTime;
import java.time.chrono.JapaneseDate;
import java.util.Collection;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.function.Supplier;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.metadata.TableInfo;
import com.baomidou.mybatisplus.core.metadata.TableInfoHelper;
import com.baomidou.mybatisplus.core.toolkit.LambdaUtils;
import com.baomidou.mybatisplus.core.toolkit.support.ColumnCache;
import com.baomidou.mybatisplus.core.toolkit.support.SFunction;
import com.baomidou.mybatisplus.core.toolkit.support.SerializedLambda;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.base.util.SpringUtils;
import com.chelvc.framework.common.function.Executor;
import com.chelvc.framework.common.function.Handler;
import com.chelvc.framework.common.function.Provider;
import com.chelvc.framework.common.model.File;
import com.chelvc.framework.common.model.Modification;
import com.chelvc.framework.common.model.Pagination;
import com.chelvc.framework.common.model.Paging;
import com.chelvc.framework.common.model.Period;
import com.chelvc.framework.common.model.Region;
import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.database.crypto.DatabaseCipherHandler;
import com.chelvc.framework.database.entity.Entity;
import com.google.common.collect.Maps;
import lombok.NonNull;
import org.apache.ibatis.reflection.property.PropertyNamer;
import org.apache.ibatis.type.TypeHandler;
import org.mybatis.spring.SqlSessionTemplate;
import org.springframework.context.ApplicationContext;
import org.springframework.dao.DuplicateKeyException;

/**
 * 数据库操作相关上下文工具类
 *
 * @author Woody
 * @date 2024/1/30
 */
public final class DatabaseContextHolder {
    /**
     * 表名称/上下文映射表
     */
    private static final Map<String, TableContext> TABLE_CONTEXT_MAPPING = Maps.newConcurrentMap();

    /**
     * 类对象/类型处理器实例映射表
     */
    private static final Map<Class<?>, TypeHandler<?>> TYPE_HANDLER_MAPPING = Maps.newConcurrentMap();

    /**
     * 事务处理器
     */
    private static volatile Transactor TRANSACTOR;

    /**
     * 全局SqlSessionTemplate实例
     */
    private static volatile SqlSessionTemplate SESSION_TEMPLATE;

    /**
     * 数据库加解密处理器
     */
    private static volatile DatabaseCipherHandler DATABASE_CIPHER_HANDLER;

    private DatabaseContextHolder() {
    }

    /**
     * 获取事务处理器
     *
     * @return 事务处理器
     */
    public static Transactor getTransactor() {
        if (TRANSACTOR == null) {
            synchronized (Transactor.class) {
                if (TRANSACTOR == null) {
                    TRANSACTOR = ApplicationContextHolder.getBean(Transactor.class);
                }
            }
        }
        return TRANSACTOR;
    }

    /**
     * 获取SqlSessionTemplate
     *
     * @return SqlSessionTemplate实例
     */
    public static SqlSessionTemplate getSessionTemplate() {
        if (SESSION_TEMPLATE == null) {
            synchronized (SqlSessionTemplate.class) {
                if (SESSION_TEMPLATE == null) {
                    SESSION_TEMPLATE = ApplicationContextHolder.getBean(SqlSessionTemplate.class);
                }
            }
        }
        return SESSION_TEMPLATE;
    }

    /**
     * 获取数据库加解密处理器
     *
     * @return 数据库加解密处理器实例
     */
    public static DatabaseCipherHandler getDatabaseCipherHandler() {
        if (DATABASE_CIPHER_HANDLER == null) {
            synchronized (DatabaseCipherHandler.class) {
                if (DATABASE_CIPHER_HANDLER == null) {
                    DATABASE_CIPHER_HANDLER = ApplicationContextHolder.getBean(DatabaseCipherHandler.class);
                }
            }
        }
        return DATABASE_CIPHER_HANDLER;
    }

    /**
     * 获取原生的数据库连接
     *
     * @return 数据库连接对象
     */
    public static Connection getConnection() {
        return getSessionTemplate().getConnection();
    }

    /**
     * 执行原生SQL
     *
     * @param sql SQL语句
     * @throws SQLException SQL执行异常
     */
    public static void execute(@NonNull String sql) throws SQLException {
        try (Statement statement = getConnection().createStatement()) {
            statement.execute(sql);
        }
    }

    /**
     * 批量执行原生SQL
     *
     * @param sqls SQL语句集合
     * @throws SQLException SQL执行异常
     */
    public static void execute(@NonNull Collection<String> sqls) throws SQLException {
        if (ObjectUtils.isEmpty(sqls)) {
            return;
        }
        try (Statement statement = getConnection().createStatement()) {
            for (String sql : sqls) {
                statement.addBatch(sql);
            }
            statement.executeBatch();
        }
    }

    /**
     * 执行原生查询SQL
     *
     * @param sql SQL语句
     * @return 查询结果集
     * @throws SQLException SQL执行异常
     */
    public static ResultSet query(@NonNull String sql) throws SQLException {
        try (Statement statement = getConnection().createStatement()) {
            return statement.executeQuery(sql);
        }
    }

    /**
     * 事务执行方法
     *
     * @param executor 回调方法
     */
    public static void transactional(@NonNull Executor executor) {
        getTransactor().execute(executor);
    }

    /**
     * 事务执行方法
     *
     * @param executor 回调方法
     * @param <T>      对象类型
     * @return 执行结果
     */
    public static <T> T transactional(@NonNull Supplier<T> executor) {
        return getTransactor().execute(executor);
    }

    /**
     * 事务执行方法
     *
     * @param executor 回调方法
     */
    public static void transactional(@NonNull Handler executor) throws Exception {
        getTransactor().execute(executor);
    }

    /**
     * 事务执行方法
     *
     * @param executor 回调方法
     * @param <T>      对象类型
     * @return 执行结果
     * @throws Exception 执行异常
     */
    public static <T> T transactional(@NonNull Provider<T> executor) throws Exception {
        return getTransactor().execute(executor);
    }

    /**
     * 数据更新或忽略
     *
     * @param function 数据更新执行函数
     * @return true/false
     */
    public static boolean modifyIgnore(@NonNull Executor function) {
        try {
            function.execute();
        } catch (DuplicateKeyException e) {
            return false;
        }
        return true;
    }

    /**
     * 数据更新或忽略
     *
     * @param function 数据更新执行函数
     * @return true/false
     */
    public static boolean modifyIgnore(@NonNull Supplier<Boolean> function) {
        try {
            return Boolean.TRUE.equals(function.get());
        } catch (DuplicateKeyException e) {
            return false;
        }
    }

    /**
     * 去掉表、字段名称反引号
     *
     * @param name 表、字段名称
     * @return 表、字段名称
     */
    public static String unquote(String name) {
        return isQuoted(name) ? name.substring(1, name.length() - 1) : name;
    }

    /**
     * 判断表、字段名称是否包含反引号
     *
     * @param name 表、字段名称
     * @return true/false
     */
    public static boolean isQuoted(String name) {
        return StringUtils.notEmpty(name) && name.charAt(0) == '`' && name.charAt(name.length() - 1) == '`';
    }

    /**
     * 获取表数据模型
     *
     * @param table 表名
     * @return 数据模型对象
     */
    public static Class<?> getTableModel(@NonNull String table) {
        return ObjectUtils.ifNull(getTableContext(table), TableContext::getModel);
    }

    /**
     * 获取表上下文
     *
     * @param table 表名称
     * @return 表上下文
     */
    public static TableContext getTableContext(@NonNull String table) {
        String unquote = unquote(table);
        TableContext context = TABLE_CONTEXT_MAPPING.get(unquote);
        if (context == null) {
            context = TABLE_CONTEXT_MAPPING.computeIfAbsent(unquote, k -> {
                TableInfo info = TableInfoHelper.getTableInfo(table);
                if (info == null && isQuoted(table)) {
                    info = TableInfoHelper.getTableInfo(unquote);
                }
                return ObjectUtils.ifNull(info, TableContext::new);
            });
        }
        return context;
    }

    /**
     * 获取表字段上下文
     *
     * @param table 表名称
     * @param field 字段名称
     * @return 表字段上下文
     */
    public static TableFieldContext getTableFieldContext(@NonNull String table, @NonNull String field) {
        return ObjectUtils.ifNull(getTableContext(table), context -> context.getFieldContext(field));
    }

    /**
     * 获取TypeHandler实例
     *
     * @param clazz TypeHandler对象类型
     * @param <T>   TypeHandler类型
     * @return TypeHandler实例
     */
    @SuppressWarnings("unchecked")
    public static <T extends TypeHandler<?>> T getTypeHandler(@NonNull Class<T> clazz) {
        T handler = (T) TYPE_HANDLER_MAPPING.get(clazz);
        if (handler == null) {
            handler = (T) TYPE_HANDLER_MAPPING.computeIfAbsent(clazz, k -> ObjectUtils.instance(clazz));
        }
        return handler;
    }

    /**
     * 将Getter方法转换成字段名称
     *
     * @param getter Getter方法
     * @return 字段名称
     */
    public static String getter2column(@NonNull SFunction<?, ?> getter) {
        SerializedLambda lambda = LambdaUtils.resolve(getter);
        String property = PropertyNamer.methodToProperty(lambda.getImplMethodName());
        return property2column(lambda.getInstantiatedType(), property);
    }

    /**
     * 将Getter方法转换成属性名称
     *
     * @param getter Getter方法
     * @return 属性名称
     */
    public static String getter2property(@NonNull SFunction<?, ?> getter) {
        SerializedLambda lambda = LambdaUtils.resolve(getter);
        return PropertyNamer.methodToProperty(lambda.getImplMethodName());
    }

    /**
     * 将属性名转换成字段名称
     *
     * @param clazz    实体对象
     * @param property 属性名称
     * @return 字段名称
     */
    public static String property2column(@NonNull Class<?> clazz, @NonNull String property) {
        Map<String, ColumnCache> columns = LambdaUtils.getColumnMap(clazz);
        ColumnCache column = ObjectUtils.ifNull(columns, cs -> cs.get(LambdaUtils.formatKey(property)));
        AssertUtils.nonnull(column, () -> "Column not found for property: " + clazz.getName() + "." + property);
        return column.getColumn();
    }

    /**
     * 判断对象类型处理器是否已注册
     *
     * @param type 对象类型
     * @return true/false
     */
    public static boolean isTypeHandlerRegistered(Class<?> type) {
        return type == byte.class || type == Byte.class || type == byte[].class || type == Byte[].class
                || type == char.class || type == Character.class || type == short.class || type == Short.class
                || type == int.class || type == Integer.class || type == long.class || type == Long.class
                || type == float.class || type == Float.class || type == double.class || type == Double.class
                || type == boolean.class || type == Boolean.class || type == String.class || type == Object.class
                || type == BigInteger.class || type == BigDecimal.class || type == Reader.class
                || type == InputStream.class || type == Date.class || type == java.sql.Date.class
                || type == java.sql.Time.class || type == java.sql.Timestamp.class || type == Instant.class
                || type == LocalDate.class || type == LocalTime.class || type == LocalDateTime.class
                || type == OffsetTime.class || type == OffsetDateTime.class || type == ZonedDateTime.class
                || type == Year.class || type == Month.class || type == YearMonth.class || type == JapaneseDate.class
                || (type != null && Enum.class.isAssignableFrom(type)) || type == File.class || type == Period.class
                || type == Region.class || type == Modification.class;
    }

    /**
     * 根据包名查找数据模型对象
     *
     * @return 数据模型对象列表
     */
    public static List<Class<?>> lookupEntityClasses() {
        return ApplicationContextHolder.lookupApplicationClasses(clazz -> Entity.class.isAssignableFrom(clazz)
                && !Modifier.isInterface(clazz.getModifiers()) && !Modifier.isAbstract(clazz.getModifiers()));
    }

    /**
     * 根据包名查找数据模型对象
     *
     * @param pack 包路径
     * @return 数据模型对象列表
     */
    public static List<Class<?>> lookupEntityClasses(String pack) {
        return SpringUtils.lookupClasses(pack, clazz -> Entity.class.isAssignableFrom(clazz)
                && !Modifier.isInterface(clazz.getModifiers()) && !Modifier.isAbstract(clazz.getModifiers()));
    }

    /**
     * 根据包名查找数据模型对象
     *
     * @param applicationContext 应用上下文
     * @return 数据模型对象列表
     */
    public static List<Class<?>> lookupEntityClasses(@NonNull ApplicationContext applicationContext) {
        return ApplicationContextHolder.lookupApplicationClasses(applicationContext,
                clazz -> Entity.class.isAssignableFrom(clazz) && !Modifier.isInterface(clazz.getModifiers())
                        && !Modifier.isAbstract(clazz.getModifiers()));
    }

    /**
     * 将分页请求参数转换成MyBatis Plus分页参数
     *
     * @param paging 分页请求参数
     * @param <T>    目标对象类型泛型
     * @return MyBatis Plus分页参数
     */
    public static <T> Page<T> page(Paging paging) {
        return paging == null ? null : page(paging.getNumber(), paging.getSize(), paging.isCounting());
    }

    /**
     * 将分页请求参数转换成MyBatis Plus分页参数
     *
     * @param paging   分页请求参数
     * @param counting 是否获取总数
     * @param <T>      目标对象类型泛型
     * @return MyBatis Plus分页参数
     */
    public static <T> Page<T> page(Paging paging, boolean counting) {
        return paging == null ? null : page(paging.getNumber(), paging.getSize(), counting);
    }

    /**
     * 构建分页信息
     *
     * @param number 页码
     * @param size   页面大小
     * @param <T>    数据类型
     * @return 分页信息
     */
    public static <T> Page<T> page(long number, long size) {
        return page(number, size, true);
    }

    /**
     * 构建分页信息
     *
     * @param number   页码
     * @param size     页面大小
     * @param counting 是否获取总数
     * @param <T>      数据类型
     * @return 分页信息
     */
    public static <T> Page<T> page(long number, long size, boolean counting) {
        return new Page<>(number, size, counting);
    }

    /**
     * 数据分页处理
     *
     * @param paging   分页信息
     * @param function 分页处理函数
     * @param <T>      数据类型
     * @return 分页信息
     */
    public static <T> Pagination<T> paging(@NonNull Paging paging, @NonNull Function<IPage<T>, IPage<T>> function) {
        IPage<T> page = function.apply(page(paging));
        return page == null || ObjectUtils.isEmpty(page.getRecords()) ? Pagination.empty() : pagination(page);
    }

    /**
     * 数据分页处理
     *
     * @param paging   分页信息
     * @param function 分页处理函数
     * @param adapter  数据转换适配器
     * @param <T>      原始数据类型
     * @param <R>      目标数据类型
     * @return 分页信息
     */
    public static <T, R> Pagination<R> paging(@NonNull Paging paging, @NonNull Function<IPage<T>, IPage<T>> function,
                                              @NonNull Function<List<T>, List<R>> adapter) {
        IPage<T> page = function.apply(page(paging));
        return page == null || ObjectUtils.isEmpty(page.getRecords()) ? Pagination.empty() : pagination(page, adapter);
    }

    /**
     * 将MyBatis Plus分页参数转换成分页返回参数
     *
     * @param page MyBatis Plus分页参数
     * @param <T>  目标对象类型泛型
     * @return 分页返回参数
     */
    public static <T> Pagination<T> pagination(IPage<T> page) {
        return pagination(page, (List<T>) ObjectUtils.ifNull(page, IPage::getRecords));
    }

    /**
     * 将MyBatis Plus分页参数转换成分页返回参数
     *
     * @param page MyBatis Plus分页参数
     * @param type 目标类型
     * @param <T>  目标对象类型泛型
     * @return 分页返回参数
     */
    public static <T> Pagination<T> pagination(IPage<?> page, @NonNull Class<T> type) {
        return pagination(page, ObjectUtils.copying((List<?>) ObjectUtils.ifNull(page, IPage::getRecords), type));
    }

    /**
     * 将MyBatis Plus分页参数转换成分页返回参数
     *
     * @param page    MyBatis Plus分页参数
     * @param adapter 对象适配函数
     * @param <T>     原始对象类型泛型
     * @param <R>     目标对象类型泛型
     * @return 分页返回参数
     */
    public static <T, R> Pagination<R> pagination(IPage<T> page, @NonNull Function<List<T>, List<R>> adapter) {
        List<T> records = ObjectUtils.ifNull(page, IPage::getRecords);
        return ObjectUtils.isEmpty(records) ? Pagination.empty() : pagination(page, adapter.apply(records));
    }

    /**
     * 将MyBatis Plus分页参数转换成分页返回参数
     *
     * @param page    MyBatis Plus分页参数
     * @param records 分页数据列表
     * @param <T>     目标对象类型泛型
     * @return 分页返回参数
     */
    public static <T> Pagination<T> pagination(IPage<?> page, List<T> records) {
        if (page == null || ObjectUtils.isEmpty(records)) {
            return Pagination.empty();
        }
        return Pagination.<T>builder().total(page.getTotal()).pages(page.getPages()).records(records).build();
    }
}