package com.chelvc.framework.database.context;

import java.io.Serializable;
import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.sql.SQLException;

import com.baomidou.mybatisplus.core.metadata.TableFieldInfo;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.database.annotation.Sensitive;
import com.chelvc.framework.database.handler.SensitiveTypeHandler;
import com.chelvc.framework.database.sql.WriteStringAcceptor;
import lombok.Getter;
import lombok.NonNull;
import org.apache.ibatis.type.JdbcType;
import org.apache.ibatis.type.TypeHandler;
import org.apache.ibatis.type.UnknownTypeHandler;

/**
 * 字段上下文对象
 *
 * @author Woody
 * @date 2024/6/10
 */
@Getter
public class TableFieldContext implements Serializable {
    private final boolean sensitive;
    private final TableFieldInfo info;

    public TableFieldContext(@NonNull TableFieldInfo info) {
        this.info = info;
        this.sensitive = info.getField().isAnnotationPresent(Sensitive.class);
    }

    /**
     * 获取属性类型
     *
     * @return 对象类型
     */
    public Class<?> getType() {
        return this.info.getPropertyType();
    }

    /**
     * 获取字段对象
     *
     * @return 字段对象实例
     */
    public Field getField() {
        return this.info.getField();
    }

    /**
     * 获取字段名称
     *
     * @return 字段名称
     */
    public String getColumn() {
        return this.info.getColumn();
    }

    /**
     * 获取属性名称
     *
     * @return 属性名称
     */
    public String getProperty() {
        return this.info.getProperty();
    }

    /**
     * 获取类型处理器实例
     *
     * @return 类型处理器实例
     */
    public TypeHandler<?> getHandler() {
        Class<? extends TypeHandler<?>> type = this.info.getTypeHandler();
        return type == null || type == UnknownTypeHandler.class ? null : DatabaseContextHolder.getTypeHandler(type);
    }

    /**
     * 处理字段写入值
     *
     * @param original 原始值
     * @return 写入值
     */
    @SuppressWarnings("unchecked")
    public Object handleWriteValue(Object original) {
        if (original == null) {
            return null;
        }

        // 获取TypeHandler实例
        TypeHandler<?> handler = this.getHandler();
        if (handler == null || handler instanceof SensitiveTypeHandler) {
            return original;
        } else if (this.sensitive) {
            Class<?> superclass = this.info.getTypeHandler().getSuperclass();
            int modifiers = superclass.getModifiers();
            if (Modifier.isAbstract(modifiers) || Modifier.isInterface(modifiers)
                    || !TypeHandler.class.isAssignableFrom(superclass)) {
                return original;
            }
            handler = DatabaseContextHolder.getTypeHandler((Class<TypeHandler<?>>) superclass);
        }

        // 接收字符串值写入，如果字符串值不为null则返回字符串值，否则返回原始值
        WriteStringAcceptor acceptor = new WriteStringAcceptor();
        try {
            ((TypeHandler<Object>) handler).setParameter(acceptor, 1, original, JdbcType.OTHER);
        } catch (SQLException e) {
            throw new RuntimeException(e);
        }
        return ObjectUtils.ifNull(acceptor.getValue(), original);
    }
}
