package com.chelvc.framework.database.interceptor;

import java.util.Collections;
import java.util.Map;
import java.util.Set;

import com.baomidou.dynamic.datasource.spring.boot.autoconfigure.DynamicDataSourceProperties;
import com.baomidou.dynamic.datasource.toolkit.DynamicDataSourceContextHolder;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.google.common.collect.Maps;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.springframework.aop.framework.AopProxyUtils;
import org.springframework.boot.autoconfigure.condition.ConditionalOnClass;
import org.springframework.core.Ordered;
import org.springframework.core.annotation.Order;

/**
 * 动态数据源拦截器
 *
 * @author Woody
 * @date 2024/1/30
 */
@Aspect
@Order(Ordered.HIGHEST_PRECEDENCE)
@ConditionalOnClass(DynamicDataSourceContextHolder.class)
public class DynamicDatasourceInterceptor {
    private final Map<String, String> datasourceContext;

    public DynamicDatasourceInterceptor(DynamicDataSourceProperties properties) {
        // 初始化数据源标识集合
        Set<String> keys = properties.getDatasource().keySet();
        if (ObjectUtils.isEmpty(keys)) {
            this.datasourceContext = Collections.emptyMap();
        } else {
            this.datasourceContext = Maps.newHashMapWithExpectedSize(keys.size());
            keys.forEach(value -> {
                for (String key : StringUtils.splitActives(value)) {
                    this.datasourceContext.put(key, value);
                }
            });
        }
    }

    /**
     * 查找数据源标识
     *
     * @param point 方法拦截点
     * @return 数据源标识
     */
    private String lookupDatasourceKey(ProceedingJoinPoint point) {
        Object target = point.getTarget();

        // 根据目标类对象查找数据源标识
        String className = AopProxyUtils.ultimateTargetClass(target).getName();
        Set<Map.Entry<String, String>> routes = this.datasourceContext.entrySet();
        String key = routes.stream().filter(entry -> className.startsWith(entry.getKey())).findAny()
                .map(Map.Entry::getValue).orElse(null);
        if (StringUtils.notEmpty(key)) {
            return key;
        }

        // 根据目标接口对象查找数据源标识
        Class<?>[] interfaces = AopProxyUtils.proxiedUserInterfaces(target);
        if (interfaces.length > 0) {
            for (Class<?> clazz : interfaces) {
                String interfaceName = clazz.getName();
                String route = routes.stream().filter(entry -> interfaceName.startsWith(entry.getKey())).findAny()
                        .map(Map.Entry::getValue).orElse(null);
                if (StringUtils.notEmpty(route)) {
                    return route;
                }
            }
        }
        return null;
    }

    /**
     * 数据源路由
     *
     * @param point 方法拦截点
     * @return 方法调用结果
     * @throws Throwable 方法调用异常
     */
    @Around("@within(org.springframework.stereotype.Service) " +
            "|| @within(org.apache.ibatis.annotations.Mapper) " +
            "|| this(com.baomidou.mybatisplus.core.mapper.BaseMapper) " +
            "|| this(com.chelvc.framework.database.support.EnhanceMapper)")
    public Object routing(ProceedingJoinPoint point) throws Throwable {
        DynamicDataSourceContextHolder.push(this.lookupDatasourceKey(point));
        try {
            return point.proceed();
        } finally {
            DynamicDataSourceContextHolder.poll();
        }
    }
}