package com.chelvc.framework.database.interceptor;

import java.lang.reflect.Field;
import java.math.BigInteger;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.Map;
import java.util.Set;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.common.util.DateUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.database.context.DatabaseContextHolder;
import com.chelvc.framework.database.context.TableFieldContext;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.google.common.collect.Sets;
import lombok.extern.slf4j.Slf4j;
import net.sf.jsqlparser.expression.Expression;
import net.sf.jsqlparser.expression.LongValue;
import net.sf.jsqlparser.expression.StringValue;
import net.sf.jsqlparser.expression.operators.relational.ExpressionList;
import net.sf.jsqlparser.schema.Column;
import org.apache.commons.lang3.tuple.Pair;

/**
 * SQL表达式工具类
 *
 * @author Woody
 * @date 2024/5/16
 */
@Slf4j
final class Expressions {
    /**
     * 枚举选项限定配置属性名
     */
    private static final String ENUM_OPTION_LIMIT_PROPERTY = "enum.option.limit";

    private Expressions() {
    }

    /**
     * 零值
     */
    public static final LongValue ZERO_VALUE = new LongValue(0) {
        @Override
        public long getValue() {
            return 0;
        }

        @Override
        public BigInteger getBigIntegerValue() {
            return BigInteger.ZERO;
        }
    };

    /**
     * 环境字段
     */
    public static final Column ENV_COLUMN = new Column("env");

    /**
     * 创建人字段
     */
    public static final Column CREATOR_COLUMN = new Column("creator");

    /**
     * 创建时间字段
     */
    public static final Column CREATE_TIME_COLUMN = new Column("create_time");

    /**
     * 更新人字段
     */
    public static final Column UPDATER_COLUMN = new Column("updater");

    /**
     * 更新时间字段
     */
    public static final Column UPDATE_TIME_COLUMN = new Column("update_time");

    /**
     * 逻辑删除字段
     */
    public static final Column DELETED_COLUMN = new Column("deleted");

    /**
     * 零值/表达式映射
     */
    public static final Pair<Integer, Expression> ZERO = Pair.of(0, ZERO_VALUE);

    /**
     * 假值/表达式映射
     */
    public static final Pair<Boolean, Expression> FALSE = Pair.of(false, ZERO_VALUE);

    /**
     * 环境标识/表达式映射
     */
    private static volatile Pair<String, Expression> ENV;

    /**
     * 解析数据查询附加条件配置
     *
     * @param config 配置信息
     * @return 表名/枚举字段名/选项值映射表
     */
    private static Map<String, Map<String, ExpressionList>> analyseSelectAdditionalCondition(String config) {
        if (StringUtils.isEmpty(config)) {
            return Collections.emptyMap();
        }

        // 构建表名/枚举字段上下文映射表
        String[] expressions = config.split(",");
        if (ObjectUtils.isEmpty(expressions)) {
            return Collections.emptyMap();
        }
        Map<String, Set<TableFieldContext>> tables = Maps.newHashMapWithExpectedSize(expressions.length);
        for (String expression : expressions) {
            if (StringUtils.isEmpty(expression = expression.trim())) {
                continue;
            }

            // 解析表字段配置表达式
            String table, column;
            int index = expression.indexOf('.');
            if (index < 1 || StringUtils.isEmpty(table = expression.substring(0, index))
                    || StringUtils.isEmpty(column = expression.substring(index + 1))) {
                log.warn("Invalid column expression: {}", expression);
                continue;
            }

            // 获取表字段上下文信息
            TableFieldContext context = DatabaseContextHolder.getTableFieldContext(table, column);
            if (context != null && Enum.class.isAssignableFrom(context.getType())) {
                tables.computeIfAbsent(DatabaseContextHolder.unquote(table), k -> Sets.newHashSet()).add(context);
            }
        }
        if (ObjectUtils.isEmpty(tables)) {
            return Collections.emptyMap();
        }

        // 构建表明/枚举字段名/选项值映射表
        Map<String, Map<String, ExpressionList>> conditions = Maps.newHashMapWithExpectedSize(tables.size());
        tables.forEach((table, contexts) -> {
            Map<String, ExpressionList> additional = Maps.newLinkedHashMapWithExpectedSize(contexts.size());
            for (TableFieldContext context : contexts) {
                Field[] fields = context.getType().getDeclaredFields();
                if (ObjectUtils.notEmpty(fields)) {
                    ArrayList<Expression> values = Lists.newArrayListWithCapacity(fields.length);
                    for (Field field : fields) {
                        if (!field.isSynthetic() && field.isEnumConstant()) {
                            values.add(new StringValue(field.getName()));
                        }
                    }
                    if (!values.isEmpty()) {
                        values.trimToSize();
                        additional.put(context.getColumn(), new ExpressionList(values));
                    }
                }
            }
            if (!additional.isEmpty()) {
                conditions.put(table, Collections.unmodifiableMap(additional));
            }
        });
        return conditions.isEmpty() ? Collections.emptyMap() : Collections.unmodifiableMap(conditions);
    }

    /**
     * 获取环境信息
     *
     * @return 环境标识/表达式
     */
    public static Pair<String, Expression> env() {
        if (ENV == null) {
            synchronized (Expressions.class) {
                if (ENV == null) {
                    String profile = ApplicationContextHolder.getProfile();
                    ENV = profile == null ? Pair.of(null, null) : Pair.of(profile, new StringValue(profile));
                }
            }
        }
        return ENV;
    }

    /**
     * 获取当前操作人
     *
     * @return 操作人/表达式
     */
    public static Pair<Long, Expression> operator() {
        Long operator = ObjectUtils.ifNull(SessionContextHolder.getId(), 0L);
        return Pair.of(operator, new LongValue(operator));
    }

    /**
     * 获取当前时间
     *
     * @return 时间/表达式
     */
    public static Pair<Date, Expression> datetime() {
        Date datetime = new Date();
        LocalDateTime now = DateUtils.convert(datetime);
        String format = DateUtils.DEFAULT_TIMESTAMP_FORMATTER.format(now);
        return Pair.of(datetime, new StringValue(format));
    }

    /**
     * 获取数据查询附加条件
     *
     * @return 表名称/字段名/值映射表
     */
    public static Map<String, Map<String, ExpressionList>> getSelectAdditionalConditions() {
        return ApplicationContextHolder.getProperty(
                ENUM_OPTION_LIMIT_PROPERTY, Expressions::analyseSelectAdditionalCondition
        );
    }
}
