package com.chelvc.framework.database.support;

import java.io.Serializable;
import java.lang.reflect.Field;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.Map;
import java.util.Objects;

import com.chelvc.framework.common.util.JacksonUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.google.common.collect.Maps;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.NonNull;
import lombok.experimental.SuperBuilder;

/**
 * binlog日志
 *
 * @author Woody
 * @date 2024/10/15
 */
@Data
@SuperBuilder
@NoArgsConstructor
@AllArgsConstructor
public class Binlog implements Serializable {
    /**
     * 操作类型（c -> 新增、u -> 修改、 d -> 删除）
     */
    private String op;

    /**
     * 更新前字段/值映射表
     */
    private Map<String, Object> before;

    /**
     * 更新后字段/值映射表
     */
    private Map<String, Object> after;

    /**
     * 转换字段值
     *
     * @param field  字段对象
     * @param object 字段原始值
     * @return 转换后字段值
     */
    public static Object convert(@NonNull Field field, Object object) {
        if (object == null) {
            return null;
        }

        Class<?> type = field.getType();
        if (StringUtils.isEmpty(object) && (type.isEnum() || type.isArray()
                || Map.class.isAssignableFrom(type) || Collection.class.isAssignableFrom(type))) {
            return null;
        }
        if (ObjectUtils.isMetaClass(type) || type.isEnum() || type == String.class
                || Date.class.isAssignableFrom(type)) {
            return JacksonUtils.convert(object, type);
        } else if (object instanceof String && StringUtils.notEmpty(object)) {
            // binlog消息不支持JSON格式，所以需要单独反序列化处理
            String string = (String) object;
            char first = string.charAt(0), last = string.charAt(string.length() - 1);
            if ((first == '{' && last == '}') || (first == '[' && last == ']')) {
                return JacksonUtils.deserialize(string, field.getGenericType());
            }
        }
        return JacksonUtils.convert(object, type);
    }

    /**
     * 转换实体对象实例
     *
     * @param model   实体模型
     * @param mapping 字段名/值映射表
     * @param <T>     实体类型
     * @return 实体对象实例
     */
    public static <T> T convert(@NonNull Class<T> model, Map<?, ?> mapping) {
        T entity = ObjectUtils.instance(model);
        update(entity, mapping);
        return entity;
    }

    /**
     * 更新实体属性值
     *
     * @param entity  实体对象实例
     * @param mapping 字段名/值映射表
     */
    public static void update(@NonNull Object entity, Map<?, ?> mapping) {
        if (ObjectUtils.isEmpty(mapping)) {
            return;
        }

        Class<?> model = entity.getClass();
        Map<String, Field> fields = ObjectUtils.getClassFields(model);
        if (ObjectUtils.notEmpty(fields)) {
            for (Map.Entry<String, Field> entry : fields.entrySet()) {
                String column = StringUtils.hump2underscore(entry.getKey());
                Object value = mapping.get(column);
                value = convert(entry.getValue(), value);
                ObjectUtils.setObjectValue(entity, entry.getValue(), value);
            }
        }
    }

    /**
     * 是否新增
     *
     * @return true/false
     */
    public boolean isInsert() {
        return Objects.equals(this.op, "c");
    }

    /**
     * 是否修改
     *
     * @return true/false
     */
    public boolean isUpdate() {
        return Objects.equals(this.op, "u");
    }

    /**
     * 是否删除
     *
     * @return true/false
     */
    public boolean isDelete() {
        return Objects.equals(this.op, "d");
    }

    /**
     * 判断数据是否更新
     *
     * @param model 数据模型
     * @return true/false
     */
    public boolean isModified(@NonNull Class<?> model) {
        if (ObjectUtils.isEmpty(this.after)) {
            return false;
        }

        Map<String, Field> fields = ObjectUtils.getClassFields(model);
        if (ObjectUtils.notEmpty(fields)) {
            for (Map.Entry<String, Field> entry : fields.entrySet()) {
                String column = StringUtils.hump2underscore(entry.getKey());
                Object a = ObjectUtils.ifNull(this.after, map -> map.get(column));
                Object b = ObjectUtils.ifNull(this.before, map -> map.get(column));
                if (!ObjectUtils.equals(a, b)) {
                    return true;
                }
            }
        }
        return false;
    }

    /**
     * 判断数据是否更新
     *
     * @param columns 数据库字段数组
     * @return true/false
     */
    public boolean isModified(@NonNull String... columns) {
        if (ObjectUtils.notEmpty(this.after) && ObjectUtils.notEmpty(columns)) {
            for (String column : columns) {
                Object a = ObjectUtils.ifNull(this.after, map -> map.get(column));
                Object b = ObjectUtils.ifNull(this.before, map -> map.get(column));
                if (!ObjectUtils.equals(a, b)) {
                    return true;
                }
            }
        }
        return false;
    }

    /**
     * 赋值实体对象更新后属性值
     *
     * @param entity 实体对象实例
     * @param <T>    实体类型
     */
    public <T> void after(@NonNull T entity) {
        update(entity, this.after);
    }

    /**
     * 赋值实体对象更新后属性值
     *
     * @param model 实体模型
     * @param <T>   实体类型
     * @return 实体对象实例
     */
    public <T> T after(@NonNull Class<T> model) {
        return ObjectUtils.isEmpty(this.after) ? null : convert(model, this.after);
    }

    /**
     * 赋值实体对象更新前属性值
     *
     * @param entity 实体对象实例
     * @param <T>    实体类型
     */
    public <T> void before(@NonNull T entity) {
        update(entity, this.before);
    }

    /**
     * 赋值实体对象更新前属性值
     *
     * @param model 实体模型
     * @param <T>   实体类型
     * @return 实体对象实例
     */
    public <T> T before(@NonNull Class<T> model) {
        return ObjectUtils.isEmpty(this.before) ? null : convert(model, this.before);
    }

    /**
     * 获取更新信息
     *
     * @param model 实体模型
     * @param <T>   数据类型
     * @return 实体模型实例
     */
    public <T> T modification(@NonNull Class<T> model) {
        return this.modification(model, null);
    }

    /**
     * 获取更新信息
     *
     * @param model 实体模型
     * @param limit 限制模型
     * @param <T>   数据类型
     * @return 实体模型实例
     */
    public <T> T modification(@NonNull Class<T> model, Class<?> limit) {
        if (ObjectUtils.isEmpty(this.after)) {
            return null;
        }

        Map<String, Field> fields = ObjectUtils.getClassFields(model);
        if (ObjectUtils.isEmpty(fields)) {
            return null;
        }
        T instance = null;
        boolean identical = limit == model;
        Map<String, Field> limits = identical ? fields : ObjectUtils.ifNull(limit, ObjectUtils::getClassFields);
        for (Map.Entry<String, Field> entry : fields.entrySet()) {
            if (!identical && ObjectUtils.notEmpty(limits) && !limits.containsKey(entry.getKey())) {
                continue;
            }
            String column = StringUtils.hump2underscore(entry.getKey());
            Object a = ObjectUtils.ifNull(this.after, map -> map.get(column));
            Object b = ObjectUtils.ifNull(this.before, map -> map.get(column));
            if (!ObjectUtils.equals(a, b)) {
                if (instance == null) {
                    instance = ObjectUtils.instance(model);
                }
                ObjectUtils.setObjectValue(instance, entry.getValue(), convert(entry.getValue(), a));
            }
        }
        return instance;
    }

    /**
     * 获取差异属性值
     *
     * @param model 实体模型
     * @return 实体对象属性名/值映射表
     */
    public Map<String, Object> different(@NonNull Class<?> model) {
        return this.different(model, null);
    }

    /**
     * 获取差异属性值
     *
     * @param model 实体模型
     * @param limit 限制模型
     * @return 实体对象属性名/值映射表
     */
    public Map<String, Object> different(@NonNull Class<?> model, Class<?> limit) {
        if (ObjectUtils.isEmpty(this.after)) {
            return Collections.emptyMap();
        }

        Map<String, Field> fields = ObjectUtils.getClassFields(model);
        if (ObjectUtils.isEmpty(fields)) {
            return Collections.emptyMap();
        }
        boolean identical = limit == model;
        Map<String, Object> different = Maps.newHashMap();
        Map<String, Field> limits = identical ? fields : ObjectUtils.ifNull(limit, ObjectUtils::getClassFields);
        for (Map.Entry<String, Field> entry : fields.entrySet()) {
            if (!identical && ObjectUtils.notEmpty(limits) && !limits.containsKey(entry.getKey())) {
                continue;
            }
            String column = StringUtils.hump2underscore(entry.getKey());
            Object a = ObjectUtils.ifNull(this.after, map -> map.get(column));
            Object b = ObjectUtils.ifNull(this.before, map -> map.get(column));
            if (!ObjectUtils.equals(a, b)) {
                different.put(entry.getKey(), convert(entry.getValue(), a));
            }
        }
        return different.isEmpty() ? Collections.emptyMap() : different;
    }

    /**
     * 获取差异字段值
     *
     * @param columns 数据库字段数组
     * @return 数据库字段名/值映射表
     */
    public Map<String, Object> different(@NonNull String... columns) {
        if (ObjectUtils.isEmpty(this.after)) {
            return Collections.emptyMap();
        }

        Map<String, Object> different = Maps.newHashMap();
        if (ObjectUtils.isEmpty(columns)) {
            this.after.forEach((key, a) -> {
                Object b = ObjectUtils.ifNull(this.before, map -> map.get(key));
                if (!ObjectUtils.equals(a, b)) {
                    different.put(key, a);
                }
            });
        } else {
            for (String column : columns) {
                Object a = ObjectUtils.ifNull(this.after, map -> map.get(column));
                Object b = ObjectUtils.ifNull(this.before, map -> map.get(column));
                if (!ObjectUtils.equals(a, b)) {
                    different.put(column, a);
                }
            }
        }
        return different.isEmpty() ? Collections.emptyMap() : different;
    }
}
