package com.chelvc.framework.database.support;

import java.io.Serializable;
import java.util.Collections;
import java.util.Map;
import java.util.Objects;
import java.util.function.Predicate;

import com.baomidou.mybatisplus.core.metadata.TableInfo;
import com.baomidou.mybatisplus.core.metadata.TableInfoHelper;
import com.baomidou.mybatisplus.core.toolkit.LambdaUtils;
import com.baomidou.mybatisplus.core.toolkit.support.ColumnCache;
import com.baomidou.mybatisplus.core.toolkit.support.SFunction;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.database.context.DatabaseContextHolder;
import com.google.common.collect.Maps;
import lombok.NonNull;

/**
 * 自定义字段更新工具类
 *
 * @author Woody
 * @date 2024/5/18
 */
public final class Updates {
    private Updates() {
    }

    /**
     * 数据更新条件枚举
     */
    public enum Condition {
        /**
         * 大于
         */
        GT,

        /**
         * 小于
         */
        LT,

        /**
         * 不等于
         */
        NE,

        /**
         * 非空
         */
        NONNULL;
    }

    /**
     * 获取字段更新对象
     *
     * @param <T> 数据类型
     * @return 字段更新对象实例
     */
    public static <T> Update<T> update() {
        return new Update<>();
    }

    /**
     * 更新所有字段
     *
     * @param clazz 对象类型
     * @param <T>   数据类型
     * @return 字段更新对象实例
     */
    public static <T> Update<T> all(@NonNull Class<T> clazz) {
        return all(clazz, null, property -> true);
    }

    /**
     * 更新所有字段
     *
     * @param clazz     对象类型
     * @param condition 更新条件
     * @param <T>       数据类型
     * @return 字段更新对象实例
     */
    public static <T> Update<T> all(@NonNull Class<T> clazz, Condition condition) {
        return all(clazz, condition, property -> true);
    }


    /**
     * 更新所有字段
     *
     * @param clazz  对象类型
     * @param filter 过滤函数
     * @param <T>    数据类型
     * @return 字段更新对象实例
     */
    public static <T> Update<T> all(@NonNull Class<T> clazz, @NonNull Predicate<String> filter) {
        return all(clazz, null, filter);
    }

    /**
     * 更新所有字段
     *
     * @param clazz     对象类型
     * @param condition 更新条件
     * @param filter    过滤函数
     * @param <T>       数据类型
     * @return 字段更新对象实例
     */
    public static <T> Update<T> all(@NonNull Class<T> clazz, Condition condition, @NonNull Predicate<String> filter) {
        Update<T> update = update();
        Map<String, ColumnCache> properties = LambdaUtils.getColumnMap(clazz);
        if (ObjectUtils.isEmpty(properties)) {
            return update;
        }
        TableInfo table = TableInfoHelper.getTableInfo(clazz);
        String key = StringUtils.ifEmpty(table.getKeyProperty(), LambdaUtils::formatKey);
        properties.forEach((property, column) -> {
            if (!Objects.equals(property, key) && filter.test(property)) {
                update.use(condition, column.getColumn());
            }
        });
        return update;
    }

    /**
     * 使用
     *
     * @param columns 列名数组
     * @param <T>     数据类型
     * @return 字段更新对象实例
     */
    public static <T> Update<T> use(@NonNull String... columns) {
        return use(null, columns);
    }

    /**
     * 使用
     *
     * @param condition 使用条件
     * @param columns   列名数组
     * @param <T>       数据类型
     * @return 字段更新对象实例
     */
    public static <T> Update<T> use(Condition condition, @NonNull String... columns) {
        Update<T> update = update();
        return update.use(condition, columns);
    }

    /**
     * 使用
     *
     * @param getters Getter方法数组
     * @param <T>     数据类型
     * @return 字段更新对象实例
     */
    @SafeVarargs
    public static <T> Update<T> use(@NonNull SFunction<T, ?>... getters) {
        return use(null, getters);
    }

    /**
     * 使用
     *
     * @param condition 使用条件
     * @param getters   Getter方法数组
     * @param <T>       数据类型
     * @return 字段更新对象实例
     */
    @SafeVarargs
    public static <T> Update<T> use(Condition condition, @NonNull SFunction<T, ?>... getters) {
        Update<T> update = update();
        return update.use(condition, getters);
    }

    /**
     * 相加
     *
     * @param columns 列名数组
     * @param <T>     数据类型
     * @return 字段更新对象实例
     */
    public static <T> Update<T> add(@NonNull String... columns) {
        Update<T> update = update();
        return update.add(columns);
    }

    /**
     * 相加
     *
     * @param getters Getter方法数组
     * @param <T>     数据类型
     * @return 字段更新对象实例
     */
    @SafeVarargs
    public static <T> Update<T> add(@NonNull SFunction<T, ?>... getters) {
        Update<T> update = update();
        return update.add(getters);
    }

    /**
     * 清空
     *
     * @param columns 列名数组
     * @param <T>     数据类型
     * @return 字段更新对象实例
     */
    public static <T> Update<T> clear(@NonNull String... columns) {
        Update<T> update = update();
        return update.clear(columns);
    }

    /**
     * 清空
     *
     * @param getters Getter方法数组
     * @param <T>     数据类型
     * @return 字段更新对象实例
     */
    @SafeVarargs
    public static <T> Update<T> clear(@NonNull SFunction<T, ?>... getters) {
        Update<T> update = update();
        return update.clear(getters);
    }

    /**
     * 相减
     *
     * @param columns 列名数组
     * @param <T>     数据类型
     * @return 字段更新对象实例
     */
    public static <T> Update<T> subtract(@NonNull String... columns) {
        Update<T> update = update();
        return update.subtract(columns);
    }

    /**
     * 相减
     *
     * @param getters Getter方法数组
     * @param <T>     数据类型
     * @return 字段更新对象实例
     */
    @SafeVarargs
    public static <T> Update<T> subtract(@NonNull SFunction<T, ?>... getters) {
        Update<T> update = update();
        return update.subtract(getters);
    }

    /**
     * 相乘
     *
     * @param columns 列名数组
     * @param <T>     数据类型
     * @return 字段更新对象实例
     */
    public static <T> Update<T> multiply(@NonNull String... columns) {
        Update<T> update = update();
        return update.multiply(columns);
    }

    /**
     * 相乘
     *
     * @param getters Getter方法数组
     * @param <T>     数据类型
     * @return 字段更新对象实例
     */
    @SafeVarargs
    public static <T> Update<T> multiply(@NonNull SFunction<T, ?>... getters) {
        Update<T> update = update();
        return update.multiply(getters);
    }

    /**
     * 相除
     *
     * @param columns 列名数组
     * @param <T>     数据类型
     * @return 字段更新对象实例
     */
    public static <T> Update<T> divide(@NonNull String... columns) {
        Update<T> update = update();
        return update.divide(columns);
    }

    /**
     * 相除
     *
     * @param getters Getter方法数组
     * @param <T>     数据类型
     * @return 字段更新对象实例
     */
    @SafeVarargs
    public static <T> Update<T> divide(@NonNull SFunction<T, ?>... getters) {
        Update<T> update = update();
        return update.divide(getters);
    }

    /**
     * JSON合并
     *
     * @param columns 列名数组
     * @param <T>     数据类型
     * @return 字段更新对象实例
     */
    public static <T> Update<T> jsonMerge(@NonNull String... columns) {
        Update<T> update = update();
        return update.jsonMerge(columns);
    }

    /**
     * JSON合并
     *
     * @param getters Getter方法数组
     * @param <T>     数据类型
     * @return 字段更新对象实例
     */
    @SafeVarargs
    public static <T> Update<T> jsonMerge(@NonNull SFunction<T, ?>... getters) {
        Update<T> update = update();
        return update.jsonMerge(getters);
    }

    /**
     * JSON长度计算
     *
     * @param column 列名
     * @param <T>    数据类型
     * @param target 计算长度目标列名
     * @return 字段更新对象实例
     */
    public static <T> Update<T> jsonLength(@NonNull String column, @NonNull String target) {
        Update<T> update = update();
        return update.jsonLength(column, target);
    }

    /**
     * JSON长度计算
     *
     * @param getter Getter方法
     * @param target 计算长度目标Getter方法
     * @param <T>    数据类型
     * @return 字段更新对象实例
     */
    public static <T> Update<T> jsonLength(@NonNull SFunction<T, ?> getter, @NonNull SFunction<T, ?> target) {
        Update<T> update = update();
        return update.jsonLength(getter, target);
    }

    /**
     * 设置更新表达式
     *
     * @param column 列名
     * @param value  值
     * @param <T>    数据类型
     * @return 字段更新对象实例
     */
    public static <T> Update<T> set(@NonNull String column, @NonNull String value) {
        Update<T> update = update();
        return update.set(column, value);
    }

    /**
     * 字段更新对象
     *
     * @param <T> 数据类型
     */
    public static class Update<T> implements Serializable {
        private final Map<String, String> expressions = Maps.newLinkedHashMap();

        /**
         * 使用
         *
         * @param columns 列名数组
         * @return 字段更新对象实例
         */
        public final Update<T> use(@NonNull String... columns) {
            return use(null, columns);
        }

        /**
         * 使用
         *
         * @param condition 使用条件
         * @param columns   列名数组
         * @return 字段更新对象实例
         */
        public final Update<T> use(Condition condition, @NonNull String... columns) {
            for (String column : columns) {
                String value = "VALUES(" + column + ")";
                if (condition == Condition.GT) {
                    this.set(column, "IF(" + value + " > " + column + ", " + value + ", " + column + ")");
                } else if (condition == Condition.LT) {
                    this.set(column, "IF(" + value + " < " + column + ", " + value + ", " + column + ")");
                } else if (condition == Condition.NE) {
                    this.set(column, "IF(" + value + " != " + column + ", " + value + ", " + column + ")");
                } else if (condition == Condition.NONNULL) {
                    this.set(column, "IFNULL(" + value + ", " + column + ")");
                } else {
                    this.set(column, value);
                }
            }
            return this;
        }

        /**
         * 使用
         *
         * @param getters Getter方法数组
         * @return 字段更新对象实例
         */
        @SafeVarargs
        public final Update<T> use(@NonNull SFunction<T, ?>... getters) {
            return use(null, getters);
        }

        /**
         * 使用
         *
         * @param condition 使用条件
         * @param getters   Getter方法数组
         * @return 字段更新对象实例
         */
        @SafeVarargs
        public final Update<T> use(Condition condition, @NonNull SFunction<T, ?>... getters) {
            for (SFunction<T, ?> getter : getters) {
                this.use(condition, DatabaseContextHolder.getter2column(getter));
            }
            return this;
        }

        /**
         * 相加
         *
         * @param columns 列名数组
         * @return 字段更新对象实例
         */
        public final Update<T> add(@NonNull String... columns) {
            for (String column : columns) {
                this.set(column, column + " + VALUES(" + column + ")");
            }
            return this;
        }

        /**
         * 相加
         *
         * @param getters Getter方法数组
         * @return 字段更新对象实例
         */
        @SafeVarargs
        public final Update<T> add(@NonNull SFunction<T, ?>... getters) {
            for (SFunction<T, ?> getter : getters) {
                this.add(DatabaseContextHolder.getter2column(getter));
            }
            return this;
        }

        /**
         * 清空
         *
         * @param columns 列名数组
         * @return 字段更新对象实例
         */
        public final Update<T> clear(@NonNull String... columns) {
            for (String column : columns) {
                this.set(column, "null");
            }
            return this;
        }

        /**
         * 清空
         *
         * @param getters Getter方法数组
         * @return 字段更新对象实例
         */
        @SafeVarargs
        public final Update<T> clear(@NonNull SFunction<T, ?>... getters) {
            for (SFunction<T, ?> getter : getters) {
                this.clear(DatabaseContextHolder.getter2column(getter));
            }
            return this;
        }

        /**
         * 相减
         *
         * @param columns 列名数组
         * @return 字段更新对象实例
         */
        public final Update<T> subtract(@NonNull String... columns) {
            for (String column : columns) {
                this.set(column, column + " - VALUES(" + column + ")");
            }
            return this;
        }

        /**
         * 相减
         *
         * @param getters Getter方法数组
         * @return 字段更新对象实例
         */
        @SafeVarargs
        public final Update<T> subtract(@NonNull SFunction<T, ?>... getters) {
            for (SFunction<T, ?> getter : getters) {
                this.subtract(DatabaseContextHolder.getter2column(getter));
            }
            return this;
        }

        /**
         * 相乘
         *
         * @param columns 列名数组
         * @return 字段更新对象实例
         */
        public final Update<T> multiply(@NonNull String... columns) {
            for (String column : columns) {
                this.set(column, column + " * VALUES(" + column + ")");
            }
            return this;
        }

        /**
         * 相乘
         *
         * @param getters Getter方法数组
         * @return 字段更新对象实例
         */
        @SafeVarargs
        public final Update<T> multiply(@NonNull SFunction<T, ?>... getters) {
            for (SFunction<T, ?> getter : getters) {
                this.multiply(DatabaseContextHolder.getter2column(getter));
            }
            return this;
        }

        /**
         * 相除
         *
         * @param columns 列名数组
         * @return 字段更新对象实例
         */
        public final Update<T> divide(@NonNull String... columns) {
            for (String column : columns) {
                this.set(column, column + " / VALUES(" + column + ")");
            }
            return this;
        }

        /**
         * 相除
         *
         * @param getters Getter方法数组
         * @return 字段更新对象实例
         */
        @SafeVarargs
        public final Update<T> divide(@NonNull SFunction<T, ?>... getters) {
            for (SFunction<T, ?> getter : getters) {
                this.divide(DatabaseContextHolder.getter2column(getter));
            }
            return this;
        }

        /**
         * JSON合并
         *
         * @param columns 列名数组
         * @return 字段更新对象实例
         */
        public final Update<T> jsonMerge(@NonNull String... columns) {
            for (String column : columns) {
                this.set(column, "JSON_MERGE_PATCH(" + column + ", VALUES(" + column + "))");
            }
            return this;
        }

        /**
         * JSON合并
         *
         * @param getters Getter方法数组
         * @return 字段更新对象实例
         */
        @SafeVarargs
        public final Update<T> jsonMerge(@NonNull SFunction<T, ?>... getters) {
            for (SFunction<T, ?> getter : getters) {
                this.jsonMerge(DatabaseContextHolder.getter2column(getter));
            }
            return this;
        }

        /**
         * JSON长度计算
         *
         * @param column 列名
         * @param target 计算长度目标列名
         * @return 字段更新对象实例
         */
        public final Update<T> jsonLength(@NonNull String column, @NonNull String target) {
            return this.set(column, "JSON_LENGTH(" + target + ")");
        }

        /**
         * JSON长度计算
         *
         * @param getter Getter方法
         * @param target 计算长度目标Getter方法
         * @return 字段更新对象实例
         */
        public final Update<T> jsonLength(@NonNull SFunction<T, ?> getter, @NonNull SFunction<T, ?> target) {
            return this.jsonLength(
                    DatabaseContextHolder.getter2column(getter), DatabaseContextHolder.getter2column(target)
            );
        }

        /**
         * 设置更新表达式
         *
         * @param column     列名
         * @param expression 值表达式
         * @return 字段更新对象实例
         */
        public final Update<T> set(@NonNull String column, @NonNull String expression) {
            this.expressions.put(column, expression);
            return this;
        }

        /**
         * 获取更新字段/值表达式映射表
         *
         * @return 字段/值表达式映射表
         */
        public final Map<String, String> getExpressions() {
            return Collections.unmodifiableMap(this.expressions);
        }

        @Override
        public String toString() {
            return this.expressions.toString();
        }
    }
}
