package com.chelvc.framework.dubbo.interceptor;

import com.chelvc.framework.base.context.LoggingContextHolder;
import com.chelvc.framework.base.context.Session;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.dubbo.adapter.DubboObjectAdapterHolder;
import com.chelvc.framework.dubbo.adapter.DubboObjectAdapterWrapper;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.common.constants.CommonConstants;
import org.apache.dubbo.common.extension.Activate;
import org.apache.dubbo.rpc.AsyncRpcResult;
import org.apache.dubbo.rpc.Filter;
import org.apache.dubbo.rpc.Invocation;
import org.apache.dubbo.rpc.Invoker;
import org.apache.dubbo.rpc.Result;
import org.apache.dubbo.rpc.RpcContext;
import org.apache.dubbo.rpc.RpcException;

/**
 * Dubbo 生产者处理拦截器
 *
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
@Activate(group = CommonConstants.PROVIDER)
public class DubboProviderInterceptor implements Filter {
    /**
     * 接口调用处理
     *
     * @param invoker    接口调用器
     * @param invocation 调用信息
     * @return 调用结果
     * @throws RpcException 接口调用异常
     */
    private Result processing(Invoker<?> invoker, Invocation invocation) throws RpcException {
        // 获取接口调用结果
        Result result = invoker.invoke(invocation);
        Object value = result.getValue();
        if (value == null) {
            return result;
        }

        // 自定义对象包装
        DubboObjectAdapterWrapper<Object, ?> wrapper = DubboObjectAdapterHolder.lookupObjectAdapter(value.getClass());
        if (wrapper == null) {
            return result;
        }
        value = wrapper.getAdapter().wrap(result.getValue());
        return AsyncRpcResult.newDefaultAsyncResult(value, invocation);
    }

    @Override
    public Result invoke(Invoker<?> invoker, Invocation invocation) throws RpcException {
        Session session = (Session) RpcContext.getServerAttachment().getObjectAttachment(Session.NAMING);
        SessionContextHolder.setSession(session);
        try {
            boolean debug = log.isDebugEnabled();
            if (debug) {
                log.debug(LoggingContextHolder.message(invocation.getMethodName(), null, invocation.getArguments()));
            }
            Result result = this.processing(invoker, invocation);
            if (debug) {
                log.debug(LoggingContextHolder.message(invocation.getMethodName(), null, result.getValue()));
            }
            return result;
        } finally {
            SessionContextHolder.clearSessionContext();
        }
    }
}
