package com.chelvc.framework.email.context;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.net.URL;
import java.nio.charset.StandardCharsets;
import java.util.LinkedList;
import java.util.List;
import java.util.regex.Matcher;
import javax.activation.DataHandler;
import javax.activation.DataSource;
import javax.activation.FileDataSource;
import javax.activation.FileTypeMap;
import javax.mail.MessagingException;
import javax.mail.Multipart;
import javax.mail.internet.MimeBodyPart;
import javax.mail.internet.MimeMultipart;
import javax.mail.internet.MimeUtility;
import javax.mail.util.ByteArrayDataSource;

import ch.qos.logback.core.util.ContentTypeUtil;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.email.Body;
import lombok.NonNull;

/**
 * 邮件工具类
 *
 * @author Woody
 * @date 2024/4/3
 */
public final class EmailContextHolder {
    /**
     * 默认内容类型
     */
    public static final String DEFAULT_MIME_TYPE = "text/plain";

    static {
        FileTypeMap.setDefaultFileTypeMap(FileTypeMap.getDefaultFileTypeMap());
    }

    private EmailContextHolder() {
    }

    /**
     * 将内容转换成邮件内容体
     *
     * @param body 邮件体
     * @return 邮件内容体
     */
    public static Multipart part(@NonNull Body body) {
        // 构建邮件内容部分
        MimeMultipart part = new MimeMultipart();
        MimeBodyPart content = new MimeBodyPart();
        String type = StringUtils.ifEmpty(body.getType(), DEFAULT_MIME_TYPE);
        String text = ObjectUtils.ifNull(body.getContent(), StringUtils.EMPTY);
        try {
            if (ContentTypeUtil.isTextual(type)) {
                content.setText(text, StandardCharsets.UTF_8.name(), ContentTypeUtil.getSubType(type));
            } else {
                content.setContent(text, type);
            }
            part.addBodyPart(content);
            if (StringUtils.isEmpty(text)) {
                return part;
            }

            // 关联文本内容里面的图片、链接
            Matcher matcher = StringUtils.getPattern(StringUtils.IMAGE_REGEX).matcher(text);
            List<MimeBodyPart> relates = new LinkedList<>();
            while (matcher.find()) {
                String img = matcher.group();
                int i = img.indexOf("\"", img.indexOf(" src")) + 1;
                String url = img.substring(i, img.indexOf("\"", i)).trim();
                if (url.isEmpty() || !StringUtils.getPattern(StringUtils.URL_REGEX).matcher(url).matches()) {
                    continue;
                }
                String name = new File(url).getName();
                text = text.replaceAll(url, "cid:" + name);

                DataHandler handler = new DataHandler(new DataSource() {
                    @Override
                    public InputStream getInputStream() throws IOException {
                        return new URL(url).openStream();
                    }

                    @Override
                    public OutputStream getOutputStream() {
                        return null;
                    }

                    @Override
                    public String getContentType() {
                        return FileTypeMap.getDefaultFileTypeMap().getContentType(name);
                    }

                    @Override
                    public String getName() {
                        return name;
                    }
                });

                MimeBodyPart image = new MimeBodyPart();
                image.setDataHandler(handler);
                image.setContentID(name);
                relates.add(image);
            }
            if (!relates.isEmpty()) {
                for (MimeBodyPart relate : relates) {
                    part.addBodyPart(relate);
                }
                part.setSubType("related");
            }
        } catch (MessagingException e) {
            throw new RuntimeException(e);
        }
        return part;
    }

    /**
     * 将文件转换成邮件附件部分
     *
     * @param file 文件对象
     * @return 邮件混合部分
     */
    public static MimeBodyPart part(@NonNull File file) {
        return part(file, file.getName());
    }

    /**
     * 将文件转换成邮件附件部分
     *
     * @param file    文件对象
     * @param comment 文件说明
     * @return 邮件混合部分
     */
    public static MimeBodyPart part(@NonNull File file, @NonNull String comment) {
        MimeBodyPart body = new MimeBodyPart();
        try {
            body.setDataHandler(new DataHandler(new FileDataSource(file)));
            body.setFileName(MimeUtility.encodeText(comment));
        } catch (MessagingException | UnsupportedEncodingException e) {
            throw new RuntimeException(e);
        }
        return body;
    }

    /**
     * 将文件转换成邮件附件部分
     *
     * @param files 文件对象数组
     * @return 邮件混合部分数组
     */
    public static MimeBodyPart[] part(@NonNull File... files) {
        MimeBodyPart[] mixes = new MimeBodyPart[files.length];
        for (int i = 0; i < files.length; i++) {
            mixes[i] = part(files[i]);
        }
        return mixes;
    }

    /**
     * 将文件流转换成邮件附件部分
     *
     * @param stream  文件流对象
     * @param type    文件类型
     * @param comment 文件说明
     * @return 邮件混合部分
     */
    public static MimeBodyPart part(@NonNull InputStream stream, @NonNull String type, @NonNull String comment) {
        MimeBodyPart body = new MimeBodyPart();
        try {
            DataSource dataSource = new ByteArrayDataSource(stream, type);
            DataHandler dataHandler = new DataHandler(dataSource);
            body.setDataHandler(dataHandler);
            body.setFileName(MimeUtility.encodeText(comment));
        } catch (IOException | MessagingException e) {
            throw new RuntimeException(e);
        }
        return body;
    }

    /**
     * 设置邮件混合文件数据源
     *
     * @param body  邮件体
     * @param files 文件对象集合
     * @return 邮件消息体对象
     */
    public static Multipart mixed(@NonNull Body body, @NonNull File... files) {
        return mixed(body, part(files));
    }

    /**
     * 设置邮件混合文件数据源
     *
     * @param body  邮件体
     * @param mixes 混合对象数组
     * @return 邮件消息体对象
     */
    public static Multipart mixed(@NonNull Body body, @NonNull MimeBodyPart... mixes) {
        Multipart part = part(body);
        if (mixes.length == 0) {
            return part;
        }

        MimeMultipart mixed = new MimeMultipart();
        MimeBodyPart content = new MimeBodyPart();
        try {
            content.setContent(part);
            mixed.addBodyPart(content);
            mixed.setSubType("mixed");
            for (MimeBodyPart mixe : mixes) {
                mixed.addBodyPart(mixe);
            }
        } catch (MessagingException e) {
            throw new RuntimeException(e);
        }
        return mixed;
    }
}
