package com.chelvc.framework.feign.config;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.feign.encoder.CustomizeEncoder;
import com.chelvc.framework.feign.encoder.CustomizeQueryEncoder;
import com.chelvc.framework.feign.interceptor.FeignFailureInterceptor;
import com.chelvc.framework.feign.interceptor.FeignInvokeInterceptor;
import feign.QueryMapEncoder;
import feign.codec.Decoder;
import feign.codec.Encoder;
import feign.codec.ErrorDecoder;
import feign.optionals.OptionalDecoder;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.ObjectFactory;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.boot.autoconfigure.AutoConfigureBefore;
import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingBean;
import org.springframework.boot.autoconfigure.http.HttpMessageConverters;
import org.springframework.cloud.openfeign.FeignAutoConfiguration;
import org.springframework.cloud.openfeign.support.ResponseEntityDecoder;
import org.springframework.cloud.openfeign.support.SpringDecoder;
import org.springframework.cloud.openfeign.support.SpringEncoder;
import org.springframework.context.ApplicationContext;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Primary;
import org.springframework.context.annotation.Scope;

/**
 * Feign配置
 *
 * @author Woody
 * @date 2024/1/30
 */
@Configuration
@AutoConfigureBefore(FeignAutoConfiguration.class)
public class FeignConfigurer {
    /**
     * 定制消息转换处理器
     *
     * @param converters 消息转换处理器实例
     * @return 消息转换处理器实例
     */
    private ObjectFactory<HttpMessageConverters> converters(HttpMessageConverters converters) {
        return new ObjectFactory<HttpMessageConverters>() {
            private final HttpMessageConverters instance =
                    new HttpMessageConverters(ApplicationContextHolder.converters(converters.getConverters()));

            @Override
            public HttpMessageConverters getObject() throws BeansException {
                return this.instance;
            }
        };
    }

    @Bean
    public QueryMapEncoder queryEncoder() {
        return new CustomizeQueryEncoder();
    }

    @Bean
    @Primary
    @Scope(BeanDefinition.SCOPE_PROTOTYPE)
    public Encoder encoder(HttpMessageConverters converters) {
        return new CustomizeEncoder(new SpringEncoder(this.converters(converters)));
    }

    @Bean
    @ConditionalOnMissingBean(Decoder.class)
    public Decoder decoder(HttpMessageConverters converters) {
        return new OptionalDecoder(new ResponseEntityDecoder(new SpringDecoder(this.converters(converters))));
    }

    @Bean
    @ConditionalOnMissingBean(ErrorDecoder.class)
    public ErrorDecoder errorDecoder() {
        return new FeignFailureInterceptor();
    }

    @Bean
    @ConditionalOnMissingBean(FeignInvokeInterceptor.class)
    public FeignInvokeInterceptor feignInvokeInterceptor(ApplicationContext applicationContext) {
        return new FeignInvokeInterceptor(applicationContext);
    }
}
