package com.chelvc.framework.feign.interceptor;

import java.net.URI;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Set;
import java.util.function.Predicate;

import com.chelvc.framework.base.config.MultiserverMvcConfigurer;
import com.chelvc.framework.base.context.Session;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.base.context.Using;
import com.chelvc.framework.base.util.HttpUtils;
import com.chelvc.framework.common.model.Platform;
import com.chelvc.framework.common.model.Terminal;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.feign.config.FeignProperties;
import com.google.common.collect.Lists;
import feign.RequestInterceptor;
import feign.RequestTemplate;
import feign.Target;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.ApplicationContext;
import org.springframework.web.servlet.mvc.method.annotation.RequestMappingHandlerMapping;

/**
 * Feign调用处理拦截器
 *
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
public class FeignInvokeInterceptor implements RequestInterceptor {
    private final Set<String> servers;
    private final Map<String, ?> mappings;
    private final List<FeignProperties.Forward> forwards;

    @SuppressWarnings("unchecked")
    public FeignInvokeInterceptor(@NonNull ApplicationContext applicationContext) {
        RequestMappingHandlerMapping handlerMapping = applicationContext.getBean(RequestMappingHandlerMapping.class);
        Map<String, Predicate<Class<?>>> prefixes = handlerMapping.getPathPrefixes();
        this.servers = ObjectUtils.isEmpty(prefixes) ? Collections.emptySet() : prefixes.keySet();
        Object registry = ObjectUtils.getObjectValue(handlerMapping, "mappingRegistry");
        Map<?, ?> mappings = (Map<?, ?>) ObjectUtils.getObjectValue(registry, "urlLookup");
        this.mappings = ObjectUtils.isEmpty(mappings) ? Collections.emptyMap() : (Map<String, ?>) mappings;

        // 初始化请求转发配置
        FeignProperties properties = applicationContext.getBean(FeignProperties.class);
        List<FeignProperties.Forward> forwards = Lists.newArrayList(properties.getForwards());
        if (applicationContext.containsBean(MultiserverMvcConfigurer.class.getName())
                && forwards.stream().noneMatch(forward -> Objects.equals(forward.getSource(), "*"))) {
            FeignProperties.Forward forward = new FeignProperties.Forward();
            forward.setSource("*");
            forward.setPrefixed(true);
            forwards.add(forward);
        }
        this.forwards = ObjectUtils.isEmpty(forwards) ? Collections.emptyList() : forwards;
    }

    /**
     * 请求转发
     *
     * @param template 请求模版对象实例
     */
    protected void forward(@NonNull RequestTemplate template) {
        Target<?> target = template.feignTarget();
        String server = target.name();
        for (FeignProperties.Forward forward : this.forwards) {
            String source = forward.getSource(), mapping = forward.getTarget();
            if (ObjectUtils.equals(source, server) || ObjectUtils.equals(source, "*")) {
                boolean internal = this.servers.contains(server);
                if (internal || forward.isPrefixed()) {
                    String uri = HttpUtils.uri(server, template.path());
                    if (!internal || this.mappings.containsKey(uri)) {
                        template.uri(uri);
                    }
                }
                if (StringUtils.notEmpty(mapping) && !Objects.equals(mapping, server)) {
                    URI uri = URI.create(target.url());
                    String url = uri.getScheme() + "://" + mapping;
                    template.target(url);
                    template.feignTarget(new Target.HardCodedTarget<>(target.type(), mapping, url));
                }
                break;
            }
        }
    }

    /**
     * 初始化请求头
     *
     * @param template 请求模版对象实例
     */
    protected void initializeRequestHeader(@NonNull RequestTemplate template) {
        Session session = SessionContextHolder.getSession(false);
        if (session == null) {
            return;
        }
        template.header("x-real-ip", session.getHost());
        template.header(SessionContextHolder.HEADER_ID, (String) ObjectUtils.ifNull(session.getId(), String::valueOf));
        template.header(SessionContextHolder.HEADER_USING, ObjectUtils.ifNull(session.getUsing(), Using::name));
        template.header(SessionContextHolder.HEADER_SCOPE, session.getScope());
        template.header(SessionContextHolder.HEADER_MOBILE, session.getMobile());
        template.header(SessionContextHolder.HEADER_DEVICE, session.getDevice());
        template.header(SessionContextHolder.HEADER_CHANNEL, session.getChannel());
        template.header(SessionContextHolder.HEADER_PLATFORM,
                ObjectUtils.ifNull(session.getPlatform(), Platform::name));
        template.header(SessionContextHolder.HEADER_TERMINAL,
                ObjectUtils.ifNull(session.getTerminal(), Terminal::name));
        template.header(SessionContextHolder.HEADER_VERSION, session.getVersion());
        template.header(SessionContextHolder.HEADER_TIMESTAMP,
                (String) ObjectUtils.ifNull(session.getTimestamp(), String::valueOf));
        template.header(SessionContextHolder.HEADER_REGISTERING,
                (String) ObjectUtils.ifNull(session.getRegistering(), String::valueOf));
        template.header(SessionContextHolder.HEADER_AUTHORITIES, session.getAuthorities());
    }

    @Override
    public void apply(RequestTemplate template) {
        this.initializeRequestHeader(template);

        if (ObjectUtils.notEmpty(this.forwards)) {
            this.forward(template);
        }

        if (log.isDebugEnabled()) {
            log.debug("Feign request: {}", template);
        }
    }
}
