package com.chelvc.framework.group.context;

import java.util.Map;
import java.util.Set;
import java.util.function.Function;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.base.context.Session;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.common.model.Caps;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.group.GroupStore;
import com.fasterxml.jackson.core.type.TypeReference;
import lombok.NonNull;

/**
 * 场景分组上下文工具类
 *
 * @author Woody
 * @date 2024/12/5
 */
public final class GroupContextHolder {
    /**
     * 场景分组存储器
     */
    private static volatile GroupStore GROUP_STORE;

    /**
     * 场景分组模值映射类型
     */
    private static final TypeReference<Map<Caps, Set<Integer>>> GROUP_MOLD_TYPE =
            new TypeReference<Map<Caps, Set<Integer>>>() {
            };

    private GroupContextHolder() {
    }

    /**
     * 获取场景分组存储器
     *
     * @return 场景分组存储器
     */
    public static GroupStore getStore() {
        if (GROUP_STORE == null) {
            synchronized (GroupContextHolder.class) {
                if (GROUP_STORE == null) {
                    GROUP_STORE = ApplicationContextHolder.getBean(GroupStore.class);
                }
            }
        }
        return GROUP_STORE;
    }

    /**
     * 获取配置分组
     *
     * @param scene 分组场景
     * @return 分组标识
     */
    public static Caps getConfig(@NonNull String scene) {
        return ApplicationContextHolder.getProperty("business.group." + scene, Caps.class);
    }

    /**
     * 获取配置分组
     *
     * @param property 配置属性
     * @param mold     分组模值
     * @return 分组标识
     */
    public static Caps getConfig(@NonNull String property, int mold) {
        return getConfig(property, mold, group -> group);
    }

    /**
     * 获取配置分组
     *
     * @param property 配置属性
     * @param mold     分组模值
     * @param function 分组回调函数
     * @return 分组标识
     */
    public static Caps getConfig(@NonNull String property, int mold, @NonNull Function<Caps, Caps> function) {
        Map<Caps, Set<Integer>> mapping = ApplicationContextHolder.getProperty(property, GROUP_MOLD_TYPE);
        if (ObjectUtils.notEmpty(mapping)) {
            for (Map.Entry<Caps, Set<Integer>> entry : mapping.entrySet()) {
                if (ObjectUtils.notEmpty(entry.getValue()) && entry.getValue().contains(mold)) {
                    return function.apply(entry.getKey());
                }
            }
        }
        return null;
    }

    /**
     * 获取场景分组
     *
     * @param scene 分组场景
     * @return 分组标识
     */
    public static Caps getGroup(@NonNull String scene) {
        Caps group = getConfig(scene);
        if (group != null) {
            return group;
        }

        Session session = SessionContextHolder.getSession(false);
        if (session == null) {
            return null;
        }
        if ((group = session.getGroup(scene)) == null) {
            GroupStore store = getStore();
            group = ObjectUtils.ifNull(store.get(scene), Caps.NONE);
            session.setGroup(scene, group);
        }
        return group == Caps.NONE ? null : group;
    }

    /**
     * 设置场景分组
     *
     * @param scene 分组场景
     * @param group 分组标识
     */
    public static void setGroup(@NonNull String scene, @NonNull Caps group) {
        Session session = SessionContextHolder.getSession(false);
        if (session != null) {
            session.setGroup(scene, group);
        }
    }
}
