package com.chelvc.framework.kafka.context;

import java.lang.reflect.Type;
import java.nio.charset.StandardCharsets;
import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Objects;
import java.util.concurrent.ExecutionException;
import java.util.function.BiFunction;
import java.util.function.Consumer;
import java.util.stream.Collectors;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.base.context.Session;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.common.util.JacksonUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.kafka.config.KafkaProperties;
import lombok.NonNull;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.apache.kafka.common.header.Header;
import org.apache.kafka.common.header.Headers;
import org.springframework.kafka.core.KafkaTemplate;

/**
 * Kafka上下文操作工具类
 *
 * @author Woody
 * @date 2024/1/30
 */
public final class KafkaContextHolder {
    /**
     * 配置属性
     */
    private static volatile KafkaProperties PROPERTIES;

    /**
     * 模版实例
     */
    private static volatile KafkaTemplate<?, ?> KAFKA_TEMPLATE;

    private KafkaContextHolder() {
    }

    /**
     * 获取配置属性
     *
     * @return 配置属性实例
     */
    private static KafkaProperties getProperties() {
        if (PROPERTIES == null) {
            synchronized (KafkaProperties.class) {
                if (PROPERTIES == null) {
                    PROPERTIES = ApplicationContextHolder.getBean(KafkaProperties.class);
                }
            }
        }
        return PROPERTIES;
    }

    /**
     * 获取Kafka模版实例
     *
     * @param <K> 顺序标识类型
     * @param <V> 消息载体类型
     * @return Kafka模版实例
     */
    @SuppressWarnings("unchecked")
    public static <K, V> KafkaTemplate<K, V> getKafkaTemplate() {
        if (KAFKA_TEMPLATE == null) {
            synchronized (KafkaTemplate.class) {
                if (KAFKA_TEMPLATE == null) {
                    KAFKA_TEMPLATE = ApplicationContextHolder.getBean(KafkaTemplate.class);
                }
            }
        }
        return (KafkaTemplate<K, V>) KAFKA_TEMPLATE;
    }

    /**
     * 环境隔离
     *
     * @param original 原始标记
     * @return 环境隔离标记
     */
    public static String isolate(@NonNull String original) {
        String namespace = getProperties().getNamespace();
        return StringUtils.isEmpty(namespace) ? original : (namespace + original);
    }

    /**
     * 从消息头中获取会话信息
     *
     * @param headers 消息头对象实例
     * @return 会话实例
     */
    public static Session getSession(@NonNull Headers headers) {
        Iterator<Header> iterator = headers.headers(Session.NAMING).iterator();
        if (iterator.hasNext()) {
            return JacksonUtils.deserialize(iterator.next().value(), Session.class);
        }
        return null;
    }

    /**
     * 初始化会话信息
     *
     * @param headers 消息头对象实例
     */
    public static void initializeSession(@NonNull Headers headers) {
        Session session = SessionContextHolder.getSession(false);
        if (session != null) {
            headers.add(Session.NAMING, JacksonUtils.serialize(session).getBytes(StandardCharsets.UTF_8));
        }
    }

    /**
     * 消息反序列化
     *
     * @param record 消息记录
     * @param type   消息类型
     * @param <K>    顺序标识类型
     * @param <V>    消息载体类型
     * @return 消息体实例
     */
    public static <K, V> Object deserialize(@NonNull ConsumerRecord<K, V> record, @NonNull Type type) {
        Object value = record.value();
        if (value instanceof byte[]) {
            return JacksonUtils.deserialize((byte[]) value, type);
        } else if (value == null || (value instanceof String && type == String.class)) {
            return value;
        }
        return JacksonUtils.deserialize(String.valueOf(value), type);
    }

    /**
     * 消息反序列化
     *
     * @param record   消息记录
     * @param type     消息类型
     * @param consumer 消息消费者
     * @param <K>      顺序标识类型
     * @param <V>      消息载体类型
     */
    public static <K, V> void deserialize(@NonNull ConsumerRecord<K, V> record, @NonNull Type type,
                                          @NonNull Consumer<Object> consumer) {
        Object payload = deserialize(record, type);
        Session session = getSession(record.headers());
        SessionContextHolder.setSession(session);
        try {
            consumer.accept(payload);
        } finally {
            SessionContextHolder.clearSessionContext();
        }
    }

    /**
     * 消息反序列化
     *
     * @param records  消息记录集合
     * @param type     消息类型
     * @param function 消息处理函数
     * @param <K>      顺序标识类型
     * @param <V>      消息载体类型
     * @param <T>      消息体类型
     * @return 消息列表
     */
    public static <K, V, T> List<T> deserialize(@NonNull Collection<ConsumerRecord<K, V>> records, @NonNull Type type,
                                                @NonNull BiFunction<Object, Session, T> function) {
        if (ObjectUtils.isEmpty(records)) {
            return Collections.emptyList();
        }
        return records.stream().map(record -> {
            Object payload = deserialize(record, type);
            Session session = getSession(record.headers());
            return function.apply(payload, session);
        }).filter(Objects::nonNull).collect(Collectors.toList());
    }

    /**
     * 发送消息
     *
     * @param topic   消息主题
     * @param payload 消息内容
     * @param <V>     消息载体类型
     */
    public static <V> void send(@NonNull String topic, @NonNull V payload) {
        KafkaTemplate<?, V> template = getKafkaTemplate();
        try {
            template.send(topic, payload).get();
        } catch (InterruptedException | ExecutionException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 发送消息
     *
     * @param topic    消息主题
     * @param payload  消息内容
     * @param ordering 顺序消息标识
     * @param <K>      顺序标识类型
     * @param <V>      消息载体类型
     */
    public static <K, V> void send(@NonNull String topic, @NonNull V payload, K ordering) {
        KafkaTemplate<K, V> template = getKafkaTemplate();
        try {
            template.send(topic, ordering, payload).get();
        } catch (InterruptedException | ExecutionException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 发送消息
     *
     * @param topic     消息主题
     * @param payload   消息内容
     * @param ordering  顺序消息标识
     * @param partition 指定分区
     * @param <K>       顺序标识类型
     * @param <V>       消息载体类型
     */
    public static <K, V> void send(@NonNull String topic, @NonNull V payload, K ordering, Integer partition) {
        KafkaTemplate<K, V> template = getKafkaTemplate();
        try {
            template.send(topic, partition, ordering, payload).get();
        } catch (InterruptedException | ExecutionException e) {
            throw new RuntimeException(e);
        }
    }
}
