package com.chelvc.framework.nacos.context;

import java.util.Collections;
import java.util.Map;
import java.util.Properties;
import java.util.stream.Collectors;

import com.alibaba.boot.nacos.config.properties.NacosConfigProperties;
import com.alibaba.nacos.api.NacosFactory;
import com.alibaba.nacos.api.PropertyKeyConst;
import com.alibaba.nacos.api.common.Constants;
import com.alibaba.nacos.api.config.ConfigService;
import com.alibaba.nacos.api.exception.NacosException;
import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.google.common.collect.ImmutableMap;
import com.google.common.collect.Maps;
import lombok.NonNull;

/**
 * Nacos上下文工具类
 *
 * @author Woody
 * @date 2024/1/30
 */
public final class NacosContextHolder {
    /**
     * 权限配置ID
     */
    public static final String PERMISSION_CONFIG_ID = "permission";

    /**
     * 配置中心业务处理对象实例
     */
    private static volatile ConfigService CONFIG_SERVICE;

    private NacosContextHolder() {
    }

    /**
     * 获取配置ID
     *
     * @return 配置ID
     */
    public static String getConfigId() {
        String id = ApplicationContextHolder.getProperty("nacos.config.id");
        if (StringUtils.isEmpty(id)) {
            String name = ApplicationContextHolder.getApplicationName();
            return AssertUtils.nonempty(name, () -> "Application name is missing");
        }
        return id;
    }

    /**
     * 获取配置分组
     *
     * @return 配置分组
     */
    public static String getConfigGroup() {
        return ApplicationContextHolder.getProperty("nacos.config.group", Constants.DEFAULT_GROUP);
    }

    /**
     * 获取配置中心业务处理对象实例
     *
     * @return 配置中心业务处理对象实例
     */
    public static ConfigService getConfigService() {
        if (CONFIG_SERVICE == null) {
            synchronized (ConfigService.class) {
                if (CONFIG_SERVICE == null) {
                    NacosConfigProperties properties = ApplicationContextHolder.getBean(NacosConfigProperties.class);
                    Properties configuration = new Properties();
                    if (StringUtils.notEmpty(properties.getServerAddr())) {
                        configuration.put(PropertyKeyConst.SERVER_ADDR, properties.getServerAddr());
                    }
                    if (StringUtils.notEmpty(properties.getContextPath())) {
                        configuration.put(PropertyKeyConst.CONTEXT_PATH, properties.getContextPath());
                    }
                    if (StringUtils.notEmpty(properties.getEncode())) {
                        configuration.put(PropertyKeyConst.ENCODE, properties.getEncode());
                    }
                    if (StringUtils.notEmpty(properties.getEndpoint())) {
                        configuration.put(PropertyKeyConst.ENDPOINT, properties.getEndpoint());
                    }
                    if (StringUtils.notEmpty(properties.getNamespace())) {
                        configuration.put(PropertyKeyConst.NAMESPACE, properties.getNamespace());
                    }
                    if (StringUtils.notEmpty(properties.getAccessKey())) {
                        configuration.put(PropertyKeyConst.ACCESS_KEY, properties.getAccessKey());
                    }
                    if (StringUtils.notEmpty(properties.getSecretKey())) {
                        configuration.put(PropertyKeyConst.SECRET_KEY, properties.getSecretKey());
                    }
                    try {
                        CONFIG_SERVICE = NacosFactory.createConfigService(configuration);
                    } catch (NacosException e) {
                        throw new RuntimeException(e);
                    }
                }
            }
        }
        return CONFIG_SERVICE;
    }

    /**
     * 解析配置信息
     *
     * @param config 配置信息
     * @return 键/值映射表
     */
    public static Map<String, String> analyseConfig(String config) {
        if (StringUtils.isEmpty(config)) {
            return Collections.emptyMap();
        }
        char eq = StringUtils.EQUAL.charAt(0);
        String[] lines = config.split(StringUtils.ENTER);
        Map<String, String> configuration = Maps.newHashMapWithExpectedSize(lines.length);
        for (String line : lines) {
            String key, value;
            int index = line.indexOf(eq);
            if (index < 1 || StringUtils.isEmpty(key = line.substring(0, index).trim())
                    || StringUtils.isEmpty(value = line.substring(index + 1).trim())) {
                continue;
            }
            configuration.put(key, value);
        }
        return configuration;
    }

    /**
     * 解析配置信息
     *
     * @param config 键/值映射表
     * @return 配置信息
     */
    public static String analyseConfig(Map<String, String> config) {
        return ObjectUtils.isEmpty(config) ? StringUtils.EMPTY : config.entrySet().stream()
                .filter(entry -> StringUtils.notEmpty(entry.getKey()) && StringUtils.notEmpty(entry.getValue()))
                .map(entry -> entry.getKey() + StringUtils.EQUAL + entry.getValue())
                .collect(Collectors.joining(StringUtils.ENTER));
    }

    /**
     * 获取配置
     *
     * @return 配置键/值映射表
     */
    public static Map<String, String> getConfiguration() {
        return getConfiguration(getConfigId());
    }

    /**
     * 获取配置
     *
     * @param id 配置ID
     * @return 配置键/值映射表
     */
    public static Map<String, String> getConfiguration(@NonNull String id) {
        return getConfiguration(id, getConfigGroup());
    }

    /**
     * 获取配置
     *
     * @param id    配置ID
     * @param group 配置分组
     * @return 配置键/值映射表
     */
    public static Map<String, String> getConfiguration(@NonNull String id, @NonNull String group) {
        String content;
        ConfigService service = getConfigService();
        try {
            content = service.getConfig(id, group, 3000);
        } catch (NacosException e) {
            throw new RuntimeException(e);
        }
        return analyseConfig(content);
    }

    /**
     * 初始化配置信息
     *
     * @param key   配置键
     * @param value 配置值
     */
    public static void initializeConfiguration(@NonNull String key, @NonNull String value) {
        initializeConfiguration(getConfigId(), key, value);
    }

    /**
     * 初始化配置信息
     *
     * @param id    配置ID
     * @param key   配置键
     * @param value 配置值
     */
    public static void initializeConfiguration(@NonNull String id, @NonNull String key, @NonNull String value) {
        initializeConfiguration(id, getConfigGroup(), key, value);
    }

    /**
     * 初始化配置信息
     *
     * @param id    配置ID
     * @param group 配置分组
     * @param key   配置键
     * @param value 配置值
     */
    public static void initializeConfiguration(@NonNull String id, @NonNull String group, @NonNull String key,
                                               @NonNull String value) {
        initializeConfiguration(id, group, ImmutableMap.of(key, value));
    }

    /**
     * 初始化配置信息
     *
     * @param configuration 配置信息键/值映射表
     */
    public static void initializeConfiguration(@NonNull Map<String, String> configuration) {
        initializeConfiguration(getConfigId(), configuration);
    }

    /**
     * 初始化配置信息
     *
     * @param id            配置ID
     * @param configuration 配置信息键/值映射表
     */
    public static void initializeConfiguration(@NonNull String id, @NonNull Map<String, String> configuration) {
        initializeConfiguration(id, getConfigGroup(), configuration);
    }

    /**
     * 初始化配置信息
     *
     * @param id            配置ID
     * @param group         配置分组
     * @param configuration 配置信息键/值映射表
     */
    public static void initializeConfiguration(@NonNull String id, @NonNull String group,
                                               @NonNull Map<String, String> configuration) {
        ConfigService service = getConfigService();
        String content = analyseConfig(configuration);
        try {
            service.publishConfig(id, group, content);
        } catch (NacosException e) {
            throw new RuntimeException(e);
        }
    }
}
