package com.chelvc.framework.redis.context;

import java.time.Duration;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Objects;

import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import lombok.NonNull;
import org.springframework.data.redis.connection.RedisScriptingCommands;
import org.springframework.data.redis.connection.ReturnType;
import org.springframework.data.redis.core.HashOperations;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.script.DigestUtils;

/**
 * Redis Hash操作工具类
 *
 * @author Woody
 * @date 2024/6/28
 */
public final class RedisHashHolder {
    /**
     * Redis带过期时间HMSET脚本
     */
    private static final String SET_DURATION_SCRIPT = "if redis.call('HMSET', KEYS[1], unpack(ARGV, 2)) then " +
            "if redis.call('TTL', KEYS[1]) < tonumber(ARGV[1]) then redis.call('EXPIRE', KEYS[1], ARGV[1]) end " +
            "return 1 end return 0";

    /**
     * Redis带过期时间HINCRBY脚本
     */
    private static final String HINCRBY_DURATION_SCRIPT =
            "local value = redis.call('HINCRBY', KEYS[1], ARGV[1], ARGV[2]) " +
                    "if redis.call('TTL', KEYS[1]) < tonumber(ARGV[3]) then " +
                    "redis.call('EXPIRE', KEYS[1], ARGV[3]) end return value";

    /**
     * Redis带过期时间HMSET脚本SHA
     */
    private static final String SET_DURATION_SCRIPT_SHA = DigestUtils.sha1DigestAsHex(SET_DURATION_SCRIPT);

    /**
     * Redis带过期时间HINCRBY脚本SHA
     */
    private static final String HINCRBY_DURATION_SCRIPT_SHA = DigestUtils.sha1DigestAsHex(HINCRBY_DURATION_SCRIPT);

    private RedisHashHolder() {
    }

    /**
     * 设置Redis Hash结构数据
     *
     * @param template Redis操作模版实例
     * @param args     参数字节二维数组
     * @param <K>      Hash结构名称类型
     * @return true/false
     */
    private static <K> boolean set(@NonNull RedisTemplate<K, ?> template, @NonNull byte[][] args) {
        return RedisContextHolder.execute(template.getConnectionFactory(), connection -> {
            Boolean success;
            RedisScriptingCommands commands = connection.scriptingCommands();
            try {
                success = commands.evalSha(SET_DURATION_SCRIPT_SHA, ReturnType.BOOLEAN, 1, args);
            } catch (Exception e) {
                success = commands.eval(SET_DURATION_SCRIPT.getBytes(), ReturnType.BOOLEAN, 1, args);
            }
            return Boolean.TRUE.equals(success);
        });
    }

    /**
     * 获取Redis Hash结构数据
     *
     * @param name Hash结构名称
     * @param <HK> Hash结构键类型
     * @param <HV> Hash结构值类型
     * @return Hash键/值映射表
     */
    public static <HK, HV> Map<HK, HV> get(@NonNull String name) {
        return get(RedisContextHolder.getRedisTemplate(), name);
    }

    /**
     * 获取Redis Hash结构数据
     *
     * @param template Redis操作模版实例
     * @param name     Hash结构名称
     * @param <K>      Hash结构名称类型
     * @param <HK>     Hash结构键类型
     * @param <HV>     Hash结构值类型
     * @return Hash键/值映射表
     */
    public static <K, HK, HV> Map<HK, HV> get(@NonNull RedisTemplate<K, ?> template, @NonNull K name) {
        HashOperations<K, HK, HV> operations = template.opsForHash();
        return operations.entries(name);
    }

    /**
     * 获取Redis Hash结构数据
     *
     * @param name Hash结构名称
     * @param key  Hash键
     * @return Hash值
     */
    public static Object get(@NonNull String name, @NonNull String key) {
        return get(RedisContextHolder.getRedisTemplate(), name, key);
    }

    /**
     * 获取Redis Hash结构数据
     *
     * @param template Redis操作模版实例
     * @param name     Hash结构名称
     * @param key      Hash键
     * @param <K>      Hash结构名称类型
     * @return Hash值
     */
    public static <K> Object get(@NonNull RedisTemplate<K, ?> template, @NonNull K name, @NonNull String key) {
        return template.opsForHash().get(name, key);
    }

    /**
     * 获取Redis Hash结构数据
     *
     * @param name Hash结构名称
     * @param keys Hash键数组
     * @return Hash值列表
     */
    public static List<?> get(@NonNull String name, @NonNull String... keys) {
        return get(RedisContextHolder.getRedisTemplate(), name, keys);
    }

    /**
     * 获取Redis Hash结构数据
     *
     * @param template Redis操作模版实例
     * @param name     Hash结构名称
     * @param keys     Hash键数组
     * @param <K>      Hash结构名称类型
     * @return Hash值列表
     */
    public static <K> List<?> get(@NonNull RedisTemplate<K, ?> template, @NonNull K name, @NonNull String... keys) {
        return ObjectUtils.isEmpty(keys) ? Collections.emptyList() : get(template, name, Arrays.asList(keys));
    }

    /**
     * 获取Redis Hash结构数据
     *
     * @param name Hash结构名称
     * @param keys Hash键集合
     * @return Hash值列表
     */
    public static List<?> get(@NonNull String name, @NonNull Collection<String> keys) {
        return get(RedisContextHolder.getRedisTemplate(), name, keys);
    }

    /**
     * 获取Redis Hash结构数据
     *
     * @param template Redis操作模版实例
     * @param name     Hash结构名称
     * @param keys     Hash键集合
     * @param <K>      Hash结构名称类型
     * @return Hash值列表
     */
    public static <K> List<?> get(@NonNull RedisTemplate<K, ?> template, @NonNull K name,
                                  @NonNull Collection<String> keys) {
        if (ObjectUtils.isEmpty(keys)) {
            return Collections.emptyList();
        }
        HashOperations<K, String, ?> operations = template.opsForHash();
        return operations.multiGet(name, keys);
    }

    /**
     * 设置Redis Hash结构数据
     *
     * @param name     Hash结构名称
     * @param array    Hash键/值数组
     * @param duration 有效周期
     * @return true/false
     */
    public static boolean set(@NonNull String name, @NonNull Object[] array, @NonNull Duration duration) {
        return set(RedisContextHolder.getRedisTemplate(), name, array, duration);
    }

    /**
     * 设置Redis Hash结构数据
     *
     * @param name     Hash结构名称
     * @param map      Hash键/值映射表
     * @param duration 有效周期
     * @return true/false
     */
    public static boolean set(@NonNull String name, @NonNull Map<String, ?> map, @NonNull Duration duration) {
        return set(RedisContextHolder.getRedisTemplate(), name, map, duration);
    }

    /**
     * 设置Redis Hash结构数据
     *
     * @param name     Hash结构名称
     * @param key      Hash键
     * @param value    Hash值
     * @param duration 有效周期
     * @return true/false
     */
    public static boolean set(@NonNull String name, @NonNull String key, Object value, @NonNull Duration duration) {
        return set(RedisContextHolder.getRedisTemplate(), name, key, value, duration);
    }

    /**
     * 设置Redis Hash结构数据
     *
     * @param template Redis操作模版实例
     * @param name     Hash结构名称
     * @param array    Hash键/值数组
     * @param duration 有效周期
     * @param <K>      Hash结构名称类型
     * @return true/false
     */
    public static <K> boolean set(@NonNull RedisTemplate<K, ?> template, @NonNull K name, @NonNull Object[] array,
                                  @NonNull Duration duration) {
        if (ObjectUtils.isEmpty(array)) {
            return false;
        }

        AssertUtils.check(array.length % 2 == 0, () -> "Invalid key/value array");
        byte[][] args = new byte[array.length + 2][];
        args[0] = RedisContextHolder.serialize(template.getKeySerializer(), name);
        args[1] = RedisContextHolder.serialize(template.getValueSerializer(), duration.getSeconds());
        for (int i = 0, n = 2; i < array.length; i += 2) {
            args[n++] = RedisContextHolder.serialize(template.getHashKeySerializer(), array[i]);
            args[n++] = RedisContextHolder.serialize(template.getHashValueSerializer(), array[i + 1]);
        }
        return set(template, args);
    }

    /**
     * 设置Redis Hash结构数据
     *
     * @param template Redis操作模版实例
     * @param name     Hash结构名称
     * @param map      Hash键/值映射表
     * @param duration 有效周期
     * @param <K>      Hash结构名称类型
     * @return true/false
     */
    public static <K> boolean set(@NonNull RedisTemplate<K, ?> template, @NonNull K name, @NonNull Map<String, ?> map,
                                  @NonNull Duration duration) {
        if (ObjectUtils.isEmpty(map)) {
            return false;
        }

        int i = 0;
        byte[][] args = new byte[map.size() * 2 + 2][];
        args[i++] = RedisContextHolder.serialize(template.getKeySerializer(), name);
        args[i++] = RedisContextHolder.serialize(template.getValueSerializer(), duration.getSeconds());
        for (Map.Entry<?, ?> entry : map.entrySet()) {
            args[i++] = RedisContextHolder.serialize(template.getHashKeySerializer(), entry.getKey());
            args[i++] = RedisContextHolder.serialize(template.getHashValueSerializer(), entry.getValue());
        }
        return set(template, args);
    }

    /**
     * 设置Redis Hash结构数据
     *
     * @param template Redis操作模版实例
     * @param name     Hash结构名称
     * @param key      Hash键
     * @param value    Hash值
     * @param duration 有效周期
     * @param <K>      Hash结构名称类型
     * @return true/false
     */
    public static <K> boolean set(@NonNull RedisTemplate<K, ?> template, @NonNull K name, @NonNull String key,
                                  Object value, @NonNull Duration duration) {
        byte[][] args = new byte[][]{
                RedisContextHolder.serialize(template.getKeySerializer(), name),
                RedisContextHolder.serialize(template.getValueSerializer(), duration.getSeconds()),
                RedisContextHolder.serialize(template.getHashKeySerializer(), key),
                RedisContextHolder.serialize(template.getHashValueSerializer(), value)
        };
        return set(template, args);
    }

    /**
     * Redis Hash结构数字自增1
     *
     * @param name     Hash结构名称
     * @param key      Hash键
     * @param duration 有效周期
     * @return 增/减后数值
     */
    public static long increment(@NonNull String name, @NonNull String key, @NonNull Duration duration) {
        return increment(name, key, 1, duration);
    }

    /**
     * Redis Hash结构数字自增/减，delta < 0 时自减
     *
     * @param name     Hash结构名称
     * @param key      Hash键
     * @param delta    增/减值
     * @param duration 有效周期
     * @return 增/减后数值
     */
    public static long increment(@NonNull String name, @NonNull String key, long delta, @NonNull Duration duration) {
        return increment(RedisContextHolder.getRedisTemplate(), name, key, delta, duration);
    }

    /**
     * Redis Hash结构数字自增1
     *
     * @param template Redis操作模版实例
     * @param name     Hash结构名称
     * @param key      Hash键
     * @param duration 有效周期
     * @param <K>      Hash结构名称类型
     * @return 增/减后数值
     */
    public static <K> long increment(@NonNull RedisTemplate<K, ?> template, @NonNull K name, @NonNull String key,
                                     @NonNull Duration duration) {
        return increment(template, name, key, 1, duration);
    }

    /**
     * Redis Hash结构数字自增/减，delta < 0 时自减
     *
     * @param template Redis操作模版实例
     * @param name     Hash结构名称
     * @param key      Hash键
     * @param delta    增/减值
     * @param duration 有效周期
     * @param <K>      Hash结构名称类型
     * @return 增/减后数值
     */
    public static <K> long increment(@NonNull RedisTemplate<K, ?> template, @NonNull K name, @NonNull String key,
                                     long delta, @NonNull Duration duration) {
        byte[][] args = new byte[][]{
                RedisContextHolder.serialize(template.getKeySerializer(), name),
                RedisContextHolder.serialize(template.getHashKeySerializer(), key),
                RedisContextHolder.serialize(template.getHashValueSerializer(), delta),
                RedisContextHolder.serialize(template.getValueSerializer(), duration.getSeconds())
        };
        return RedisContextHolder.execute(template.getConnectionFactory(), connection -> {
            Number value;
            RedisScriptingCommands commands = connection.scriptingCommands();
            try {
                value = commands.evalSha(HINCRBY_DURATION_SCRIPT_SHA, ReturnType.INTEGER, 1, args);
            } catch (Exception e) {
                value = commands.eval(HINCRBY_DURATION_SCRIPT.getBytes(), ReturnType.INTEGER, 1, args);
            }
            return Objects.requireNonNull(value).longValue();
        });
    }

    /**
     * Redis Hash结构删除数据
     *
     * @param name Hash结构名称
     * @param keys Hash键数组
     * @return 成功删除数量
     */
    public static long delete(@NonNull String name, @NonNull String... keys) {
        return delete(RedisContextHolder.getRedisTemplate(), name, keys);
    }

    /**
     * Redis Hash结构删除数据
     *
     * @param template Redis操作模版实例
     * @param name     Hash结构名称
     * @param keys     Hash键数组
     * @param <K>      Hash结构名称类型
     * @return 成功删除数量
     */
    public static <K> long delete(@NonNull RedisTemplate<K, ?> template, @NonNull K name, @NonNull String... keys) {
        return ObjectUtils.isEmpty(keys) ? 0 : template.opsForHash().delete(name, (Object[]) keys);
    }

    /**
     * Redis Hash结构删除数据
     *
     * @param name Hash结构名称
     * @param keys Hash键集合
     * @return 成功删除数量
     */
    public static long delete(@NonNull String name, @NonNull Collection<String> keys) {
        return delete(RedisContextHolder.getRedisTemplate(), name, keys);
    }

    /**
     * Redis Hash结构删除数据
     *
     * @param template Redis操作模版实例
     * @param name     Hash结构名称
     * @param keys     Hash键集合
     * @param <K>      Hash结构名称类型
     * @return 成功删除数量
     */
    public static <K> long delete(@NonNull RedisTemplate<K, ?> template, @NonNull K name,
                                  @NonNull Collection<String> keys) {
        return ObjectUtils.isEmpty(keys) ? 0 : template.opsForHash().delete(name, keys.toArray());
    }
}
