package com.chelvc.framework.redis.context;

import java.io.Serializable;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Map;

import lombok.NonNull;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.script.DefaultRedisScript;
import org.springframework.data.redis.core.script.RedisScript;

/**
 * Redis Hash结构当日数据操作工具类
 *
 * @author Woody
 * @date 2024/6/29
 */
public final class RedisUserDailyHashHolder {
    /**
     * 数据类型
     */
    public static final String TYPE = "user";

    /**
     * Redis用户每天系统使用时间戳刷新脚本
     */
    private static final RedisScript<Object> USING_REFRESH_SCRIPT = new DefaultRedisScript<>(
            "local value = redis.call('HGET', KEYS[1], 'using') " +
                    "redis.call('HSET', KEYS[1], 'using', ARGV[1]) " +
                    "if redis.call('TTL', KEYS[1]) < 1 then redis.call('EXPIRE', KEYS[1], ARGV[2]) end " +
                    "return value", Object.class
    );

    private RedisUserDailyHashHolder() {
    }

    /**
     * 获取Redis Hash结构用户当日数据
     *
     * @param id  用户ID
     * @param key Hash键
     * @return Hash值
     */
    public static Object get(@NonNull Serializable id, @NonNull String key) {
        return RedisDailyHashHolder.get(TYPE, id, key);
    }

    /**
     * 获取Redis Hash结构用户当日数据
     *
     * @param template Redis操作模版实例
     * @param id       用户ID
     * @param key      Hash键
     * @return Hash值
     */
    public static Object get(@NonNull RedisTemplate<String, ?> template, @NonNull Serializable id,
                             @NonNull String key) {
        return RedisDailyHashHolder.get(template, TYPE, id, key);
    }

    /**
     * 获取Redis Hash结构用户当日数据
     *
     * @param id   用户ID
     * @param keys Hash键数组
     * @return Hash值列表
     */
    public static List<?> get(@NonNull Serializable id, @NonNull String... keys) {
        return RedisDailyHashHolder.get(TYPE, id, keys);
    }

    /**
     * 获取Redis Hash结构用户当日数据
     *
     * @param template Redis操作模版实例
     * @param id       用户ID
     * @param keys     Hash键数组
     * @return Hash值列表
     */
    public static List<?> get(@NonNull RedisTemplate<String, ?> template, @NonNull Serializable id,
                              @NonNull String... keys) {
        return RedisDailyHashHolder.get(template, TYPE, id, keys);
    }

    /**
     * 获取Redis Hash结构用户当日数据
     *
     * @param id   用户ID
     * @param keys Hash键集合
     * @return Hash值列表
     */
    public static List<?> get(@NonNull Serializable id, @NonNull Collection<String> keys) {
        return RedisDailyHashHolder.get(TYPE, id, keys);
    }

    /**
     * 获取Redis Hash结构用户当日数据
     *
     * @param template Redis操作模版实例
     * @param id       用户ID
     * @param keys     Hash键集合
     * @return Hash值列表
     */
    public static List<?> get(@NonNull RedisTemplate<String, ?> template, @NonNull Serializable id,
                              @NonNull Collection<String> keys) {
        return RedisDailyHashHolder.get(template, TYPE, id, keys);
    }

    /**
     * 设置Redis Hash结构用户当日数据
     *
     * @param id    用户ID
     * @param array Hash键/值数组
     * @return true/false
     */
    public static boolean set(@NonNull Serializable id, @NonNull Object... array) {
        return RedisDailyHashHolder.set(TYPE, id, array);
    }

    /**
     * 设置Redis Hash结构用户当日数据
     *
     * @param id  用户ID
     * @param map Hash键/值映射表
     * @return true/false
     */
    public static boolean set(@NonNull Serializable id, @NonNull Map<String, ?> map) {
        return RedisDailyHashHolder.set(TYPE, id, map);
    }

    /**
     * 设置Redis Hash结构用户当日数据
     *
     * @param template Redis操作模版实例
     * @param id       用户ID
     * @param array    Hash键/值数组
     * @return true/false
     */
    public static boolean set(@NonNull RedisTemplate<String, ?> template, @NonNull Serializable id,
                              @NonNull Object... array) {
        return RedisDailyHashHolder.set(template, TYPE, id, array);
    }

    /**
     * 设置Redis Hash结构用户当日数据
     *
     * @param template Redis操作模版实例
     * @param id       用户ID
     * @param map      Hash键/值映射表
     * @return true/false
     */
    public static boolean set(@NonNull RedisTemplate<String, ?> template, @NonNull Serializable id,
                              @NonNull Map<String, ?> map) {
        return RedisDailyHashHolder.set(template, TYPE, id, map);
    }

    /**
     * Redis Hash结构用户当日数字自增1
     *
     * @param id  用户ID
     * @param key Hash键
     * @return 增/减后数值
     */
    public static long increment(@NonNull Serializable id, @NonNull String key) {
        return RedisDailyHashHolder.increment(TYPE, id, key);
    }

    /**
     * Redis Hash结构用户当日数字自增/减，delta < 0 时自减
     *
     * @param id    用户ID
     * @param key   Hash键
     * @param delta 增/减值
     * @return 增/减后数值
     */
    public static long increment(@NonNull Serializable id, @NonNull String key, long delta) {
        return RedisDailyHashHolder.increment(TYPE, id, key, delta);
    }

    /**
     * Redis Hash结构用户当日数字自增1
     *
     * @param template Redis操作模版实例
     * @param id       用户ID
     * @param key      Hash键
     * @return 增/减后数值
     */
    public static long increment(@NonNull RedisTemplate<String, ?> template, @NonNull Serializable id,
                                 @NonNull String key) {
        return RedisDailyHashHolder.increment(template, TYPE, id, key);
    }

    /**
     * Redis Hash结构用户当日数字自增/减，delta < 0 时自减
     *
     * @param template Redis操作模版实例
     * @param id       用户ID
     * @param key      Hash键
     * @param delta    增/减值
     * @return 增/减后数值
     */
    public static long increment(@NonNull RedisTemplate<String, ?> template, @NonNull Serializable id,
                                 @NonNull String key, long delta) {
        return RedisDailyHashHolder.increment(template, TYPE, id, key, delta);
    }

    /**
     * Redis Hash结构删除用户当日数据
     *
     * @param id   用户ID
     * @param keys Hash键数组
     * @return 成功删除数量
     */
    public static long delete(Serializable id, @NonNull String... keys) {
        return RedisDailyHashHolder.delete(TYPE, id, keys);
    }

    /**
     * Redis Hash结构删除用户当日数据
     *
     * @param template Redis操作模版实例
     * @param id       用户ID
     * @param keys     Hash键数组
     * @return 成功删除数量
     */
    public static long delete(@NonNull RedisTemplate<String, ?> template, @NonNull Serializable id,
                              @NonNull String... keys) {
        return RedisDailyHashHolder.delete(template, TYPE, id, keys);
    }

    /**
     * Redis Hash结构删除用户当日数据
     *
     * @param id   用户ID
     * @param keys Hash键集合
     * @return 成功删除数量
     */
    public static long delete(@NonNull Serializable id, @NonNull Collection<String> keys) {
        return RedisDailyHashHolder.delete(TYPE, id, keys);
    }

    /**
     * Redis Hash结构删除用户当日数据
     *
     * @param template Redis操作模版实例
     * @param id       用户ID
     * @param keys     Hash键集合
     * @return 成功删除数量
     */
    public static long delete(@NonNull RedisTemplate<String, ?> template, @NonNull Serializable id,
                              @NonNull Collection<String> keys) {
        return RedisDailyHashHolder.delete(template, TYPE, id, keys);
    }

    /**
     * 获取并刷新用户当日使用时间戳
     *
     * @param id 用户ID
     * @return 上次使用时间戳
     */
    public static Long using(@NonNull Serializable id) {
        return using(RedisContextHolder.getRedisTemplate(), id);
    }

    /**
     * 获取并刷新用户当日使用时间戳
     *
     * @param template Redis操作模版实例
     * @param id       用户ID
     * @return 上次使用时间戳
     */
    public static Long using(@NonNull RedisTemplate<String, ?> template, @NonNull Serializable id) {
        List<String> keys = Collections.singletonList(RedisDailyHashHolder.name(TYPE, id));
        long duration = RedisDailyHashHolder.countdown().getSeconds();
        return (Long) template.execute(USING_REFRESH_SCRIPT, keys, System.currentTimeMillis(), duration);
    }
}
