package com.chelvc.framework.redis.queue;

import java.lang.reflect.Type;
import java.util.Objects;
import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.redis.annotation.RedisMQConsumer;
import com.chelvc.framework.redis.context.RedisStreamHolder;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.springframework.core.env.Environment;
import org.springframework.data.redis.connection.stream.Consumer;
import org.springframework.data.redis.connection.stream.MapRecord;
import org.springframework.data.redis.connection.stream.ReadOffset;
import org.springframework.data.redis.connection.stream.StreamInfo;
import org.springframework.data.redis.connection.stream.StreamOffset;
import org.springframework.data.redis.stream.StreamMessageListenerContainer;

/**
 * 消息监听器容器默认实现
 *
 * @param <T> 监听消息对象类型
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
public class DefaultRedisMQListenerContainer<T> implements RedisMQListenerContainer<T> {
    private final String name = StringUtils.uuid();
    private int idle;
    private String topic;
    private String group;
    private Consumer consumer;
    private ExecutorService executor;
    private ScheduledExecutorService cleaner;
    private MessageStreamListener<T> listener;
    private StreamMessageListenerContainer<String, MapRecord<String, String, String>> container;

    /**
     * 清理消息
     */
    private void clear() {
        // 发送心跳包
        try {
            RedisStreamHolder.heartbeat(this.topic);
        } catch (Throwable t) {
            log.warn("RedisMQ consumer heartbeat message send failed: {}, {}", this.consumer, t.getMessage());
        }

        // 执行消费者清理逻辑
        int idle = this.idle * 2 * 1000;
        StreamInfo.XInfoConsumers consumers = RedisStreamHolder.consumers(this.topic, this.group);
        for (int i = 0, length = consumers.size(); i < length; i++) {
            StreamInfo.XInfoConsumer consumer = consumers.get(i);
            if (Objects.equals(consumer.consumerName(), this.name) || consumer.idleTimeMs() < idle) {
                continue;
            }
            Consumer source = Consumer.from(consumer.groupName(), consumer.consumerName());
            if (consumer.pendingCount() == 0) {
                RedisStreamHolder.delete(this.topic, source);
                log.info("RedisMQ consumer deleted: {}", source);
                continue;
            }
            new ConsumerPendingCleaner<>(this.topic, source, this.consumer, this.listener, this.executor).start();
        }
    }

    @Override
    public void initialize(@NonNull Type type, @NonNull RedisMQListener<T> listener,
                           @NonNull RedisMQConsumer annotation) {
        int concurrency = annotation.concurrency();
        AssertUtils.check(concurrency > 0, () -> "Consumer concurrency must be greater than 0");
        BlockingQueue<Runnable> queue = new ArrayBlockingQueue<>((int) (concurrency / 0.1) * 2);
        ExecutorService executor = new ThreadPoolExecutor(
                concurrency, concurrency, 60L, TimeUnit.SECONDS, queue, new ThreadPoolExecutor.CallerRunsPolicy()
        );
        this.initialize(type, annotation.topic(), annotation.group(), annotation.batch(), listener, executor);
    }

    @Override
    public void initialize(@NonNull Type type, @NonNull String topic, @NonNull String group, int batch,
                           @NonNull RedisMQListener<T> listener, @NonNull ExecutorService executor) {
        AssertUtils.nonempty(topic, () -> "Consumer topic must not be empty");
        AssertUtils.nonempty(group, () -> "Consumer group must not be empty");
        AssertUtils.check(batch > 0, () -> "Consumer batch must be greater than 0");
        AssertUtils.check((this.idle = RedisStreamHolder.getIdle()) > 0, () -> "Consumer idle must be greater than 0");
        Environment environment = ApplicationContextHolder.getEnvironment();
        this.topic = RedisStreamHolder.isolate(environment.resolvePlaceholders(topic));
        this.group = RedisStreamHolder.isolate(environment.resolvePlaceholders(group));
        this.consumer = Consumer.from(this.group, this.name);
        this.listener = new MessageStreamListener<>(
                type, this.topic, this.consumer, listener, this.executor = executor
        );

        // 初始化消费者组
        RedisStreamHolder.initializeConsumerGroup(this.topic, this.group);

        // 初始化消息监听器容器
        StreamOffset<String> offset = StreamOffset.create(this.topic, ReadOffset.lastConsumed());
        this.container = RedisStreamHolder.initializeMessageListenerContainer(
                this.listener, this.consumer, offset, batch, this.executor
        );
        this.container.start();

        // 初始化消费者清理定时器
        this.cleaner = Executors.newScheduledThreadPool(1);
        this.cleaner.scheduleAtFixedRate(this::clear, 0, this.idle, TimeUnit.SECONDS);
    }

    @Override
    public void destroy() throws Exception {
        try {
            if (this.cleaner != null) {
                this.cleaner.shutdown();
            }
        } finally {
            try {
                if (this.container != null) {
                    this.container.stop();
                }
            } finally {
                if (this.executor != null) {
                    this.executor.shutdown();
                }
            }
        }
    }
}
