package com.chelvc.framework.redis.queue;

import java.time.Duration;
import java.util.AbstractQueue;
import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.redis.context.RedisContextHolder;
import lombok.NonNull;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.script.DefaultRedisScript;
import org.springframework.data.redis.core.script.RedisScript;

/**
 * Redis队列实现
 *
 * @param <E> 元素类型
 * @author Woody
 * @date 2024/8/19
 */
public class RedisQueue<E> extends AbstractQueue<E> {
    /**
     * 带过期时间Redis队列添加元素脚本
     */
    private static final RedisScript<Long> ADD_DURATION_SCRIPT = new DefaultRedisScript<>(
            "local value = redis.call('RPUSH', KEYS[1], unpack(ARGV, 2)) " +
                    "if value > 0 and redis.call('TTL', KEYS[1]) < 0 then redis.call('EXPIRE', KEYS[1], ARGV[1]) end " +
                    "return value", Long.class
    );

    private final String name;
    private final long duration;
    private final RedisTemplate<String, E> template;

    public RedisQueue(@NonNull String name) {
        this(name, Duration.ofSeconds(-1));
    }

    public RedisQueue(@NonNull String name, @NonNull Duration duration) {
        this(name, null, duration);
    }

    public RedisQueue(@NonNull String name, RedisTemplate<String, E> template) {
        this(name, template, Duration.ofSeconds(-1));
    }

    public RedisQueue(@NonNull String name, RedisTemplate<String, E> template, @NonNull Duration duration) {
        this.name = name;
        this.template = template;
        this.duration = Math.max(duration.getSeconds(), -1);
    }

    /**
     * 集合迭代器实现
     */
    private class Iter implements Iterator<E> {
        private E element;
        private int index = 0;
        private final int size;

        public Iter(int size) {
            AssertUtils.check(size > -1, () -> "size must be greater than -1");
            this.size = size;
        }

        @Override
        public boolean hasNext() {
            return this.index < this.size;
        }

        @Override
        public E next() {
            return this.element = template().opsForList().index(name, this.index++);
        }

        @Override
        public void remove() {
            if (this.element != null) {
                template().opsForList().remove(name, 1, this.element);
                this.element = null;
            }
        }
    }

    /**
     * 获取RedisTemplate实例
     *
     * @return RedisTemplate实例
     */
    private RedisTemplate<String, E> template() {
        return ObjectUtils.ifNull(this.template, RedisContextHolder::getDefaultTemplate);
    }

    @Override
    public Iterator<E> iterator() {
        return new Iter(this.size());
    }

    @Override
    public int size() {
        Long size = this.template().opsForList().size(this.name);
        return size == null ? 0 : size.intValue();
    }

    @Override
    public boolean offer(E e) {
        if (e == null) {
            return false;
        }

        Long count;
        if (this.duration > 0) {
            List<String> keys = Collections.singletonList(this.name);
            count = this.template().execute(ADD_DURATION_SCRIPT, keys, this.duration, e);
        } else {
            count = this.template().opsForList().rightPush(this.name, e);
        }
        return count != null && count > 0;
    }

    @Override
    public E poll() {
        return this.template().opsForList().leftPop(this.name);
    }

    @Override
    public E peek() {
        return this.template().opsForList().index(this.name, 0);
    }

    @Override
    public void clear() {
        this.template().delete(this.name);
    }

    @Override
    @SuppressWarnings("unchecked")
    public boolean addAll(Collection<? extends E> c) {
        if (ObjectUtils.isEmpty(c)) {
            return false;
        }

        Long count;
        if (this.duration > 0) {
            int i = 0;
            Object[] args = new Object[c.size() + 1];
            args[i++] = this.duration;
            for (E e : c) {
                args[i++] = e;
            }
            count = this.template().execute(ADD_DURATION_SCRIPT, Collections.singletonList(this.name), args);
        } else {
            count = this.template().opsForList().rightPushAll(this.name, (Collection<E>) c);
        }
        return count != null && count > 0;
    }

    @Override
    public Object[] toArray() {
        List<E> values = this.template().opsForList().range(this.name, 0, -1);
        return ObjectUtils.ifNull(values, Collections::emptyList).toArray();
    }

    @Override
    public <T> T[] toArray(T[] a) {
        List<E> values = this.template().opsForList().range(this.name, 0, -1);
        return ObjectUtils.ifNull(values, Collections::emptyList).toArray(a);
    }

    @Override
    public String toString() {
        List<E> values = this.template().opsForList().range(this.name, 0, -1);
        return ObjectUtils.ifNull(values, Collections::emptyList).toString();
    }
}
