package com.chelvc.framework.rocketmq.config;

import java.lang.reflect.Type;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.rocketmq.annotation.RocketMQConsumer;
import com.chelvc.framework.rocketmq.consumer.MultipleRocketMQListenerContainer;
import com.chelvc.framework.rocketmq.consumer.RocketMQListener;
import com.chelvc.framework.rocketmq.consumer.RocketMQListenerContainer;
import com.chelvc.framework.rocketmq.consumer.SingleRocketMQListenerContainer;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.apache.rocketmq.client.apis.ClientConfiguration;
import org.apache.rocketmq.client.apis.ClientServiceProvider;
import org.springframework.aop.framework.AopProxyUtils;
import org.springframework.boot.context.event.ApplicationStartedEvent;
import org.springframework.context.ApplicationListener;

/**
 * RocketMQ消息监听器注册器
 *
 * @author Woody
 * @date 2024/7/13
 */
@Slf4j
public class RocketMQListenerRegistry implements ApplicationListener<ApplicationStartedEvent> {
    private final ClientServiceProvider provider;
    private final ClientConfiguration configuration;

    public RocketMQListenerRegistry(@NonNull ClientServiceProvider provider,
                                    @NonNull ClientConfiguration configuration) {
        this.provider = provider;
        this.configuration = configuration;
    }

    /**
     * 创建监听器容器
     *
     * @param multiple 是否批量消费
     * @param <T>      消息对象类型
     * @return 容器实例
     */
    @SuppressWarnings("unchecked")
    protected <T> RocketMQListenerContainer<T> createListenerContainer(boolean multiple) {
        return ApplicationContextHolder.register(
                StringUtils.uuid(),
                RocketMQListenerContainer.class,
                () -> multiple ? new MultipleRocketMQListenerContainer<>() : new SingleRocketMQListenerContainer<>()
        );
    }

    /**
     * 注册消息监听器容器
     *
     * @param name     消息监听器名称
     * @param listener 消息监听器实例
     * @param <T>      消息对象类型
     */
    protected <T> void registerListenerContainer(@NonNull String name, @NonNull RocketMQListener<T> listener) {
        Class<?> clazz = AopProxyUtils.ultimateTargetClass(listener);
        RocketMQConsumer annotation = AssertUtils.nonnull(
                clazz.getAnnotation(RocketMQConsumer.class),
                () -> "@RocketMQConsumer annotation is missing: " + clazz.getName()
        );
        RocketMQListenerContainer<T> container = this.createListenerContainer(annotation.batch() > 1);
        Type type = ObjectUtils.lookupSuperclassParameterized(clazz, RocketMQListener.class, Object.class);
        try {
            container.initialize(type, this.provider, this.configuration, listener, annotation);
        } catch (Exception e) {
            log.error("RocketMQListenerContainer initialize failed", e);
        }
    }

    @Override
    public void onApplicationEvent(ApplicationStartedEvent event) {
        ApplicationContextHolder.getBeans(RocketMQListener.class).forEach(this::registerListenerContainer);
    }
}
