package com.chelvc.framework.rocketmq.consumer;

import java.lang.reflect.Type;
import java.time.Duration;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.base.context.Session;
import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.ThreadUtils;
import com.chelvc.framework.rocketmq.annotation.RocketMQConsumer;
import com.chelvc.framework.rocketmq.context.RocketMQContextHolder;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.tuple.Pair;
import org.apache.rocketmq.client.apis.ClientConfiguration;
import org.apache.rocketmq.client.apis.ClientException;
import org.apache.rocketmq.client.apis.ClientServiceProvider;
import org.apache.rocketmq.client.apis.consumer.FilterExpression;
import org.apache.rocketmq.client.apis.consumer.FilterExpressionType;
import org.apache.rocketmq.client.apis.consumer.SimpleConsumer;
import org.apache.rocketmq.client.apis.consumer.SimpleConsumerBuilder;
import org.apache.rocketmq.client.apis.message.MessageView;
import org.springframework.core.env.Environment;

/**
 * 消息批量消费监听器容器
 *
 * @param <T> 监听消息对象类型
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
public class MultipleRocketMQListenerContainer<T> implements RocketMQListenerContainer<T> {
    private Type type;
    private String topic;
    private String group;
    private int batch;
    private Duration duration;
    private Thread thread;
    private SimpleConsumer consumer;
    private RocketMQListener<T> listener;
    private volatile boolean running = true;

    /**
     * 接收消息
     *
     * @return 消息视图列表
     */
    private List<MessageView> receive() {
        try {
            return this.consumer.receive(this.batch, this.duration);
        } catch (Throwable t) {
            if (t.getCause() instanceof InterruptedException) {
                Thread.currentThread().interrupt();
            } else {
                log.error("RocketMQ message receive failed: {}:{}", this.topic, this.group, t);
            }
        }
        return Collections.emptyList();
    }

    /**
     * 批量消费消息
     *
     * @param messages 消息视图列表
     * @return true/false
     */
    @SuppressWarnings("unchecked")
    private boolean consume(List<MessageView> messages) {
        try {
            List<Pair<T, Session>> payloads = RocketMQContextHolder.deserialize(
                    messages, this.type, (payload, session) -> Pair.of((T) payload, session)
            );
            return this.listener.consume(payloads);
        } catch (Throwable t) {
            String ids = messages.stream().map(MessageView::getMessageId).map(String::valueOf)
                    .collect(Collectors.joining(","));
            log.error("RocketMQ message consume failed: {}:{}, {}", this.topic, this.group, ids, t);
        }
        return false;
    }

    /**
     * 确认消息
     *
     * @param messages 消息视图列表
     */
    private void acknowledge(List<MessageView> messages) {
        for (MessageView message : messages) {
            try {
                this.consumer.ack(message);
            } catch (Throwable t) {
                if (t.getCause() instanceof InterruptedException) {
                    Thread.currentThread().interrupt();
                } else {
                    String id = String.valueOf(message.getMessageId());
                    log.error("RocketMQ message acknowledge failed: {}:{}, {}", this.topic, this.group, id, t);
                }
            }
        }
    }

    /**
     * 启动消费线程
     *
     * @return 消费线程实例
     */
    private Thread startup() {
        return ThreadUtils.run(() -> {
            while (this.running && !Thread.currentThread().isInterrupted()) {
                // 批量接收消息
                List<MessageView> messages = this.receive();
                if (ObjectUtils.isEmpty(messages)) {
                    continue;
                }

                // 消费消息
                boolean success = this.consume(messages);

                // 回执消息
                if (success) {
                    this.acknowledge(messages);
                }
            }
        });
    }

    /**
     * 关闭消息消费者线程
     *
     * @param thread 消息消费者线程
     */
    private void shutdown(Thread thread) {
        if (thread != null) {
            thread.interrupt();
            ThreadUtils.join(thread);
        }
    }

    /**
     * 关闭消息消费者
     *
     * @param consumer 消息消费者实例
     */
    private void shutdown(SimpleConsumer consumer) {
        if (consumer != null) {
            try {
                consumer.close();
            } catch (Throwable t) {
                log.warn("RocketMQ consumer shutdown failed: {}:{}, {}", this.topic, this.group, t.getMessage());
            }
        }
    }

    @Override
    public void initialize(@NonNull Type type, @NonNull ClientServiceProvider provider,
                           @NonNull ClientConfiguration configuration, @NonNull RocketMQListener<T> listener,
                           @NonNull RocketMQConsumer annotation) {
        // 参数配置有效性校验
        AssertUtils.nonempty(annotation.topic(), () -> "Consumer topic must not be empty");
        AssertUtils.nonempty(annotation.group(), () -> "Consumer group must not be empty");
        AssertUtils.check(annotation.batch() > 0, () -> "Consumer batch must be greater than 0");
        AssertUtils.check(annotation.duration() > 0, () -> "Consumer duration must be greater than 0");
        AssertUtils.check(annotation.interval() > 0, () -> "Consumer interval must be greater than 0");
        AssertUtils.nonempty(annotation.tag(), () -> "Consumer tag must not be empty");

        // 初始化消费者实例
        this.type = type;
        Environment environment = ApplicationContextHolder.getEnvironment();
        this.topic = RocketMQContextHolder.isolate(environment.resolvePlaceholders(annotation.topic()));
        this.group = RocketMQContextHolder.isolate(environment.resolvePlaceholders(annotation.group()));
        this.batch = annotation.batch();
        this.duration = Duration.ofSeconds(annotation.duration());
        this.listener = listener;
        Duration interval = Duration.ofSeconds(annotation.interval());
        FilterExpression filter = new FilterExpression(annotation.tag(), FilterExpressionType.TAG);
        SimpleConsumerBuilder builder = provider.newSimpleConsumerBuilder().setClientConfiguration(configuration)
                .setConsumerGroup(this.group).setAwaitDuration(interval)
                .setSubscriptionExpressions(Collections.singletonMap(this.topic, filter));
        try {
            this.consumer = builder.build();
        } catch (ClientException e) {
            throw new RuntimeException(e);
        }

        // 启动消费线程
        this.thread = this.startup();
    }

    @Override
    public void destroy() throws Exception {
        this.running = false;
        try {
            this.shutdown(this.thread);
        } finally {
            this.shutdown(this.consumer);
        }
    }
}
