package com.chelvc.framework.security.context;

import java.util.Collections;
import java.util.List;
import java.util.Set;
import java.util.stream.Stream;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.base.context.Session;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.security.crypto.SecurityCipherHandler;
import com.google.common.collect.Sets;
import lombok.NonNull;
import org.springframework.security.oauth2.jwt.Jwt;

/**
 * 应用安全相关操作上下文工具类
 *
 * @author Woody
 * @date 2024/5/24
 */
public final class SecurityContextHolder {
    /**
     * 应用范围标识
     */
    public static final String SCOPE = "scope";

    /**
     * 客户端登陆标识
     */
    public static final String CLIENT = "client";

    /**
     * 手机号码标识
     */
    public static final String MOBILE = "mobile";

    /**
     * 用户账号标识
     */
    public static final String USERNAME = "user_name";

    /**
     * 主体创建时间戳
     */
    public static final String CREATING = "creating";

    /**
     * 主体注册时间戳
     */
    public static final String REGISTERING = "registering";

    /**
     * 用户权限标识
     */
    public static final String AUTHORITIES = "authorities";

    /**
     * 默认权限分组
     */
    public static final String DEFAULT_PERMISSION_GROUP = "DEFAULT";

    /**
     * 数据安全加解密处理器
     */
    private static volatile SecurityCipherHandler SECURITY_CIPHER_HANDLER;

    private SecurityContextHolder() {
    }

    /**
     * 获取主体身份
     *
     * @param jwt JWT对象
     * @return 主体身份标识
     */
    public static Long getId(Jwt jwt) {
        return jwt == null ? null : StringUtils.ifEmpty(jwt.getClaimAsString(USERNAME), Long::parseLong);
    }

    /**
     * 获取应用范围
     *
     * @param jwt Jwt对象
     * @return 应用范围
     */
    public static String getScope(Jwt jwt) {
        return jwt == null ? null : StringUtils.ifEmpty(jwt.getClaimAsString(SCOPE), (String) null);
    }

    /**
     * 是否是客户端登陆
     *
     * @param jwt Jwt对象
     * @return true/false
     */
    public static boolean isClient(Jwt jwt) {
        return jwt != null && Boolean.TRUE.equals(jwt.getClaimAsBoolean(CLIENT));
    }

    /**
     * 获取用户授权信息
     *
     * @param jwt JWT对象
     * @return 授权信息集合
     */
    public static Set<String> getAuthorities(Jwt jwt) {
        if (jwt == null) {
            return Collections.emptySet();
        }
        List<String> authorities = jwt.getClaimAsStringList(AUTHORITIES);
        return ObjectUtils.isEmpty(authorities) ? Collections.emptySet() : Sets.newHashSet(authorities);
    }

    /**
     * 获取数据安全加解密处理器
     *
     * @return 数据安全加解密处理器实例
     */
    public static SecurityCipherHandler getSecurityCipherHandler() {
        if (SECURITY_CIPHER_HANDLER == null) {
            synchronized (SecurityCipherHandler.class) {
                if (SECURITY_CIPHER_HANDLER == null) {
                    SECURITY_CIPHER_HANDLER = ApplicationContextHolder.getBean(SecurityCipherHandler.class);
                }
            }
        }
        return SECURITY_CIPHER_HANDLER;
    }

    /**
     * 判断当前用户是否拥有任意权限
     *
     * @param permissions 权限标识数组
     * @return true/false
     */
    public static boolean hashAnyPermission(@NonNull String... permissions) {
        if (ObjectUtils.isEmpty(permissions)) {
            return true;
        }
        Session session = SessionContextHolder.getSession(false);
        Set<String> authorities = ObjectUtils.ifNull(session, Session::getAuthorities);
        if (ObjectUtils.notEmpty(authorities)) {
            Stream<String> stream = Stream.of(permissions);
            if (stream.anyMatch(authorities::contains)) {
                return true;
            }
            for (String authority : authorities) {
                Set<?> properties = ApplicationContextHolder.getProperty(authority, Set.class);
                if (ObjectUtils.notEmpty(properties) && stream.anyMatch(properties::contains)) {
                    return true;
                }
            }
        }
        return false;
    }
}
