package com.chelvc.framework.security.interceptor;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.Method;
import java.lang.reflect.Type;
import java.util.List;
import java.util.Objects;
import java.util.Set;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.base.context.JacksonContextHolder;
import com.chelvc.framework.base.context.LoggingContextHolder;
import com.chelvc.framework.base.context.Result;
import com.chelvc.framework.base.context.Session;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.base.interceptor.BufferedRequestWrapper;
import com.chelvc.framework.base.util.HttpUtils;
import com.chelvc.framework.base.util.SpringUtils;
import com.chelvc.framework.common.exception.FrameworkException;
import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.common.util.CodecUtils;
import com.chelvc.framework.common.util.FileUtils;
import com.chelvc.framework.common.util.ObjectUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.security.annotation.Crypto;
import com.chelvc.framework.security.annotation.Permission;
import com.chelvc.framework.security.annotation.Sign;
import com.chelvc.framework.security.config.SecurityProperties;
import com.chelvc.framework.security.context.SecurityContextHolder;
import com.google.common.collect.Sets;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.aop.framework.AopProxyUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.context.event.ApplicationStartedEvent;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationListener;
import org.springframework.core.MethodParameter;
import org.springframework.core.Ordered;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpInputMessage;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.converter.HttpMessageConverter;
import org.springframework.http.server.ServerHttpRequest;
import org.springframework.http.server.ServerHttpResponse;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.HandlerInterceptor;
import org.springframework.web.servlet.config.annotation.InterceptorRegistry;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurer;
import org.springframework.web.servlet.mvc.method.annotation.RequestBodyAdvice;
import org.springframework.web.servlet.mvc.method.annotation.ResponseBodyAdvice;

/**
 * 请求安全拦截器
 *
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
@ControllerAdvice
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class SecurityValidateInterceptor implements HandlerInterceptor, WebMvcConfigurer,
        RequestBodyAdvice, ResponseBodyAdvice<Object>, ApplicationListener<ApplicationStartedEvent> {
    /**
     * 签名信息请求头
     */
    private static final String HEADER_SIGNATURE = "signature";

    private final SecurityProperties properties;

    /**
     * 数据加密
     *
     * @param original 原始数据
     * @return 数据密文
     */
    private String encrypt(Object original) {
        String plaintext = JacksonContextHolder.serialize(original);
        return SecurityContextHolder.getSecurityCipherHandler().encrypt(plaintext);
    }

    /**
     * 数据解密
     *
     * @param message 原始消息
     * @return 明文消息
     * @throws IOException I/O异常
     */
    private HttpInputMessage decrypt(HttpInputMessage message) throws IOException {
        byte[] ciphertext;
        try (InputStream input = message.getBody()) {
            if (input instanceof BufferedRequestWrapper.BufferedServletInputStream) {
                ciphertext = ((BufferedRequestWrapper.BufferedServletInputStream) input).decodeBase64();
            } else {
                ciphertext = CodecUtils.decodeBase64(FileUtils.getBytes(input));
            }
        }
        byte[] plaintext = SecurityContextHolder.getSecurityCipherHandler().decrypt(ciphertext);
        return new HttpInputMessage() {
            @Override
            public HttpHeaders getHeaders() {
                return message.getHeaders();
            }

            @Override
            public InputStream getBody() throws IOException {
                return new ByteArrayInputStream(plaintext);
            }
        };
    }

    @Override
    public void onApplicationEvent(ApplicationStartedEvent event) {
        // 检查权限标识是否重复
        ApplicationContext applicationContext = event.getApplicationContext();
        List<Object> controllers = ApplicationContextHolder.lookupControllers(applicationContext);
        if (ObjectUtils.isEmpty(controllers)) {
            return;
        }
        Set<String> permissions = Sets.newHashSet();
        controllers.forEach(controller -> {
            Class<?> clazz = AopProxyUtils.ultimateTargetClass(controller);
            for (Method method : clazz.getDeclaredMethods()) {
                Permission annotation = method.getAnnotation(Permission.class);
                if (annotation == null) {
                    continue;
                }
                String value = annotation.value(), group = annotation.group();
                AssertUtils.nonempty(value, () -> "Permission value must not be empty: " + method);
                AssertUtils.nonempty(group, () -> "Permission group must not be empty: " + method);
                AssertUtils.check(permissions.add(value), () -> "Duplicate permission value: " + value);
            }
        });
    }

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler)
            throws Exception {
        // 忽略地址处理
        if (SpringUtils.isPath(request.getRequestURI(), this.properties.getIgnores())) {
            return true;
        }

        // 请求头校验
        Session session = SessionContextHolder.getSession();
        if (session.getPlatform() == null || session.getTerminal() == null
                || StringUtils.isEmpty(session.getVersion()) || session.getTimestamp() == null) {
            throw new FrameworkException(HttpStatus.BAD_REQUEST.name(), null,
                    ApplicationContextHolder.getMessage("Header.Missing"));
        }
        long expiration = this.properties.getRequest().getExpiration();
        if (expiration > 0 && Math.abs(System.currentTimeMillis() - session.getTimestamp()) > expiration) {
            throw new FrameworkException(HttpStatus.BAD_REQUEST.name(), null,
                    ApplicationContextHolder.getMessage("Time.Deviated"));
        }

        if (!(handler instanceof HandlerMethod)) {
            return true;
        }

        // 数字签名校验
        HandlerMethod method = (HandlerMethod) handler;
        Sign sign = method.getMethodAnnotation(Sign.class);
        if (sign == null) {
            sign = method.getBeanType().getAnnotation(Sign.class);
        }
        if (sign != null && sign.enabled()) {
            String payload = HttpUtils.serialize(request, true);
            String signature = request.getHeader(HEADER_SIGNATURE);
            String ciphertext = SecurityContextHolder.getSecurityCipherHandler().sign(session, payload);
            LoggingContextHolder.debug(log, signature, ciphertext, payload);
            if (!Objects.equals(ciphertext, signature)) {
                throw new FrameworkException(HttpStatus.BAD_REQUEST.name(), null,
                        ApplicationContextHolder.getMessage("Signature.Invalid"));
            }
        }

        // 接口权限校验
        Permission permission = method.getMethodAnnotation(Permission.class);
        if (permission != null && !SecurityContextHolder.hashAnyPermission(permission.value(), permission.group())) {
            throw new FrameworkException(HttpStatus.FORBIDDEN.name(), null,
                    ApplicationContextHolder.getMessage("Forbidden"));
        }
        return true;
    }

    @Override
    public void addInterceptors(InterceptorRegistry registry) {
        registry.addInterceptor(this).addPathPatterns("/**").order(Ordered.HIGHEST_PRECEDENCE);
    }

    @Override
    public boolean supports(MethodParameter method, Type type, Class<? extends HttpMessageConverter<?>> clazz) {
        Crypto crypto = method.getMethodAnnotation(Crypto.class);
        return crypto != null && crypto.request();
    }

    @Override
    public HttpInputMessage beforeBodyRead(HttpInputMessage message, MethodParameter method, Type type,
                                           Class<? extends HttpMessageConverter<?>> clazz) throws IOException {
        try {
            return this.decrypt(message);
        } catch (Exception e) {
            log.warn("Request parameter decrypt failed: {}", e.getMessage());
            throw new FrameworkException(HttpStatus.BAD_REQUEST.name(), null,
                    ApplicationContextHolder.getMessage("Request.Invalid"));
        }
    }

    @Override
    public Object afterBodyRead(Object body, HttpInputMessage message, MethodParameter method, Type type,
                                Class<? extends HttpMessageConverter<?>> clazz) {
        return body;
    }

    @Override
    public Object handleEmptyBody(Object body, HttpInputMessage message, MethodParameter method, Type type,
                                  Class<? extends HttpMessageConverter<?>> clazz) {
        return body;
    }

    @Override
    public boolean supports(MethodParameter method, Class<? extends HttpMessageConverter<?>> clazz) {
        Crypto crypto = method.getMethodAnnotation(Crypto.class);
        return crypto != null && crypto.response();
    }

    @Override
    public Object beforeBodyWrite(Object body, MethodParameter method, MediaType media,
                                  Class<? extends HttpMessageConverter<?>> clazz,
                                  ServerHttpRequest request, ServerHttpResponse response) {
        if (body == null) {
            return null;
        } else if (body instanceof Result) {
            Result<?> result = (Result<?>) body;
            if (result.getData() == null) {
                return result;
            }
            String ciphertext = this.encrypt(result.getData());
            return Result.of(result.getCode(), ciphertext, result.getMessage());
        }
        return this.encrypt(body);
    }
}
