package com.chelvc.framework.sms.support;

import java.util.List;
import java.util.Map;
import java.util.function.Consumer;

import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.sms.Captcha;
import com.chelvc.framework.sms.TemplateSmsHandler;
import com.google.common.collect.Lists;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;

/**
 * 模版短信发送处理器委派实现
 *
 * @author Woody
 * @date 2024/1/30
 */
@Slf4j
public class DelegatingTemplateSmsHandler implements TemplateSmsHandler {
    private final List<TemplateSmsHandler> handlers;

    public DelegatingTemplateSmsHandler(@NonNull List<TemplateSmsHandler> handlers) {
        AssertUtils.nonempty(handlers, () -> "Delegating handlers must not be empty");
        this.handlers = Lists.newArrayList(handlers);
    }

    /**
     * 遍历所有短信处理器并执行短信发送处理，直到发送成功
     *
     * @param callback 短信处理回调函数
     */
    private void execute(Consumer<TemplateSmsHandler> callback) {
        RuntimeException unsupported = null;
        for (int i = 0, last = this.handlers.size() - 1; i <= last; i++) {
            try {
                callback.accept(this.handlers.get(i));
                return;
            } catch (RuntimeException e) {
                if (e instanceof UnsupportedOperationException) {
                    unsupported = e;
                } else if (i < last) {
                    log.warn("Template SMS send failed: {}", e.getMessage());
                } else {
                    throw e;
                }
            }
        }
        if (unsupported != null) {
            throw unsupported;
        }
    }

    @Override
    public void send(@NonNull Captcha captcha) {
        this.execute(handler -> handler.send(captcha));
    }

    @Override
    public void send(@NonNull String template, @NonNull Captcha captcha) {
        this.execute(handler -> handler.send(template, captcha));
    }

    @Override
    public void send(@NonNull String mobile, @NonNull String template, @NonNull String... parameters) {
        this.execute(handler -> handler.send(mobile, template, parameters));
    }

    @Override
    public void send(@NonNull String mobile, @NonNull String template, @NonNull Map<String, ?> parameters) {
        this.execute(handler -> handler.send(mobile, template, parameters));
    }
}
