package com.chelvc.framework.wechat.support;

import java.util.List;
import java.util.Objects;

import com.chelvc.framework.base.context.ApplicationContextHolder;
import com.chelvc.framework.common.util.AssertUtils;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.wechat.PayChannel;
import com.chelvc.framework.wechat.WechatPaymentCallback;
import com.chelvc.framework.wechat.WechatPaymentHandler;
import com.chelvc.framework.wechat.WechatPaymentProcessor;
import com.google.common.collect.Lists;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;

/**
 * 微信支付操作默认实现
 *
 * @author Woody
 * @date 2024/6/19
 */
@Slf4j
public class DefaultWechatPaymentHandler implements WechatPaymentHandler {
    private final List<WechatPaymentProcessor> processors;

    public DefaultWechatPaymentHandler(List<WechatPaymentProcessor> processors) {
        AssertUtils.nonempty(processors, () -> "Wechat payment processors must not be empty");
        this.processors = Lists.newArrayList(processors);
    }

    @Override
    public boolean supports(@NonNull String name, PayChannel channel) {
        for (WechatPaymentProcessor processor : this.processors) {
            if (Objects.equals(name, processor.getName()) && (channel == null || channel == processor.getChannel())) {
                return true;
            }
        }
        return false;
    }

    @Override
    public WechatPaymentProcessor getProcessor(@NonNull String name, PayChannel channel) {
        WechatPaymentProcessor optional = null;
        String merchant = ApplicationContextHolder.getProperty("wechat.payment." + name + ".merchant");
        for (WechatPaymentProcessor processor : this.processors) {
            if (Objects.equals(name, processor.getName()) && (channel == null || channel == processor.getChannel())) {
                if (optional == null) {
                    optional = processor;
                }

                // 优先使用指定商户号支付配置
                if (StringUtils.isEmpty(merchant) || Objects.equals(merchant, processor.getMchid())) {
                    return processor;
                }
            }
        }
        if (optional == null) {
            throw new IllegalArgumentException("Wechat payment processor not found: " + name);
        }
        return optional;
    }

    @Override
    public WechatPaymentCallback callback(@NonNull String name, @NonNull String content) {
        // 首次处理（使用匹配的处理器）
        StringBuilder error = null;
        WechatPaymentCallback callback;
        WechatPaymentProcessor primary = this.getProcessor(name, null);
        try {
            if (Objects.nonNull(callback = primary.callback(content))) {
                return callback;
            }
        } catch (Exception e) {
            (error = new StringBuilder()).append(e);
        }

        // 兼容处理，防止在支付配置切换过程中导致支付回调失败的情况
        for (WechatPaymentProcessor processor : this.processors) {
            if (processor == primary || !Objects.equals(name, processor.getName())) {
                continue;
            }
            try {
                if (Objects.nonNull(callback = processor.callback(content))) {
                    return callback;
                }
            } catch (Exception e) {
                if (error == null) {
                    error = new StringBuilder();
                }
                if (error.length() > 0) {
                    error.append(" ");
                }
                error.append(e);
            }
        }
        log.error("Wechat payment callback process failed: {}\n{}", error, content);
        return null;
    }
}
